/* This file is part of GDB.

   Copyright 2002, 2007 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   --

   This file was generated by the program igen */

#ifndef ICACHE_C
#define ICACHE_C


#include "cpu.h"
#include "idecode.h"
#include "semantics.h"
#include "icache.h"
#ifdef HAVE_COMMON_FPU
#include "sim-inline.h"
#include "sim-fpu.h"
#endif
#include "support.h"


PSIM_INLINE_ICACHE\
(unsigned_word)
icache_illegal
(cpu *processor,
 instruction_word instruction,
 unsigned_word cia,
 idecode_cache *cache_entry)
{
  /* semantic routine */
#line 982 "../../sim/ppc/powerpc.igen"
  program_interrupt(processor, cia,
                    illegal_instruction_program_interrupt);
  error("Internal function must longjump\n");
  return 0;
#line 52 "icache.c"
}

PSIM_INLINE_ICACHE\
(unsigned_word)
icache_floating_point_unavailable
(cpu *processor,
 instruction_word instruction,
 unsigned_word cia,
 idecode_cache *cache_entry)
{
  /* semantic routine */
#line 990 "../../sim/ppc/powerpc.igen"
  floating_point_unavailable_interrupt(processor, cia);
  error("Internal function must longjump\n");
  return 0;
#line 68 "icache.c"
}


EXTERN_ICACHE\
(unsigned_word)
icache_instruction_call(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_instruction_call
#undef MY_PREFIX
#define MY_PREFIX instruction_call
#line 1493 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode instruction_call\n"));
#line 85 "icache.c"

  /* validate: 0.0,6./,11./,16./,21./,31.1 */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x1)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.0,6./,11./,16./,21./,31.1 #define get-values-from-icache|put-values-in-icache */
#line 94 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_instruction_call;

    /* semantic routine */
#line 1493 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics instruction_call\n"));
#line 104 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_instruction_call, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1493 "../../sim/ppc/powerpc.igen"
    {
      if (!os_emul_instruction_call(processor, cia, real_addr(cia, 1)))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
    }
#line 119 "icache.c"
    return nia;
    /* extraction: 0.0,6./,11./,16./,21./,31.1 #undef get-values-from-icache|put-values-in-icache */
#line 122 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Trap_Word_Immediate(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Trap_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Trap_Word_Immediate
#line 2897 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Trap Word Immediate\n"));
#line 141 "icache.c"

  /* validate: 0.3,6.TO,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.3,6.TO,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 2897 "../../sim/ppc/powerpc.igen"
    unsigned const TO ATTRIBUTE_UNUSED = cache_entry->crack.D.TO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 64 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_SI ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(int16_t)instruction); /* in icache - extraction */
#line 161 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Trap_Word_Immediate;

    /* semantic routine */
#line 2897 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Trap Word Immediate\n"));
#line 171 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Trap_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2897 "../../sim/ppc/powerpc.igen"
    {
      signed_word a = EXTENDED(*rA);
      signed_word b = EXTS(SI);
      if ((a < b && TO_0_)
          || (a > b && TO_1_)
          || (a == b && TO_2_)
          || ((unsigned_word)a < (unsigned_word)b && TO_3_)
          || ((unsigned_word)a > (unsigned_word)b && TO_4_)
          )
        program_interrupt(processor, cia,
                          trap_program_interrupt);
    }
#line 193 "icache.c"
    return nia;
    /* extraction: 0.3,6.TO,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 197 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Immediate(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Immediate
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Immediate
#line 2687 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Multiply Low Immediate\n"));
#line 216 "icache.c"

  /* validate: 0.7,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x1c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.7,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 64 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_SI ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(int16_t)instruction); /* in icache - extraction */
#line 240 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Immediate;

    /* semantic routine */
#line 2687 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Multiply Low Immediate\n"));
#line 250 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2687 "../../sim/ppc/powerpc.igen"
    {
      signed_word prod = *rA * EXTS(SI);
      *rT = prod;
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 0/*Rc*/);
    }
#line 265 "icache.c"
    return nia;
    /* extraction: 0.7,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 269 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Immediate_Carrying(cpu *processor,
                                        instruction_word instruction,
                                        unsigned_word cia,
                                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Immediate_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Immediate_Carrying
#line 2574 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From Immediate Carrying\n"));
#line 288 "icache.c"

  /* validate: 0.8,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x20000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.8,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 64 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_SI ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(int16_t)instruction); /* in icache - extraction */
#line 312 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Immediate_Carrying;

    /* semantic routine */
#line 2574 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From Immediate Carrying\n"));
#line 322 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Immediate_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2574 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(EXTS(SI));
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, 0/*OE*/, 0/*Rc*/);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 0/*Rc*/);
    }
#line 340 "icache.c"
    return nia;
    /* extraction: 0.8,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 344 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare_Logical_Immediate(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare_Logical_Immediate
#undef MY_PREFIX
#define MY_PREFIX Compare_Logical_Immediate
#line 2833 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Compare Logical Immediate\n"));
#line 363 "icache.c"

  /* validate: 0.10,6.BF,9./,10.L,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc400000) != 0x28000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.10,6.BF,9./,10.L,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 67 "../../sim/ppc/powerpc.igen"
    unsigned const BF ATTRIBUTE_UNUSED = cache_entry->crack.D.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "../../sim/ppc/powerpc.igen"
    uint32_t const BF_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2833 "../../sim/ppc/powerpc.igen"
    unsigned const L ATTRIBUTE_UNUSED = cache_entry->crack.D.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2833 "../../sim/ppc/powerpc.igen"
    unsigned const UI ATTRIBUTE_UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 387 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare_Logical_Immediate;

    /* semantic routine */
#line 2833 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Compare Logical Immediate\n"));
#line 397 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare_Logical_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2833 "../../sim/ppc/powerpc.igen"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        unsigned_word a;
        unsigned_word b = UI;
        if (L == 0)
          a = MASKED(*rA, 32, 63);
        else
          a = *rA;
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK, BF_BITMASK);
    }
#line 422 "icache.c"
    return nia;
    /* extraction: 0.10,6.BF,9./,10.L,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 426 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare_Immediate(cpu *processor,
                         instruction_word instruction,
                         unsigned_word cia,
                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare_Immediate
#undef MY_PREFIX
#define MY_PREFIX Compare_Immediate
#line 2791 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Compare Immediate\n"));
#line 445 "icache.c"

  /* validate: 0.11,6.BF,9./,10.L,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc400000) != 0x2c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.11,6.BF,9./,10.L,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 67 "../../sim/ppc/powerpc.igen"
    unsigned const BF ATTRIBUTE_UNUSED = cache_entry->crack.D.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "../../sim/ppc/powerpc.igen"
    uint32_t const BF_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2791 "../../sim/ppc/powerpc.igen"
    unsigned const L ATTRIBUTE_UNUSED = cache_entry->crack.D.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 64 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_SI ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(int16_t)instruction); /* in icache - extraction */
#line 469 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare_Immediate;

    /* semantic routine */
#line 2791 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Compare Immediate\n"));
#line 479 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2791 "../../sim/ppc/powerpc.igen"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        signed_word a;
        signed_word b = EXTS(SI);
        if (L == 0)
          a = EXTENDED(*rA);
        else
          a = *rA;
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK, BF_BITMASK);
    }
#line 504 "icache.c"
    return nia;
    /* extraction: 0.11,6.BF,9./,10.L,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 508 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Carrying(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Carrying
#line 2554 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add Immediate Carrying\n"));
#line 527 "icache.c"

  /* validate: 0.12,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x30000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.12,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 64 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_SI ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(int16_t)instruction); /* in icache - extraction */
#line 551 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Carrying;

    /* semantic routine */
#line 2554 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add Immediate Carrying\n"));
#line 561 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2554 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(EXTS(SI));
      ALU_END(*rT, 1/*CA*/, 0/*OE*/, 0/*Rc*/);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 0/*Rc*/);
    }
#line 577 "icache.c"
    return nia;
    /* extraction: 0.12,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 581 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Carrying_and_Record(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Carrying_and_Record
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Carrying_and_Record
#line 2564 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add Immediate Carrying and Record\n"));
#line 600 "icache.c"

  /* validate: 0.13,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x34000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.13,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 64 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_SI ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(int16_t)instruction); /* in icache - extraction */
#line 624 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Carrying_and_Record;

    /* semantic routine */
#line 2564 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add Immediate Carrying and Record\n"));
#line 634 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Carrying_and_Record, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2564 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(EXTS(SI));
      ALU_END(*rT, 1/*CA*/, 0/*OE*/, 1/*Rc*/);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, 1/*Rc*/);
    }
#line 650 "icache.c"
    return nia;
    /* extraction: 0.13,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 654 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_RA0(cpu *processor,
                         instruction_word instruction,
                         unsigned_word cia,
                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_RA0
#line 2512 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add Immediate\n"));
#line 673 "icache.c"

  /* validate: 0.14,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x38000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = 0; /* in icache - constant field */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 1 /* RA == 0 */ /* semantic compute - constant compare */
#line 64 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_SI ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(int16_t)instruction); /* in icache - extraction */
#line 697 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_RA0;

    /* semantic routine */
#line 2512 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add Immediate\n"));
#line 707 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2512 "../../sim/ppc/powerpc.igen"
    {
      if (RA_is_0)	*rT = EXTS(SI);
      else		*rT = *rA + EXTS(SI);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 723 "icache.c"
    return nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 727 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_RA(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_RA
#line 2512 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add Immediate\n"));
#line 746 "icache.c"

  /* validate: 0.14,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x38000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 0 /* RA == 0 */ /* semantic compute - constant compare */
#line 64 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_SI ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(int16_t)instruction); /* in icache - extraction */
#line 770 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_RA;

    /* semantic routine */
#line 2512 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add Immediate\n"));
#line 780 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2512 "../../sim/ppc/powerpc.igen"
    {
      if (RA_is_0)	*rT = EXTS(SI);
      else		*rT = *rA + EXTS(SI);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 796 "icache.c"
    return nia;
    /* extraction: 0.14,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 800 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Shifted_RA0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Shifted_RA0
#line 2522 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add Immediate Shifted\n"));
#line 819 "icache.c"

  /* validate: 0.15,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x3c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = 0; /* in icache - constant field */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 1 /* RA == 0 */ /* semantic compute - constant compare */
#line 64 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_SI ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(int16_t)instruction); /* in icache - extraction */
#line 843 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Shifted_RA0;

    /* semantic routine */
#line 2522 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add Immediate Shifted\n"));
#line 853 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2522 "../../sim/ppc/powerpc.igen"
    {
      if (RA_is_0)	*rT = EXTS(SI) << 16;
      else		*rT = *rA + (EXTS(SI) << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 869 "icache.c"
    return nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 873 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Immediate_Shifted_RA(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX Add_Immediate_Shifted_RA
#line 2522 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add Immediate Shifted\n"));
#line 892 "icache.c"

  /* validate: 0.15,6.RT,11.RA,16.SI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x3c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 0 /* RA == 0 */ /* semantic compute - constant compare */
#line 64 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_SI ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_SI = ((signed_word)(int16_t)instruction); /* in icache - extraction */
#line 916 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Immediate_Shifted_RA;

    /* semantic routine */
#line 2522 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add Immediate Shifted\n"));
#line 926 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2522 "../../sim/ppc/powerpc.igen"
    {
      if (RA_is_0)	*rT = EXTS(SI) << 16;
      else		*rT = *rA + (EXTS(SI) << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rT, (long)*rT));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 942 "icache.c"
    return nia;
    /* extraction: 0.15,6.RT,11.RA,16.SI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 946 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO0
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 965 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 985 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO0;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 995 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1044 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1048 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO1
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1067 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1087 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO1;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1097 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1146 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1150 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO2
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1169 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1189 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO2;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1199 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1248 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1252 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO3
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1271 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1291 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO3;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1301 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1350 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1354 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO4
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1373 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1393 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO4;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1403 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1452 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1456 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO5
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1475 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1495 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO5;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1505 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1554 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1558 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO6
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1577 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1597 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO6;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1607 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1656 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1660 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO7
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1679 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1699 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO7;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1709 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1758 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1762 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO8
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1781 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1801 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO8;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1811 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1860 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1864 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO9
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1883 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1903 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO9;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 1913 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 1962 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 1966 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO10
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 1985 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2005 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO10;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2015 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2064 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2068 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO11
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2087 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2107 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO11;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2117 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2166 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2170 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO12
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2189 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2209 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO12;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2219 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2268 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2272 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO13
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2291 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2311 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO13;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2321 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2370 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2374 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO14
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2393 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2413 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO14;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2423 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2472 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2476 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO15
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2495 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2515 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO15;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2525 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2574 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2578 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO16
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2597 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2617 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO16;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2627 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2676 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2680 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO17
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2699 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2719 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO17;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2729 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2778 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2782 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO18
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2801 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2821 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO18;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2831 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2880 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2884 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO19
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 2903 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2923 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO19;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 2933 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 2982 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 2986 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO20
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3005 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 3025 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO20;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3035 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3084 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3088 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO21
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3107 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 3127 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO21;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3137 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3186 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3190 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO22
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3209 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 3229 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO22;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3239 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3288 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3292 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO23
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3311 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 3331 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO23;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3341 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3390 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3394 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO24
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3413 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 3433 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO24;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3443 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3492 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3496 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO25
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3515 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 3535 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO25;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3545 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3594 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3598 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO26
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3617 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 3637 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO26;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3647 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3696 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3700 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO27
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3719 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 3739 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO27;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3749 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3798 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3802 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO28
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3821 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 3841 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO28;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3851 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 3900 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 3904 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO29
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 3923 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 3943 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO29;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 3953 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4002 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4006 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO30
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4025 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 4045 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO30;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4055 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4104 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4108 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK0_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK0_BO31
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4127 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 4147 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK0_BO31;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4157 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4206 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4210 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO0
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4229 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 4249 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO0;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4259 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4308 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4312 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO1
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4331 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 4351 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO1;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4361 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4410 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4414 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO2
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4433 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 4453 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO2;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4463 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4512 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4516 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO3
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4535 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 4555 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO3;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4565 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4614 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4618 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO4
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4637 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 4657 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO4;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4667 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4716 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4720 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO5
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4739 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 4759 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO5;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4769 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4818 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4822 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO6
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4841 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 4861 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO6;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4871 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 4920 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 4924 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO7
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 4943 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 4963 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO7;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 4973 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5022 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5026 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO8
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5045 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 5065 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO8;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5075 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5124 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5128 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO9
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5147 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 5167 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO9;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5177 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5226 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5230 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO10
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5249 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 5269 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO10;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5279 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5328 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5332 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO11
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5351 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 5371 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO11;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5381 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5430 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5434 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO12
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5453 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 5473 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO12;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5483 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5532 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5536 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO13
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5555 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 5575 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO13;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5585 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5634 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5638 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO14
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5657 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 5677 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO14;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5687 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5736 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5740 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO15
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5759 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 5779 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO15;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5789 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5838 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5842 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO16
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5861 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 5881 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO16;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5891 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 5940 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 5944 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO17
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 5963 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 5983 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO17;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 5993 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6042 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6046 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO18
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6065 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 6085 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO18;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6095 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6144 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6148 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO19
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6167 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 6187 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO19;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6197 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6246 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6250 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO20
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6269 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 6289 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO20;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6299 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6348 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6352 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO21
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6371 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 6391 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO21;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6401 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6450 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6454 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO22
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6473 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 6493 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO22;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6503 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6552 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6556 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO23
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6575 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 6595 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO23;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6605 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6654 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6658 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO24
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6677 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 6697 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO24;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6707 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6756 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6760 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO25
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6779 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 6799 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO25;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6809 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6858 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6862 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO26
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6881 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 6901 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO26;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 6911 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 6960 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 6964 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO27
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 6983 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 7003 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO27;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7013 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7062 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7066 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO28
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7085 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 7105 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO28;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7115 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7164 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7168 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO29
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7187 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 7207 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO29;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7217 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7266 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7270 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO30
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7289 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 7309 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO30;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7319 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7368 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7372 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA0_LK1_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA0_LK1_BO31
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7391 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 7411 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA0_LK1_BO31;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7421 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7470 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7474 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO0
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7493 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 7513 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO0;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7523 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7572 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7576 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO1
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7595 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 7615 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO1;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7625 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7674 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7678 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO2
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7697 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 7717 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO2;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7727 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7776 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7780 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO3
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7799 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 7819 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO3;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7829 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7878 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7882 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO4
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 7901 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 7921 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO4;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 7931 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 7980 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 7984 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO5
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8003 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 8023 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO5;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8033 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8082 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8086 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO6
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8105 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 8125 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO6;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8135 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8184 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8188 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO7
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8207 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 8227 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO7;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8237 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8286 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8290 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO8
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8309 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 8329 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO8;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8339 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8388 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8392 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO9
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8411 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 8431 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO9;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8441 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8490 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8494 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO10
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8513 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 8533 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO10;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8543 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8592 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8596 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO11
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8615 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 8635 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO11;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8645 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8694 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8698 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO12
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8717 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 8737 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO12;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8747 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8796 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8800 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO13
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8819 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 8839 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO13;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8849 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 8898 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 8902 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO14
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 8921 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 8941 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO14;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 8951 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9000 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9004 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO15
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9023 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 9043 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO15;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9053 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9102 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9106 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO16
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9125 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 9145 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO16;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9155 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9204 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9208 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO17
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9227 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 9247 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO17;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9257 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9306 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9310 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO18
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9329 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 9349 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO18;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9359 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9408 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9412 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO19
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9431 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 9451 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO19;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9461 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9510 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9514 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO20
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9533 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 9553 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO20;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9563 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9612 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9616 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO21
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9635 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 9655 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO21;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9665 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9714 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9718 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO22
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9737 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 9757 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO22;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9767 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9816 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9820 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO23
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9839 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 9859 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO23;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9869 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 9918 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 9922 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO24
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 9941 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 9961 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO24;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 9971 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10020 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10024 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO25
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10043 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 10063 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO25;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10073 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10122 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10126 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO26
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10145 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 10165 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO26;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10175 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10224 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10228 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO27
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10247 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 10267 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO27;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10277 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10326 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10330 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO28
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10349 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 10369 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO28;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10379 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10428 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10432 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO29
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10451 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 10471 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO29;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10481 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10530 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10534 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO30
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10553 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 10573 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO30;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10583 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10632 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10636 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK0_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK0_BO31
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10655 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 10675 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK0_BO31;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10685 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10734 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10738 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO0
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10757 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 10777 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO0;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10787 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10836 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10840 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO1
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10859 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 10879 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO1;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10889 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 10938 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 10942 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO2(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO2
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 10961 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 2; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 10981 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO2;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 10991 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11040 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11044 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO3(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO3
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11063 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 3; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 11083 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO3;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11093 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11142 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11146 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO4(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO4
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11165 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 4; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 11185 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO4;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11195 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11244 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11248 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO5(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO5
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11267 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 5; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 11287 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO5;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11297 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11346 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11350 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO6(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO6
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11369 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 6; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 11389 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO6;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11399 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11448 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11452 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO7(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO7
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11471 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 7; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 11491 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO7;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11501 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11550 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11554 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO8(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO8
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11573 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 8; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 11593 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO8;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11603 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11652 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11656 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO9(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO9
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11675 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 9; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 11695 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO9;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11705 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11754 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11758 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO10(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO10
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11777 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 10; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 11797 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO10;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11807 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11856 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11860 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO11(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO11
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11879 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 11; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 11899 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO11;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 11909 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 11958 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 11962 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO12(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO12
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 11981 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 12; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 12001 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO12;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12011 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12060 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12064 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO13(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO13
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12083 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 13; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 12103 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO13;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12113 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12162 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12166 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO14(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO14
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12185 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 14; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 12205 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO14;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12215 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12264 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12268 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO15(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO15
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12287 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 15; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 12307 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO15;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12317 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12366 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12370 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO16(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO16
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12389 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 16; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 12409 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO16;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12419 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12468 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12472 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO17(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO17
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12491 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 17; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 12511 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO17;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12521 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12570 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12574 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO18(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO18
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12593 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 18; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 12613 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO18;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12623 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12672 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12676 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO19(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO19
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12695 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 19; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 12715 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO19;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12725 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12774 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12778 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO20(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO20
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12797 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 20; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 12817 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO20;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12827 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12876 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12880 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO21(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO21
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 12899 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 21; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 12919 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO21;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 12929 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 12978 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 12982 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO22(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO22
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13001 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 22; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 13021 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO22;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13031 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13080 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13084 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO23(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO23
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13103 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 23; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 13123 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO23;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13133 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13182 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13186 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO24(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO24
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13205 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 24; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 13225 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO24;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13235 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13284 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13288 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO25(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO25
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13307 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 25; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 13327 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO25;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13337 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13386 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13390 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO26(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO26
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13409 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 26; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 13429 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO26;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13439 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13488 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13492 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO27(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO27
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13511 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 27; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 13531 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO27;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13541 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13590 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13594 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO28(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO28
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13613 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 28; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 13633 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO28;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13643 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13692 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13696 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO29(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO29
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13715 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 29; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 13735 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO29;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13745 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13794 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13798 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO30(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO30
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13817 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 30; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 13837 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO30;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13847 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13896 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 13900 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_AA1_LK1_BO31(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_AA1_LK1_BO31
#line 1518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional\n"));
#line 13919 "icache.c"

  /* validate: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x40000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = 31; /* default in icache - constant field */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.B.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 77 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_BD_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.B.EXTS_BD_0b00 = (((signed_word)(int16_t)instruction) & ~3); /* in icache - extraction */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1518 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 13939 "icache.c"

#define insn cache_entry->crack.B.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_AA1_LK1_BO31;

    /* semantic routine */
#line 1518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional\n"));
#line 13949 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1518 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != (BO_3_));
      cond_ok = BO_0_ || ((CR_BI_) == (BO_1_));
      if (ctr_ok && cond_ok) {
        if (AA) NIA = IEA(EXTS(BD_0b00));
        else    NIA = IEA(CIA + EXTS(BD_0b00));
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as "normal".
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), succeed, BO);
      if (! BO_0_) {
        int reverse;
        if (BO_4_) {	/* branch prediction bit set, reverse sense of test */
          reverse = EXTS(BD_0b00) < 0;
        } else {	/* branch prediction bit not set */
          reverse = EXTS(BD_0b00) >= 0;
        }
        if (CURRENT_MODEL_ISSUE > 0)
          model_branch_predict(cpu_model(processor), reverse ? !succeed : succeed);
      }
    }
#line 13998 "icache.c"
    return nia;
    /* extraction: 0.16,6.BO,11.BI,16.BD,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 14002 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_System_Call(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_System_Call
#undef MY_PREFIX
#define MY_PREFIX System_Call
#line 1630 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode System Call\n"));
#line 14021 "icache.c"

  /* validate: 0.17,6./,11./,16./,30.1,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x44000002)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.17,6./,11./,16./,30.1,31./ #define get-values-from-icache|put-values-in-icache */
#line 14030 "icache.c"

#define insn cache_entry->crack.SC.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_System_Call;

    /* semantic routine */
#line 1630 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics System Call\n"));
#line 14040 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_System_Call, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1630 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL_ISSUE > 0)
        model_serialize(MY_INDEX, cpu_model(processor));
      system_call_interrupt(processor, cia);
    }
#line 14055 "icache.c"
    return nia;
    /* extraction: 0.17,6./,11./,16./,30.1,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14058 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA0_LK0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA0_LK0
#line 1505 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14077 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 78 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_LI_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(int32_t)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1505 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1505 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 14092 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA0_LK0;

    /* semantic routine */
#line 1505 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14102 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1505 "../../sim/ppc/powerpc.igen"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14121 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14124 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA0_LK1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA0_LK1
#line 1505 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14143 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 78 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_LI_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(int32_t)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1505 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 1505 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 14158 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA0_LK1;

    /* semantic routine */
#line 1505 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14168 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1505 "../../sim/ppc/powerpc.igen"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14187 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14190 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA1_LK0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA1_LK0
#line 1505 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14209 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 78 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_LI_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(int32_t)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1505 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1505 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 14224 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA1_LK0;

    /* semantic routine */
#line 1505 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14234 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1505 "../../sim/ppc/powerpc.igen"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14253 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14256 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_AA1_LK1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch
#undef MY_PREFIX
#define MY_PREFIX Branch_AA1_LK1
#line 1505 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch\n"));
#line 14275 "icache.c"

  /* validate: 0.18,6.LI,30.AA,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x48000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #define get-values-from-icache|put-values-in-icache */
#line 78 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_LI_0b00 ATTRIBUTE_UNUSED = cache_entry->crack.I.EXTS_LI_0b00 = ((((signed_word)(int32_t)(instruction << 6)) >> 6) & ~0x3); /* in icache - extraction */
#line 1505 "../../sim/ppc/powerpc.igen"
    unsigned const AA ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 1505 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 14290 "icache.c"

#define insn cache_entry->crack.I.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_AA1_LK1;

    /* semantic routine */
#line 1505 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch\n"));
#line 14300 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1505 "../../sim/ppc/powerpc.igen"
    {
      /* option_mpc860c0:
      No problem here because this branch is predicted taken (unconditional). */
      if (AA) NIA = IEA(EXTS(LI_0b00));
      else    NIA = IEA(CIA + EXTS(LI_0b00));
      if (LK) LR = (spreg)CIA+4;
      if (CURRENT_MODEL_ISSUE > 0)
        model_branches(cpu_model(processor), 1, -1);
    }
#line 14319 "icache.c"
    return nia;
    /* extraction: 0.18,6.LI,30.AA,31.LK #undef get-values-from-icache|put-values-in-icache */
#line 14322 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_Condition_Register_Field(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_Condition_Register_Field
#undef MY_PREFIX
#define MY_PREFIX Move_Condition_Register_Field
#line 1709 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move Condition Register Field\n"));
#line 14341 "icache.c"

  /* validate: 0.19,6.BF,9./,11.BFA,14./,16./,21.0,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc63ffff) != 0x4c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BF,9./,11.BFA,14./,16./,21.0,31./ #define get-values-from-icache|put-values-in-icache */
#line 67 "../../sim/ppc/powerpc.igen"
    unsigned const BF ATTRIBUTE_UNUSED = cache_entry->crack.XL.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "../../sim/ppc/powerpc.igen"
    uint32_t const BF_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 1709 "../../sim/ppc/powerpc.igen"
    unsigned const BFA ATTRIBUTE_UNUSED = cache_entry->crack.XL.BFA = EXTRACTED32(instruction, 11, 13); /* default in icache - extraction */
#line 14356 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_Condition_Register_Field;

    /* semantic routine */
#line 1709 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move Condition Register Field\n"));
#line 14366 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_Condition_Register_Field, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1709 "../../sim/ppc/powerpc.igen"
    {
      MBLIT32(CR, 4*BF, 4*BF+3, EXTRACTED32(CR, 4*BFA, 4*BFA+3));
      PPC_INSN_CR(BF_BITMASK, 1 << BFA);
    }
#line 14380 "icache.c"
    return nia;
    /* extraction: 0.19,6.BF,9./,11.BFA,14./,16./,21.0,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14383 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Link_Register_LK0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Link_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Link_Register_LK0
#line 1561 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Link Register\n"));
#line 14402 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.16,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000020)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1561 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1561 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 14418 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Link_Register_LK0;

    /* semantic routine */
#line 1561 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Link Register\n"));
#line 14428 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Link_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1561 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != BO_3_);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (ctr_ok && cond_ok) {
        NIA = IEA(LR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 14469 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 14473 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Link_Register_LK1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Link_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Link_Register_LK1
#line 1561 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Link Register\n"));
#line 14492 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.16,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000020)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1561 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1561 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 14508 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Link_Register_LK1;

    /* semantic routine */
#line 1561 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Link Register\n"));
#line 14518 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Link_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1561 "../../sim/ppc/powerpc.igen"
    {
      int M, ctr_ok, cond_ok, succeed;
      if (is_64bit_implementation && is_64bit_mode) M = 0;
      else                                          M = 32;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      if (!BO_2_) CTR = CTR - 1;
      ctr_ok = BO_2_ || ((MASKED(CTR, M, 63) != 0) != BO_3_);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (ctr_ok && cond_ok) {
        NIA = IEA(LR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 14559 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.16,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 14563 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_NOR(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_NOR
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_NOR
#line 1674 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Condition Register NOR\n"));
#line 14582 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.33,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000042)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.33,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "../../sim/ppc/powerpc.igen"
    unsigned const BT ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "../../sim/ppc/powerpc.igen"
    uint32_t const BT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BA ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "../../sim/ppc/powerpc.igen"
    uint32_t const BA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BB ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "../../sim/ppc/powerpc.igen"
    uint32_t const BB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14605 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_NOR;

    /* semantic routine */
#line 1674 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Condition Register NOR\n"));
#line 14615 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_NOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1674 "../../sim/ppc/powerpc.igen"
    {
      BLIT32(CR, BT, !(CR_BA_ || CR_BB_));
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14629 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.33,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14634 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Return_From_Interrupt(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Return_From_Interrupt
#undef MY_PREFIX
#define MY_PREFIX Return_From_Interrupt
#line 4861 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Return From Interrupt\n"));
#line 14653 "icache.c"

  /* validate: 0.19,6./,11./,16./,21.50,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x4c000064)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6./,11./,16./,21.50,31./ #define get-values-from-icache|put-values-in-icache */
#line 14662 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Return_From_Interrupt;

    /* semantic routine */
#line 4861 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Return From Interrupt\n"));
#line 14672 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Return_From_Interrupt, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4861 "../../sim/ppc/powerpc.igen"
    {
      if (IS_PROBLEM_STATE(processor)) {
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      }
      else {
        MSR = (MASKED(SRR1, 0, 32)
               | MASKED(SRR1, 37, 41)
               | MASKED(SRR1, 48, 63));
        NIA = MASKED(SRR0, 0, 61);
        cpu_synchronize_context(processor, cia);
        check_masked_interrupts(processor);
      }
    }
#line 14696 "icache.c"
    return nia;
    /* extraction: 0.19,6./,11./,16./,21.50,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14699 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_AND_with_Complement(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_AND_with_Complement
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_AND_with_Complement
#line 1690 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Condition Register AND with Complement\n"));
#line 14718 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.129,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000102)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.129,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "../../sim/ppc/powerpc.igen"
    unsigned const BT ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "../../sim/ppc/powerpc.igen"
    uint32_t const BT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BA ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "../../sim/ppc/powerpc.igen"
    uint32_t const BA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BB ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "../../sim/ppc/powerpc.igen"
    uint32_t const BB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14741 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_AND_with_Complement;

    /* semantic routine */
#line 1690 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Condition Register AND with Complement\n"));
#line 14751 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_AND_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1690 "../../sim/ppc/powerpc.igen"
    {
      BLIT32(CR, BT, CR_BA_ && !CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14765 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.129,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14770 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Instruction_Synchronize(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Instruction_Synchronize
#undef MY_PREFIX
#define MY_PREFIX Instruction_Synchronize
#line 4774 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Instruction Synchronize\n"));
#line 14789 "icache.c"

  /* validate: 0.19,6./,11./,16./,21.150,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x4c00012c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6./,11./,16./,21.150,31./ #define get-values-from-icache|put-values-in-icache */
#line 14798 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Instruction_Synchronize;

    /* semantic routine */
#line 4774 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Instruction Synchronize\n"));
#line 14808 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Instruction_Synchronize, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4774 "../../sim/ppc/powerpc.igen"
    {
      cpu_synchronize_context(processor, cia);
      PPC_INSN_INT(0, 0, 0);
    }
#line 14822 "icache.c"
    return nia;
    /* extraction: 0.19,6./,11./,16./,21.150,31./ #undef get-values-from-icache|put-values-in-icache */
#line 14825 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_XOR(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_XOR
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_XOR
#line 1658 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Condition Register XOR\n"));
#line 14844 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.193,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000182)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.193,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "../../sim/ppc/powerpc.igen"
    unsigned const BT ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "../../sim/ppc/powerpc.igen"
    uint32_t const BT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BA ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "../../sim/ppc/powerpc.igen"
    uint32_t const BA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BB ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "../../sim/ppc/powerpc.igen"
    uint32_t const BB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14867 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_XOR;

    /* semantic routine */
#line 1658 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Condition Register XOR\n"));
#line 14877 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_XOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1658 "../../sim/ppc/powerpc.igen"
    {
      BLIT32(CR, BT, CR_BA_ != CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14891 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.193,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14896 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_NAND(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_NAND
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_NAND
#line 1666 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Condition Register NAND\n"));
#line 14915 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.225,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c0001c2)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.225,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "../../sim/ppc/powerpc.igen"
    unsigned const BT ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "../../sim/ppc/powerpc.igen"
    uint32_t const BT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BA ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "../../sim/ppc/powerpc.igen"
    uint32_t const BA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BB ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "../../sim/ppc/powerpc.igen"
    uint32_t const BB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 14938 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_NAND;

    /* semantic routine */
#line 1666 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Condition Register NAND\n"));
#line 14948 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_NAND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1666 "../../sim/ppc/powerpc.igen"
    {
      BLIT32(CR, BT, !(CR_BA_ && CR_BB_));
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 14962 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.225,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 14967 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_AND(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_AND
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_AND
#line 1642 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Condition Register AND\n"));
#line 14986 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.257,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000202)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.257,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "../../sim/ppc/powerpc.igen"
    unsigned const BT ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "../../sim/ppc/powerpc.igen"
    uint32_t const BT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BA ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "../../sim/ppc/powerpc.igen"
    uint32_t const BA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BB ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "../../sim/ppc/powerpc.igen"
    uint32_t const BB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15009 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_AND;

    /* semantic routine */
#line 1642 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Condition Register AND\n"));
#line 15019 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_AND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1642 "../../sim/ppc/powerpc.igen"
    {
      BLIT32(CR, BT, CR_BA_ && CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15033 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.257,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15038 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_Equivalent(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_Equivalent
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_Equivalent
#line 1682 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Condition Register Equivalent\n"));
#line 15057 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.289,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000242)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.289,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "../../sim/ppc/powerpc.igen"
    unsigned const BT ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "../../sim/ppc/powerpc.igen"
    uint32_t const BT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BA ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "../../sim/ppc/powerpc.igen"
    uint32_t const BA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BB ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "../../sim/ppc/powerpc.igen"
    uint32_t const BB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15080 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_Equivalent;

    /* semantic routine */
#line 1682 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Condition Register Equivalent\n"));
#line 15090 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_Equivalent, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1682 "../../sim/ppc/powerpc.igen"
    {
      BLIT32(CR, BT, CR_BA_ == CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15104 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.289,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15109 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_OR_with_Complement(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_OR_with_Complement
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_OR_with_Complement
#line 1698 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Condition Register OR with Complement\n"));
#line 15128 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.417,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000342)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.417,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "../../sim/ppc/powerpc.igen"
    unsigned const BT ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "../../sim/ppc/powerpc.igen"
    uint32_t const BT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BA ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "../../sim/ppc/powerpc.igen"
    uint32_t const BA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BB ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "../../sim/ppc/powerpc.igen"
    uint32_t const BB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15151 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_OR_with_Complement;

    /* semantic routine */
#line 1698 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Condition Register OR with Complement\n"));
#line 15161 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_OR_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1698 "../../sim/ppc/powerpc.igen"
    {
      BLIT32(CR, BT, CR_BA_ || !CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15175 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.417,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15180 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Condition_Register_OR(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Condition_Register_OR
#undef MY_PREFIX
#define MY_PREFIX Condition_Register_OR
#line 1650 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Condition Register OR\n"));
#line 15199 "icache.c"

  /* validate: 0.19,6.BT,11.BA,16.BB,21.449,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x4c000382)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.449,31./ #define get-values-from-icache|put-values-in-icache */
#line 75 "../../sim/ppc/powerpc.igen"
    unsigned const BT ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "../../sim/ppc/powerpc.igen"
    uint32_t const BT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BT_BITMASK = (1 << BT); /* in icache - extraction */
#define BA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 70 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BA ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BA = BIT32(BA); /* in icache - extraction */
#line 71 "../../sim/ppc/powerpc.igen"
    uint32_t const BA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BA_BITMASK = (1 << BA); /* in icache - extraction */
#define BB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 73 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BB ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BB = BIT32(BB); /* in icache - extraction */
#line 74 "../../sim/ppc/powerpc.igen"
    uint32_t const BB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XL.BB_BITMASK = (1 << BB); /* in icache - extraction */
#line 15222 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Condition_Register_OR;

    /* semantic routine */
#line 1650 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Condition Register OR\n"));
#line 15232 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Condition_Register_OR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1650 "../../sim/ppc/powerpc.igen"
    {
      BLIT32(CR, BT, CR_BA_ || CR_BB_);
      PPC_INSN_CR(BT_BITMASK, BA_BITMASK | BB_BITMASK);
    }
#line 15246 "icache.c"
    return nia;
    /* extraction: 0.19,6.BT,11.BA,16.BB,21.449,31./ #undef get-values-from-icache|put-values-in-icache */
#undef BA
#undef BB
#line 15251 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Count_Register_LK0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Count_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Count_Register_LK0
#line 1596 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Count Register\n"));
#line 15270 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.528,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000420)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1596 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1596 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 15286 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Count_Register_LK0;

    /* semantic routine */
#line 1596 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Count Register\n"));
#line 15296 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Count_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1596 "../../sim/ppc/powerpc.igen"
    {
      int cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (cond_ok) {
        NIA = IEA(CTR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 15333 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 15337 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Branch_Conditional_to_Count_Register_LK1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Branch_Conditional_to_Count_Register
#undef MY_PREFIX
#define MY_PREFIX Branch_Conditional_to_Count_Register_LK1
#line 1596 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Branch Conditional to Count Register\n"));
#line 15356 "icache.c"

  /* validate: 0.19,6.BO,11.BI,16./,21.528,31.LK */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x4c000420)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #define get-values-from-icache|put-values-in-icache */
#line 1596 "../../sim/ppc/powerpc.igen"
    unsigned const BO ATTRIBUTE_UNUSED = cache_entry->crack.XL.BO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#define BI EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 66 "../../sim/ppc/powerpc.igen"
    unsigned const BIT32_BI ATTRIBUTE_UNUSED = cache_entry->crack.XL.BIT32_BI = BIT32(BI); /* in icache - extraction */
#line 1596 "../../sim/ppc/powerpc.igen"
    unsigned const LK ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 15372 "icache.c"

#define insn cache_entry->crack.XL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Branch_Conditional_to_Count_Register_LK1;

    /* semantic routine */
#line 1596 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Branch Conditional to Count Register\n"));
#line 15382 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Branch_Conditional_to_Count_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1596 "../../sim/ppc/powerpc.igen"
    {
      int cond_ok, succeed;
      if (CURRENT_MODEL_ISSUE > 0 && ! BO_0_)
        model_wait_for_cr(cpu_model(processor), BIT32_BI);
      cond_ok = BO_0_ || (CR_BI_ == BO_1_);
      if (cond_ok) {
        NIA = IEA(CTR_0b00);
        succeed = 1;
      }
      else
        succeed = 0;
      if (LK) LR = (spreg)IEA(CIA + 4);
      if (option_mpc860c0 && (!BO_0_ || !BO_2_) && !BO_4_) {
        /* This branch is predicted as not-taken.
        If this is a forward branch and it is near the end of a page,
        we've detected a problematic branch. */
        if (succeed && NIA > CIA) {
          if (MPC860C0_PAGE_SIZE - (CIA & (MPC860C0_PAGE_SIZE-1)) <= option_mpc860c0)
            program_interrupt(processor, cia, mpc860c0_instruction_program_interrupt);
        }
      }
      if (CURRENT_MODEL_ISSUE > 0) {
        model_branches(cpu_model(processor), succeed, BO);
        if (! BO_0_)
          model_branch_predict(cpu_model(processor), BO_4_ ? !succeed : succeed);
      }
    }
#line 15419 "icache.c"
    return nia;
    /* extraction: 0.19,6.BO,11.BI,16./,21.528,31.LK #undef get-values-from-icache|put-values-in-icache */
#undef BI
#line 15423 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc0(cpu *processor,
                                                       instruction_word instruction,
                                                       unsigned_word cia,
                                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_Mask_Insert
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_Mask_Insert_Rc0
#line 3231 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then Mask Insert\n"));
#line 15442 "icache.c"

  /* validate: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x50000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3231 "../../sim/ppc/powerpc.igen"
    unsigned const SH ATTRIBUTE_UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3231 "../../sim/ppc/powerpc.igen"
    unsigned const MB ATTRIBUTE_UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3231 "../../sim/ppc/powerpc.igen"
    unsigned const ME ATTRIBUTE_UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3231 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 15472 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc0;

    /* semantic routine */
#line 3231 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then Mask Insert\n"));
#line 15482 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_Mask_Insert, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3231 "../../sim/ppc/powerpc.igen"
    {
      long n = SH;
      uint32_t r = ROTL32(*rS, n);
      uint32_t m = MASK(MB+32, ME+32);
      signed_word result = (r & m) | (*rA & ~m);
      *rA = result;
      ITRACE(trace_alu, (": n=%ld *rS=0x%lx r=0x%lx m=0x%lx result=0x%lx\n",
                         n, (unsigned long)*rS, (unsigned long)r, (unsigned long)m,
      		   (unsigned long)result));
      CR0_COMPARE(result, 0, Rc);
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15504 "icache.c"
    return nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15508 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc1(cpu *processor,
                                                       instruction_word instruction,
                                                       unsigned_word cia,
                                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_Mask_Insert
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_Mask_Insert_Rc1
#line 3231 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then Mask Insert\n"));
#line 15527 "icache.c"

  /* validate: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x50000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3231 "../../sim/ppc/powerpc.igen"
    unsigned const SH ATTRIBUTE_UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3231 "../../sim/ppc/powerpc.igen"
    unsigned const MB ATTRIBUTE_UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3231 "../../sim/ppc/powerpc.igen"
    unsigned const ME ATTRIBUTE_UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3231 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 15557 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_Mask_Insert_Rc1;

    /* semantic routine */
#line 3231 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then Mask Insert\n"));
#line 15567 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_Mask_Insert, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3231 "../../sim/ppc/powerpc.igen"
    {
      long n = SH;
      uint32_t r = ROTL32(*rS, n);
      uint32_t m = MASK(MB+32, ME+32);
      signed_word result = (r & m) | (*rA & ~m);
      *rA = result;
      ITRACE(trace_alu, (": n=%ld *rS=0x%lx r=0x%lx m=0x%lx result=0x%lx\n",
                         n, (unsigned long)*rS, (unsigned long)r, (unsigned long)m,
      		   (unsigned long)result));
      CR0_COMPARE(result, 0, Rc);
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15589 "icache.c"
    return nia;
    /* extraction: 0.20,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15593 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc0(cpu *processor,
                                                         instruction_word instruction,
                                                         unsigned_word cia,
                                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc0
#line 3178 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then AND with Mask\n"));
#line 15612 "icache.c"

  /* validate: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x54000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3178 "../../sim/ppc/powerpc.igen"
    unsigned const SH ATTRIBUTE_UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3178 "../../sim/ppc/powerpc.igen"
    unsigned const MB ATTRIBUTE_UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3178 "../../sim/ppc/powerpc.igen"
    unsigned const ME ATTRIBUTE_UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3178 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 15642 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc0;

    /* semantic routine */
#line 3178 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then AND with Mask\n"));
#line 15652 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3178 "../../sim/ppc/powerpc.igen"
    {
      long n = SH;
      uint32_t s = *rS;
      uint32_t r = ROTL32(s, n);
      uint32_t m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
      ITRACE(trace_alu,
             ("n=%ld, s=0x%lx, r=0x%lx, m=0x%lx, result=0x%lx, cr=0x%lx\n",
              n, (unsigned long)s, (unsigned long)r, (unsigned long)m,
      	(unsigned long)result, (unsigned long)CR));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15676 "icache.c"
    return nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15680 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc1(cpu *processor,
                                                         instruction_word instruction,
                                                         unsigned_word cia,
                                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_Immediate_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc1
#line 3178 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Rotate Left Word Immediate then AND with Mask\n"));
#line 15699 "icache.c"

  /* validate: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x54000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3178 "../../sim/ppc/powerpc.igen"
    unsigned const SH ATTRIBUTE_UNUSED = cache_entry->crack.M.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3178 "../../sim/ppc/powerpc.igen"
    unsigned const MB ATTRIBUTE_UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3178 "../../sim/ppc/powerpc.igen"
    unsigned const ME ATTRIBUTE_UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3178 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 15729 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_Immediate_then_AND_with_Mask_Rc1;

    /* semantic routine */
#line 3178 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Rotate Left Word Immediate then AND with Mask\n"));
#line 15739 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_Immediate_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3178 "../../sim/ppc/powerpc.igen"
    {
      long n = SH;
      uint32_t s = *rS;
      uint32_t r = ROTL32(s, n);
      uint32_t m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
      ITRACE(trace_alu,
             ("n=%ld, s=0x%lx, r=0x%lx, m=0x%lx, result=0x%lx, cr=0x%lx\n",
              n, (unsigned long)s, (unsigned long)r, (unsigned long)m,
      	(unsigned long)result, (unsigned long)CR));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 15763 "icache.c"
    return nia;
    /* extraction: 0.21,6.RS,11.RA,16.SH,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15767 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_then_AND_with_Mask_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_then_AND_with_Mask_Rc0
#line 3210 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Rotate Left Word then AND with Mask\n"));
#line 15786 "icache.c"

  /* validate: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x5c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.M.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.M.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.M.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3210 "../../sim/ppc/powerpc.igen"
    unsigned const MB ATTRIBUTE_UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3210 "../../sim/ppc/powerpc.igen"
    unsigned const ME ATTRIBUTE_UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3210 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 15820 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_then_AND_with_Mask_Rc0;

    /* semantic routine */
#line 3210 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Rotate Left Word then AND with Mask\n"));
#line 15830 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3210 "../../sim/ppc/powerpc.igen"
    {
      long n = MASKED(*rB, 59, 63);
      uint32_t r = ROTL32(*rS, n);
      uint32_t m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
    }
#line 15848 "icache.c"
    return nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15852 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Rotate_Left_Word_then_AND_with_Mask_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Rotate_Left_Word_then_AND_with_Mask
#undef MY_PREFIX
#define MY_PREFIX Rotate_Left_Word_then_AND_with_Mask_Rc1
#line 3210 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Rotate Left Word then AND with Mask\n"));
#line 15871 "icache.c"

  /* validate: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x5c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.M.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.M.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.M.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.M.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.M.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.M.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.M.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.M.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.M.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3210 "../../sim/ppc/powerpc.igen"
    unsigned const MB ATTRIBUTE_UNUSED = cache_entry->crack.M.MB = EXTRACTED32(instruction, 21, 25); /* default in icache - extraction */
#line 3210 "../../sim/ppc/powerpc.igen"
    unsigned const ME ATTRIBUTE_UNUSED = cache_entry->crack.M.ME = EXTRACTED32(instruction, 26, 30); /* default in icache - extraction */
#line 3210 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 15905 "icache.c"

#define insn cache_entry->crack.M.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Rotate_Left_Word_then_AND_with_Mask_Rc1;

    /* semantic routine */
#line 3210 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Rotate Left Word then AND with Mask\n"));
#line 15915 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Rotate_Left_Word_then_AND_with_Mask, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3210 "../../sim/ppc/powerpc.igen"
    {
      long n = MASKED(*rB, 59, 63);
      uint32_t r = ROTL32(*rS, n);
      uint32_t m = MASK(MB+32, ME+32);
      signed_word result = r & m;
      *rA = result;
      CR0_COMPARE(result, 0, Rc);
    }
#line 15933 "icache.c"
    return nia;
    /* extraction: 0.23,6.RS,11.RA,16.RB,21.MB,26.ME,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 15937 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Immediate(cpu *processor,
                    instruction_word instruction,
                    unsigned_word cia,
                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_Immediate
#undef MY_PREFIX
#define MY_PREFIX OR_Immediate
#line 2974 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode OR Immediate\n"));
#line 15956 "icache.c"

  /* validate: 0.24,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x60000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.24,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2974 "../../sim/ppc/powerpc.igen"
    unsigned const UI ATTRIBUTE_UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 15980 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Immediate;

    /* semantic routine */
#line 2974 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics OR Immediate\n"));
#line 15990 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2974 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS | UI;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16005 "icache.c"
    return nia;
    /* extraction: 0.24,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16009 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Immediate_Shifted(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX OR_Immediate_Shifted
#line 2983 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode OR Immediate Shifted\n"));
#line 16028 "icache.c"

  /* validate: 0.25,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x64000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.25,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2983 "../../sim/ppc/powerpc.igen"
    unsigned const UI ATTRIBUTE_UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16052 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Immediate_Shifted;

    /* semantic routine */
#line 2983 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics OR Immediate Shifted\n"));
#line 16062 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2983 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS | (UI << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16077 "icache.c"
    return nia;
    /* extraction: 0.25,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16081 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Immediate(cpu *processor,
                     instruction_word instruction,
                     unsigned_word cia,
                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR_Immediate
#undef MY_PREFIX
#define MY_PREFIX XOR_Immediate
#line 2992 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode XOR Immediate\n"));
#line 16100 "icache.c"

  /* validate: 0.26,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x68000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.26,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2992 "../../sim/ppc/powerpc.igen"
    unsigned const UI ATTRIBUTE_UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16124 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Immediate;

    /* semantic routine */
#line 2992 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics XOR Immediate\n"));
#line 16134 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2992 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS ^ UI;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16149 "icache.c"
    return nia;
    /* extraction: 0.26,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16153 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Immediate_Shifted(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX XOR_Immediate_Shifted
#line 3001 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode XOR Immediate Shifted\n"));
#line 16172 "icache.c"

  /* validate: 0.27,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x6c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.27,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3001 "../../sim/ppc/powerpc.igen"
    unsigned const UI ATTRIBUTE_UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16196 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Immediate_Shifted;

    /* semantic routine */
#line 3001 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics XOR Immediate Shifted\n"));
#line 16206 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3001 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS ^ (UI << 16);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 0/*Rc*/);
    }
#line 16221 "icache.c"
    return nia;
    /* extraction: 0.27,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16225 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Immediate(cpu *processor,
                     instruction_word instruction,
                     unsigned_word cia,
                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_Immediate
#undef MY_PREFIX
#define MY_PREFIX AND_Immediate
#line 2954 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode AND Immediate\n"));
#line 16244 "icache.c"

  /* validate: 0.28,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x70000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.28,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2954 "../../sim/ppc/powerpc.igen"
    unsigned const UI ATTRIBUTE_UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16268 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Immediate;

    /* semantic routine */
#line 2954 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics AND Immediate\n"));
#line 16278 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2954 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS & UI;
      CR0_COMPARE(*rA, 0, 1/*Rc*/);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 1/*Rc*/);
    }
#line 16294 "icache.c"
    return nia;
    /* extraction: 0.28,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16298 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Immediate_Shifted(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_Immediate_Shifted
#undef MY_PREFIX
#define MY_PREFIX AND_Immediate_Shifted
#line 2964 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode AND Immediate Shifted\n"));
#line 16317 "icache.c"

  /* validate: 0.29,6.RS,11.RA,16.UI */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x74000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.29,6.RS,11.RA,16.UI #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2964 "../../sim/ppc/powerpc.igen"
    unsigned const UI ATTRIBUTE_UNUSED = cache_entry->crack.D.UI = EXTRACTED32(instruction, 16, 31); /* default in icache - extraction */
#line 16341 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Immediate_Shifted;

    /* semantic routine */
#line 2964 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics AND Immediate Shifted\n"));
#line 16351 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_Immediate_Shifted, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2964 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS & (UI << 16);
      CR0_COMPARE(*rA, 0, 1/*Rc*/);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, 1/*Rc*/);
    }
#line 16367 "icache.c"
    return nia;
    /* extraction: 0.29,6.RS,11.RA,16.UI #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16371 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare
#undef MY_PREFIX
#define MY_PREFIX Compare
#line 2810 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Compare\n"));
#line 16390 "icache.c"

  /* validate: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.0,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc4007ff) != 0x7c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.0,31./ #define get-values-from-icache|put-values-in-icache */
#line 67 "../../sim/ppc/powerpc.igen"
    unsigned const BF ATTRIBUTE_UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "../../sim/ppc/powerpc.igen"
    uint32_t const BF_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2810 "../../sim/ppc/powerpc.igen"
    unsigned const L ATTRIBUTE_UNUSED = cache_entry->crack.X.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 16418 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare;

    /* semantic routine */
#line 2810 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Compare\n"));
#line 16428 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2810 "../../sim/ppc/powerpc.igen"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        signed_word a;
        signed_word b;
        if (L == 0) {
          a = EXTENDED(*rA);
          b = EXTENDED(*rB);
        }
        else {
          a = *rA;
          b = *rB;
        }
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK | RB_BITMASK, BF_BITMASK);
    }
#line 16457 "icache.c"
    return nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.0,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16461 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Trap_Word(cpu *processor,
                 instruction_word instruction,
                 unsigned_word cia,
                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Trap_Word
#undef MY_PREFIX
#define MY_PREFIX Trap_Word
#line 2930 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Trap Word\n"));
#line 16480 "icache.c"

  /* validate: 0.31,6.TO,11.RA,16.RB,21.4,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c000008)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.TO,11.RA,16.RB,21.4,31./ #define get-values-from-icache|put-values-in-icache */
#line 2930 "../../sim/ppc/powerpc.igen"
    unsigned const TO ATTRIBUTE_UNUSED = cache_entry->crack.X.TO = EXTRACTED32(instruction, 6, 10); /* default in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 16504 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Trap_Word;

    /* semantic routine */
#line 2930 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Trap Word\n"));
#line 16514 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Trap_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2930 "../../sim/ppc/powerpc.igen"
    {
      signed_word a = EXTENDED(*rA);
      signed_word b = EXTENDED(*rB);
      if (TO == 12 && rA == rB) {
        ITRACE(trace_breakpoint, ("breakpoint\n"));
        cpu_halt(processor, cia, was_trap, 0);
      }
      else if ((a < b && TO_0_)
          || (a > b && TO_1_)
          || (a == b && TO_2_)
          || ((unsigned_word)a < (unsigned_word)b && TO_3_)
          || ((unsigned_word)a > (unsigned_word)b && TO_4_)
          )
        program_interrupt(processor, cia,
                          trap_program_interrupt);
    }
#line 16540 "icache.c"
    return nia;
    /* extraction: 0.31,6.TO,11.RA,16.RB,21.4,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16544 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE0_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE0_Rc0
#line 2596 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 16563 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2596 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2596 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 16595 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE0_Rc0;

    /* semantic routine */
#line 2596 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 16605 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2596 "../../sim/ppc/powerpc.igen"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16624 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16628 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE0_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE0_Rc1
#line 2596 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 16647 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2596 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2596 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 16679 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE0_Rc1;

    /* semantic routine */
#line 2596 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 16689 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2596 "../../sim/ppc/powerpc.igen"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16708 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16712 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE0_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE0_Rc0
#line 2586 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 16731 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2586 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2586 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 16763 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE0_Rc0;

    /* semantic routine */
#line 2586 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 16773 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2586 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16789 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16793 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE0_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE0_Rc1
#line 2586 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 16812 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2586 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2586 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 16844 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE0_Rc1;

    /* semantic routine */
#line 2586 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 16854 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2586 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16870 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16874 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Unsigned_Rc0(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Unsigned_Rc0
#line 2730 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Multiply High Word Unsigned\n"));
#line 16893 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000016)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2730 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 16923 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Unsigned_Rc0;

    /* semantic routine */
#line 2730 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Multiply High Word Unsigned\n"));
#line 16933 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2730 "../../sim/ppc/powerpc.igen"
    {
      uint64_t a = (uint32_t)(*rA);
      uint64_t b = (uint32_t)(*rB);
      uint64_t prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 16952 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 16956 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Unsigned_Rc1(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Unsigned_Rc1
#line 2730 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Multiply High Word Unsigned\n"));
#line 16975 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000016)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2730 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 17005 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Unsigned_Rc1;

    /* semantic routine */
#line 2730 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Multiply High Word Unsigned\n"));
#line 17015 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2730 "../../sim/ppc/powerpc.igen"
    {
      uint64_t a = (uint32_t)(*rA);
      uint64_t b = (uint32_t)(*rB);
      uint64_t prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 17034 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.11,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17038 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Condition_Register(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Condition_Register
#undef MY_PREFIX
#define MY_PREFIX Move_From_Condition_Register
#line 3452 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move From Condition Register\n"));
#line 17057 "icache.c"

  /* validate: 0.31,6.RT,11./,16./,21.19,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1fffff) != 0x7c000026)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,16./,21.19,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 17072 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Condition_Register;

    /* semantic routine */
#line 3452 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move From Condition Register\n"));
#line 17082 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Condition_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3452 "../../sim/ppc/powerpc.igen"
    {
      *rT = (uint32_t)CR;
      PPC_INSN_MFCR(RT_BITMASK);
    }
#line 17096 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,16./,21.19,31./ #undef get-values-from-icache|put-values-in-icache */
#line 17099 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_And_Reserve_Indexed(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_And_Reserve_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_And_Reserve_Indexed
#line 2422 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Word And Reserve Indexed\n"));
#line 17118 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.20,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.20,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 17146 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_And_Reserve_Indexed;

    /* semantic routine */
#line 2422 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Word And Reserve Indexed\n"));
#line 17156 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_And_Reserve_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2422 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      RESERVE = 1;
      RESERVE_ADDR = real_addr(EA, 1/*is-read?*/);
      RESERVE_DATA = MEM(unsigned, EA, 4);
      *rT = RESERVE_DATA;
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 17178 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.20,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17182 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero_Indexed(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero_Indexed
#line 1898 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Word and Zero Indexed\n"));
#line 17201 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.23,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.23,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 17229 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero_Indexed;

    /* semantic routine */
#line 1898 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Word and Zero Indexed\n"));
#line 17239 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1898 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(unsigned, EA, 4);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 17258 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.23,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17262 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Left_Word_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Left_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Left_Word_Rc0
#line 3250 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Shift Left Word\n"));
#line 17281 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3250 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 17311 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Left_Word_Rc0;

    /* semantic routine */
#line 3250 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Shift Left Word\n"));
#line 17321 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Left_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3250 "../../sim/ppc/powerpc.igen"
    {
      int n = MASKED(*rB, 58, 63);
      uint32_t source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source << n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu,
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 17346 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17350 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Left_Word_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Left_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Left_Word_Rc1
#line 3250 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Shift Left Word\n"));
#line 17369 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3250 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 17399 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Left_Word_Rc1;

    /* semantic routine */
#line 3250 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Shift Left Word\n"));
#line 17409 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Left_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3250 "../../sim/ppc/powerpc.igen"
    {
      int n = MASKED(*rB, 58, 63);
      uint32_t source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source << n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu,
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 17434 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17438 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Count_Leading_Zeros_Word_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Count_Leading_Zeros_Word
#undef MY_PREFIX
#define MY_PREFIX Count_Leading_Zeros_Word_Rc0
#line 3129 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Count Leading Zeros Word\n"));
#line 17457 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3129 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 17481 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Count_Leading_Zeros_Word_Rc0;

    /* semantic routine */
#line 3129 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Count Leading Zeros Word\n"));
#line 17491 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Count_Leading_Zeros_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3129 "../../sim/ppc/powerpc.igen"
    {
      int count = 0;
      uint32_t mask = BIT32(0);
      uint32_t source = *rS;
      while (!(source & mask) && mask != 0) {
        mask >>= 1;
        count++;
      }
      *rA = count;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      CR0_COMPARE(count, 0, Rc); /* FIXME - is this correct */
    }
#line 17513 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17517 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Count_Leading_Zeros_Word_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Count_Leading_Zeros_Word
#undef MY_PREFIX
#define MY_PREFIX Count_Leading_Zeros_Word_Rc1
#line 3129 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Count Leading Zeros Word\n"));
#line 17536 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3129 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 17560 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Count_Leading_Zeros_Word_Rc1;

    /* semantic routine */
#line 3129 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Count Leading Zeros Word\n"));
#line 17570 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Count_Leading_Zeros_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3129 "../../sim/ppc/powerpc.igen"
    {
      int count = 0;
      uint32_t mask = BIT32(0);
      uint32_t source = *rS;
      while (!(source & mask) && mask != 0) {
        mask >>= 1;
        count++;
      }
      *rA = count;
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      CR0_COMPARE(count, 0, Rc); /* FIXME - is this correct */
    }
#line 17592 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17596 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Rc0(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND
#undef MY_PREFIX
#define MY_PREFIX AND_Rc0
#line 3010 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode AND\n"));
#line 17615 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3010 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 17645 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Rc0;

    /* semantic routine */
#line 3010 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics AND\n"));
#line 17655 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3010 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS & *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 17671 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17675 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_Rc1(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND
#undef MY_PREFIX
#define MY_PREFIX AND_Rc1
#line 3010 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode AND\n"));
#line 17694 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3010 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 17724 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_Rc1;

    /* semantic routine */
#line 3010 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics AND\n"));
#line 17734 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3010 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS & *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 17750 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17754 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Compare_Logical(cpu *processor,
                       instruction_word instruction,
                       unsigned_word cia,
                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Compare_Logical
#undef MY_PREFIX
#define MY_PREFIX Compare_Logical
#line 2852 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Compare Logical\n"));
#line 17773 "icache.c"

  /* validate: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.32,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc4007ff) != 0x7c000040)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.32,31./ #define get-values-from-icache|put-values-in-icache */
#line 67 "../../sim/ppc/powerpc.igen"
    unsigned const BF ATTRIBUTE_UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "../../sim/ppc/powerpc.igen"
    uint32_t const BF_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 2852 "../../sim/ppc/powerpc.igen"
    unsigned const L ATTRIBUTE_UNUSED = cache_entry->crack.X.L = EXTRACTED32(instruction, 10, 10); /* default in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 17801 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Compare_Logical;

    /* semantic routine */
#line 2852 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Compare Logical\n"));
#line 17811 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Compare_Logical, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2852 "../../sim/ppc/powerpc.igen"
    {
      if (!is_64bit_mode && L)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        unsigned_word a;
        unsigned_word b;
        if (L == 0) {
          a = MASKED(*rA, 32, 63);
          b = MASKED(*rB, 32, 63);
        }
        else {
          a = *rA;
          b = *rB;
        }
        CR_COMPARE(BF, a, b);
      }
      PPC_INSN_INT_CR(0, RA_BITMASK | RB_BITMASK, BF_BITMASK);
    }
#line 17840 "icache.c"
    return nia;
    /* extraction: 0.31,6.BF,9./,10.L,11.RA,16.RB,21.32,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17844 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE0_Rc0(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE0_Rc0
#line 2542 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 17863 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2542 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2542 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 17895 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE0_Rc0;

    /* semantic routine */
#line 2542 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 17905 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2542 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 17923 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 17927 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE0_Rc1(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE0_Rc1
#line 2542 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 17946 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2542 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2542 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 17978 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE0_Rc1;

    /* semantic routine */
#line 2542 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 17988 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2542 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 18006 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18010 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Store(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Store
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Store
#line 4811 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Data Cache Block Store\n"));
#line 18029 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.54,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c00006c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.54,31./ #define get-values-from-icache|put-values-in-icache */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18051 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Store;

    /* semantic routine */
#line 4811 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Data Cache Block Store\n"));
#line 18061 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Store, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4811 "../../sim/ppc/powerpc.igen"
    {
      TRACE(trace_tbd,("Data Cache Block Store\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 18075 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.54,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18079 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero_with_Update_Indexed(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero_with_Update_Indexed
#line 1925 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Word and Zero with Update Indexed\n"));
#line 18098 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.55,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00006e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.55,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18126 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero_with_Update_Indexed;

    /* semantic routine */
#line 1925 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Word and Zero with Update Indexed\n"));
#line 18136 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1925 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(unsigned, EA, 4);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 18156 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.55,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18160 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_with_Complement_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_with_Complement
#undef MY_PREFIX
#define MY_PREFIX AND_with_Complement_Rc0
#line 3070 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode AND with Complement\n"));
#line 18179 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000078)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3070 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 18209 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_with_Complement_Rc0;

    /* semantic routine */
#line 3070 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics AND with Complement\n"));
#line 18219 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3070 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS & ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 18235 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18239 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_AND_with_Complement_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_AND_with_Complement
#undef MY_PREFIX
#define MY_PREFIX AND_with_Complement_Rc1
#line 3070 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode AND with Complement\n"));
#line 18258 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000078)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3070 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 18288 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_AND_with_Complement_Rc1;

    /* semantic routine */
#line 3070 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics AND with Complement\n"));
#line 18298 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_AND_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3070 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS & ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 18314 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.60,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18318 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Rc0(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Rc0
#line 2715 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Multiply High Word\n"));
#line 18337 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000096)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2715 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 18367 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Rc0;

    /* semantic routine */
#line 2715 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Multiply High Word\n"));
#line 18377 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2715 "../../sim/ppc/powerpc.igen"
    {
      int64_t a = (int32_t)(*rA);
      int64_t b = (int32_t)(*rB);
      int64_t prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 18396 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18400 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_High_Word_Rc1(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_High_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_High_Word_Rc1
#line 2715 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Multiply High Word\n"));
#line 18419 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000096)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2715 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 18449 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_High_Word_Rc1;

    /* semantic routine */
#line 2715 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Multiply High Word\n"));
#line 18459 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_High_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2715 "../../sim/ppc/powerpc.igen"
    {
      int64_t a = (int32_t)(*rA);
      int64_t b = (int32_t)(*rB);
      int64_t prod = a * b;
      signed_word t = EXTRACTED64(prod, 0, 31);
      *rT = t;
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 18478 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21./,22.75,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18482 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Machine_State_Register(cpu *processor,
                                        instruction_word instruction,
                                        unsigned_word cia,
                                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Machine_State_Register
#undef MY_PREFIX
#define MY_PREFIX Move_From_Machine_State_Register
#line 4898 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move From Machine State Register\n"));
#line 18501 "icache.c"

  /* validate: 0.31,6.RT,11./,16./,21.83,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1fffff) != 0x7c0000a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,16./,21.83,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 18516 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Machine_State_Register;

    /* semantic routine */
#line 4898 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move From Machine State Register\n"));
#line 18526 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Machine_State_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4898 "../../sim/ppc/powerpc.igen"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        *rT = MSR;
        check_masked_interrupts(processor);
      }
    }
#line 18545 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,16./,21.83,31./ #undef get-values-from-icache|put-values-in-icache */
#line 18548 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Flush(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Flush
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Flush
#line 4819 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Data Cache Block Flush\n"));
#line 18567 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.86,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0000ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.86,31./ #define get-values-from-icache|put-values-in-icache */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18589 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Flush;

    /* semantic routine */
#line 4819 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Data Cache Block Flush\n"));
#line 18599 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Flush, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4819 "../../sim/ppc/powerpc.igen"
    {
      TRACE(trace_tbd,("Data Cache Block Flush\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 18613 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.86,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18617 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero_Indexed(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero_Indexed
#line 1736 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero Indexed\n"));
#line 18636 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.87,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0000ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.87,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18664 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero_Indexed;

    /* semantic routine */
#line 1736 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero Indexed\n"));
#line 18674 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1736 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(unsigned, EA, 1);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 18693 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.87,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18697 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE0_Rc0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE0_Rc0
#line 2676 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 18716 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2676 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2676 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 18742 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE0_Rc0;

    /* semantic routine */
#line 2676 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 18752 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2676 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 18769 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18773 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE0_Rc1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE0_Rc1
#line 2676 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 18792 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2676 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2676 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 18818 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE0_Rc1;

    /* semantic routine */
#line 2676 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 18828 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2676 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 18845 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18849 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero_with_Update_Indexed(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero_with_Update_Indexed
#line 1763 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero with Update Indexed\n"));
#line 18868 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.119,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0000ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.119,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 18896 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero_with_Update_Indexed;

    /* semantic routine */
#line 1763 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero with Update Indexed\n"));
#line 18906 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1763 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(unsigned, EA, 1);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 18926 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.119,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 18930 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NOR_Rc0(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NOR
#undef MY_PREFIX
#define MY_PREFIX NOR_Rc0
#line 3050 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode NOR\n"));
#line 18949 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0000f8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3050 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 18979 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NOR_Rc0;

    /* semantic routine */
#line 3050 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics NOR\n"));
#line 18989 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3050 "../../sim/ppc/powerpc.igen"
    {
      *rA = ~(*rS | *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 19005 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19009 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NOR_Rc1(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NOR
#undef MY_PREFIX
#define MY_PREFIX NOR_Rc1
#line 3050 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode NOR\n"));
#line 19028 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0000f8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3050 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 19058 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NOR_Rc1;

    /* semantic routine */
#line 3050 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics NOR\n"));
#line 19068 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3050 "../../sim/ppc/powerpc.igen"
    {
      *rA = ~(*rS | *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 19084 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.124,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19088 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE0_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE0_Rc0
#line 2620 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 19107 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2620 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2620 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 19139 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE0_Rc0;

    /* semantic routine */
#line 2620 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 19149 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2620 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19167 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19171 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE0_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE0_Rc1
#line 2620 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 19190 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2620 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2620 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 19222 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE0_Rc1;

    /* semantic routine */
#line 2620 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 19232 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2620 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19250 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19254 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE0_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE0_Rc0
#line 2609 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 19273 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2609 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2609 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 19305 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE0_Rc0;

    /* semantic routine */
#line 2609 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 19315 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2609 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19332 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19336 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE0_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE0_Rc1
#line 2609 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 19355 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2609 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2609 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 19387 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE0_Rc1;

    /* semantic routine */
#line 2609 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 19397 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2609 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 19414 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19418 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Condition_Register_Fields(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Condition_Register_Fields
#undef MY_PREFIX
#define MY_PREFIX Move_to_Condition_Register_Fields
#line 3429 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move to Condition Register Fields\n"));
#line 19437 "icache.c"

  /* validate: 0.31,6.RS,11./,12.FXM,20./,21.144,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc100fff) != 0x7c000120)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,12.FXM,20./,21.144,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.XFX.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.XFX.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XFX.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 3429 "../../sim/ppc/powerpc.igen"
    unsigned const FXM ATTRIBUTE_UNUSED = cache_entry->crack.XFX.FXM = EXTRACTED32(instruction, 12, 19); /* default in icache - extraction */
#line 19454 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Condition_Register_Fields;

    /* semantic routine */
#line 3429 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move to Condition Register Fields\n"));
#line 19464 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Condition_Register_Fields, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3429 "../../sim/ppc/powerpc.igen"
    {
      if (FXM == 0xff) {
        CR = *rS;
      }
      else {
        unsigned_word mask = 0;
        unsigned_word f;
        for (f = 0; f < 8; f++) {
          if (FXM & (0x80 >> f))
            mask |= (0xf << 4*(7-f));
        }
        CR = (MASKED(*rS, 32, 63) & mask) | (CR & ~mask);
      }
      PPC_INSN_MTCR(RS_BITMASK, FXM);
    }
#line 19489 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,12.FXM,20./,21.144,31./ #undef get-values-from-icache|put-values-in-icache */
#line 19492 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_Machine_State_Register(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_Machine_State_Register
#undef MY_PREFIX
#define MY_PREFIX Move_To_Machine_State_Register
#line 4885 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move To Machine State Register\n"));
#line 19511 "icache.c"

  /* validate: 0.31,6.RS,11./,16./,21.146,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1fffff) != 0x7c000124)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,16./,21.146,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 19526 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_Machine_State_Register;

    /* semantic routine */
#line 4885 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move To Machine State Register\n"));
#line 19536 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_Machine_State_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4885 "../../sim/ppc/powerpc.igen"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        MSR = *rS;
        check_masked_interrupts(processor);
      }
    }
#line 19555 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,16./,21.146,31./ #undef get-values-from-icache|put-values-in-icache */
#line 19558 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_Conditional_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_Conditional_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_Conditional_Indexed
#line 2450 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Word Conditional Indexed\n"));
#line 19577 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.150,31.1 */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00012d)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.150,31.1 #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 19605 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_Conditional_Indexed;

    /* semantic routine */
#line 2450 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Word Conditional Indexed\n"));
#line 19615 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_Conditional_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2450 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      if (RESERVE) {
        if (RESERVE_ADDR == real_addr(EA, 0/*is-read?*/)
            && /*HACK*/ RESERVE_DATA == MEM(unsigned, EA, 4)) {
          STORE(EA, 4, *rS);
          CR_SET_XER_SO(0, cr_i_zero);
        }
        else {
          /* ment to randomly to store, we never do! */	
          CR_SET_XER_SO(0, 0);
        }
        RESERVE = 0;
      }
      else {
        CR_SET_XER_SO(0, 0);
      }
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 1/*Rc*/);
    }
#line 19648 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.150,31.1 #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19652 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_Indexed(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_Indexed
#line 2125 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Word Indexed\n"));
#line 19671 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.151,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00012e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.151,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 19699 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_Indexed;

    /* semantic routine */
#line 2125 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Word Indexed\n"));
#line 19709 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2125 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 19728 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.151,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19732 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_with_Update_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_with_Update_Indexed
#line 2152 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Word with Update Indexed\n"));
#line 19751 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.183,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00016e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.183,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 19779 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_with_Update_Indexed;

    /* semantic routine */
#line 2152 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Word with Update Indexed\n"));
#line 19789 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2152 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 4, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RB_BITMASK | RS_BITMASK, 0);
    }
#line 19809 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.183,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19813 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE0_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE0_Rc0
#line 2665 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 19832 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2665 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2665 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 19858 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE0_Rc0;

    /* semantic routine */
#line 2665 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 19868 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2665 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 19885 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19889 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE0_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE0_Rc1
#line 2665 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 19908 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2665 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2665 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 19934 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE0_Rc1;

    /* semantic routine */
#line 2665 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 19944 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2665 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 19961 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 19965 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE0_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE0_Rc0
#line 2655 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 19984 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2655 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2655 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 20010 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE0_Rc0;

    /* semantic routine */
#line 2655 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 20020 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2655 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20036 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20040 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE0_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE0_Rc1
#line 2655 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 20059 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2655 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2655 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 20085 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE0_Rc1;

    /* semantic routine */
#line 2655 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 20095 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2655 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20111 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20115 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_Segment_Register(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_Segment_Register
#undef MY_PREFIX
#define MY_PREFIX Move_To_Segment_Register
#line 4931 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move To Segment Register\n"));
#line 20134 "icache.c"

  /* validate: 0.31,6.RS,11./,12.SR,16./,21.210,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc10ffff) != 0x7c0001a4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,12.SR,16./,21.210,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 4931 "../../sim/ppc/powerpc.igen"
    unsigned const SR ATTRIBUTE_UNUSED = cache_entry->crack.X.SR = EXTRACTED32(instruction, 12, 15); /* default in icache - extraction */
#line 20151 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_Segment_Register;

    /* semantic routine */
#line 4931 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move To Segment Register\n"));
#line 20161 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_Segment_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4931 "../../sim/ppc/powerpc.igen"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        SEGREG(SR) = *rS;
    }
#line 20178 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,12.SR,16./,21.210,31./ #undef get-values-from-icache|put-values-in-icache */
#line 20181 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte_Indexed(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Byte_Indexed
#line 2017 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Byte Indexed\n"));
#line 20200 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.215,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0001ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.215,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20228 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte_Indexed;

    /* semantic routine */
#line 2017 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Byte Indexed\n"));
#line 20238 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2017 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 1, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 20257 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.215,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20261 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE0_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE0_Rc0
#line 2643 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 20280 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2643 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2643 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 20306 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE0_Rc0;

    /* semantic routine */
#line 2643 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 20316 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2643 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20334 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20338 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE0_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE0_Rc1
#line 2643 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 20357 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2643 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2643 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 20383 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE0_Rc1;

    /* semantic routine */
#line 2643 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 20393 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2643 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20411 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20415 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE0_Rc0(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE0_Rc0
#line 2632 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 20434 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2632 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2632 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 20460 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE0_Rc0;

    /* semantic routine */
#line 2632 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 20470 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2632 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20487 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20491 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE0_Rc1(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE0_Rc1
#line 2632 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 20510 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2632 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2632 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 20536 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE0_Rc1;

    /* semantic routine */
#line 2632 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 20546 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2632 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 20563 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20567 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE0_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE0_Rc0
#line 2698 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 20586 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2698 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2698 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 20618 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE0_Rc0;

    /* semantic routine */
#line 2698 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 20628 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2698 "../../sim/ppc/powerpc.igen"
    {
      int64_t a = (int32_t)(*rA);
      int64_t b = (int32_t)(*rB);
      int64_t prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 20649 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20653 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE0_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE0_Rc1
#line 2698 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 20672 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2698 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2698 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 20704 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE0_Rc1;

    /* semantic routine */
#line 2698 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 20714 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2698 "../../sim/ppc/powerpc.igen"
    {
      int64_t a = (int32_t)(*rA);
      int64_t b = (int32_t)(*rB);
      int64_t prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 20735 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20739 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_Segment_Register_Indirect(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_Segment_Register_Indirect
#undef MY_PREFIX
#define MY_PREFIX Move_To_Segment_Register_Indirect
#line 4942 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move To Segment Register Indirect\n"));
#line 20758 "icache.c"

  /* validate: 0.31,6.RS,11./,16.RB,21.242,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07ff) != 0x7c0001e4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11./,16.RB,21.242,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20779 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_Segment_Register_Indirect;

    /* semantic routine */
#line 4942 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move To Segment Register Indirect\n"));
#line 20789 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_Segment_Register_Indirect, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4942 "../../sim/ppc/powerpc.igen"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        SEGREG(EXTRACTED32(*rB, 0, 3)) = *rS;
    }
#line 20806 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11./,16.RB,21.242,31./ #undef get-values-from-icache|put-values-in-icache */
#line 20809 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Touch_for_Store(cpu *processor,
                                        instruction_word instruction,
                                        unsigned_word cia,
                                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Touch_for_Store
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Touch_for_Store
#line 4795 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Data Cache Block Touch for Store\n"));
#line 20828 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.246,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0001ec)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.246,31./ #define get-values-from-icache|put-values-in-icache */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20850 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Touch_for_Store;

    /* semantic routine */
#line 4795 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Data Cache Block Touch for Store\n"));
#line 20860 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Touch_for_Store, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4795 "../../sim/ppc/powerpc.igen"
    {
      TRACE(trace_tbd,("Data Cache Block Touch for Store\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 20874 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.246,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20878 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte_with_Update_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Byte_with_Update_Indexed
#line 2044 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Byte with Update Indexed\n"));
#line 20897 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.247,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0001ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.247,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 20925 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte_with_Update_Indexed;

    /* semantic routine */
#line 2044 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Byte with Update Indexed\n"));
#line 20935 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2044 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 1, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RB_BITMASK | RS_BITMASK, 0);
    }
#line 20955 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.247,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 20959 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE0_Rc0(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE0_Rc0
#line 2532 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 20978 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2532 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2532 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 21010 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE0_Rc0;

    /* semantic routine */
#line 2532 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 21020 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2532 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 21036 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21040 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE0_Rc1(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE0_Rc1
#line 2532 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 21059 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2532 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2532 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 21091 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE0_Rc1;

    /* semantic routine */
#line 2532 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 21101 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2532 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 21117 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21121 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Touch(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Touch
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Touch
#line 4787 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Data Cache Block Touch\n"));
#line 21140 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.278,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c00022c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.278,31./ #define get-values-from-icache|put-values-in-icache */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21162 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Touch;

    /* semantic routine */
#line 4787 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Data Cache Block Touch\n"));
#line 21172 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Touch, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4787 "../../sim/ppc/powerpc.igen"
    {
      TRACE(trace_tbd,("Data Cache Block Touch\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 21186 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.278,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21190 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero_Indexed(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero_Indexed
#line 1790 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero Indexed\n"));
#line 21209 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.279,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00022e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.279,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21237 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero_Indexed;

    /* semantic routine */
#line 1790 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero Indexed\n"));
#line 21247 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1790 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(unsigned, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 21266 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.279,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21270 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Equivalent_Rc0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Equivalent
#undef MY_PREFIX
#define MY_PREFIX Equivalent_Rc0
#line 3060 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Equivalent\n"));
#line 21289 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000238)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3060 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 21319 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Equivalent_Rc0;

    /* semantic routine */
#line 3060 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Equivalent\n"));
#line 21329 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Equivalent, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3060 "../../sim/ppc/powerpc.igen"
    {
      *rA = ~(*rS ^ *rB); /* A === B */
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21345 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21349 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Equivalent_Rc1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Equivalent
#undef MY_PREFIX
#define MY_PREFIX Equivalent_Rc1
#line 3060 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Equivalent\n"));
#line 21368 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000238)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3060 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 21398 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Equivalent_Rc1;

    /* semantic routine */
#line 3060 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Equivalent\n"));
#line 21408 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Equivalent, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3060 "../../sim/ppc/powerpc.igen"
    {
      *rA = ~(*rS ^ *rB); /* A === B */
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21424 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.284,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21428 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_TLB_Invalidate_Entry(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_TLB_Invalidate_Entry
#undef MY_PREFIX
#define MY_PREFIX TLB_Invalidate_Entry
#line 4980 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode TLB Invalidate Entry\n"));
#line 21447 "icache.c"

  /* validate: 0.31,6./,11./,16.RB,21.306,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffff07ff) != 0x7c000264)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16.RB,21.306,31./ #define get-values-from-icache|put-values-in-icache */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21462 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_TLB_Invalidate_Entry;

    /* semantic routine */
#line 4980 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics TLB Invalidate Entry\n"));
#line 21472 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_TLB_Invalidate_Entry, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4980 "../../sim/ppc/powerpc.igen"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        int nr = 0;
        cpu *proc;
        while (1) {
          proc = psim_cpu(cpu_system(processor), nr);
          if (proc == NULL) break;
          cpu_page_tlb_invalidate_entry(proc, *rB);
          nr++;
        }
      }
    }
#line 21497 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16.RB,21.306,31./ #undef get-values-from-icache|put-values-in-icache */
#line 21500 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero_with_Update_Indexed(cpu *processor,
                                                  instruction_word instruction,
                                                  unsigned_word cia,
                                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero_with_Update_Indexed
#line 1817 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero with Update Indexed\n"));
#line 21519 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.311,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00026e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.311,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21547 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero_with_Update_Indexed;

    /* semantic routine */
#line 1817 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero with Update Indexed\n"));
#line 21557 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1817 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(unsigned, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 21577 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.311,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21581 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Rc0(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR
#undef MY_PREFIX
#define MY_PREFIX XOR_Rc0
#line 3030 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode XOR\n"));
#line 21600 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000278)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3030 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 21630 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Rc0;

    /* semantic routine */
#line 3030 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics XOR\n"));
#line 21640 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3030 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS ^ *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21656 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21660 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_XOR_Rc1(cpu *processor,
               instruction_word instruction,
               unsigned_word cia,
               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_XOR
#undef MY_PREFIX
#define MY_PREFIX XOR_Rc1
#line 3030 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode XOR\n"));
#line 21679 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000278)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3030 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 21709 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_XOR_Rc1;

    /* semantic routine */
#line 3030 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics XOR\n"));
#line 21719 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_XOR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3030 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS ^ *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 21735 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.316,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 21739 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_from_Special_Purpose_Register_SPR256(cpu *processor,
                                                 instruction_word instruction,
                                                 unsigned_word cia,
                                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_from_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_from_Special_Purpose_Register_SPR256
#line 3388 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move from Special Purpose Register\n"));
#line 21758 "icache.c"

  /* validate: 0.31,6.RT,11.SPR,21.339,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XFX.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XFX.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XFX.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 3388 "../../sim/ppc/powerpc.igen"
    unsigned const SPR ATTRIBUTE_UNUSED = 256; /* default in icache - constant field */
#line 21775 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_from_Special_Purpose_Register_SPR256;

    /* semantic routine */
#line 3388 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move from Special Purpose Register\n"));
#line 21785 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_from_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3388 "../../sim/ppc/powerpc.igen"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        /* HACK - time base registers need to be calculated */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_dec:
            *rT = cpu_get_decrementer(processor);
            break;
      	case spr_tbrl:
        	  if (is_64bit_implementation) *rT = TB;
        	  else                         *rT = EXTRACTED64(TB, 32, 63);
      	break;
      	case spr_tbru:
        	  if (is_64bit_implementation) *rT = EXTRACTED64(TB, 0, 31);
      	  else                         *rT = EXTRACTED64(TB, 0, 31);
      	break;
          case spr_tbu:
          case spr_tbl:
            /* NOTE - these SPR's are not readable. Use mftb[ul] */
          default:
            *rT = SPREG(n);
            break;
          }
        }
        else {
          *rT = SPREG(n);
        }
      }
      PPC_INSN_FROM_SPR(RT_BITMASK, n);
    }
#line 21832 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #undef get-values-from-icache|put-values-in-icache */
#line 21835 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_from_Special_Purpose_Register_SPR(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_from_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_from_Special_Purpose_Register_SPR
#line 3388 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move from Special Purpose Register\n"));
#line 21854 "icache.c"

  /* validate: 0.31,6.RT,11.SPR,21.339,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XFX.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XFX.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XFX.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 3388 "../../sim/ppc/powerpc.igen"
    unsigned const SPR ATTRIBUTE_UNUSED = cache_entry->crack.XFX.SPR = EXTRACTED32(instruction, 11, 20); /* default in icache - extraction */
#line 21871 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_from_Special_Purpose_Register_SPR;

    /* semantic routine */
#line 3388 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move from Special Purpose Register\n"));
#line 21881 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_from_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3388 "../../sim/ppc/powerpc.igen"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        /* HACK - time base registers need to be calculated */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_dec:
            *rT = cpu_get_decrementer(processor);
            break;
      	case spr_tbrl:
        	  if (is_64bit_implementation) *rT = TB;
        	  else                         *rT = EXTRACTED64(TB, 32, 63);
      	break;
      	case spr_tbru:
        	  if (is_64bit_implementation) *rT = EXTRACTED64(TB, 0, 31);
      	  else                         *rT = EXTRACTED64(TB, 0, 31);
      	break;
          case spr_tbu:
          case spr_tbl:
            /* NOTE - these SPR's are not readable. Use mftb[ul] */
          default:
            *rT = SPREG(n);
            break;
          }
        }
        else {
          *rT = SPREG(n);
        }
      }
      PPC_INSN_FROM_SPR(RT_BITMASK, n);
    }
#line 21928 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.SPR,21.339,31./ #undef get-values-from-icache|put-values-in-icache */
#line 21931 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic_Indexed(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic_Indexed
#line 1844 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic Indexed\n"));
#line 21950 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.343,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.343,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 21978 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic_Indexed;

    /* semantic routine */
#line 1844 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic Indexed\n"));
#line 21988 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1844 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = MEM(signed, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 22007 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.343,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22011 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_TLB_Invalidate_All(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_TLB_Invalidate_All
#undef MY_PREFIX
#define MY_PREFIX TLB_Invalidate_All
#line 4995 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode TLB Invalidate All\n"));
#line 22030 "icache.c"

  /* validate: 0.31,6./,11./,16./,21.370,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x7c0002e4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16./,21.370,31./ #define get-values-from-icache|put-values-in-icache */
#line 22039 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_TLB_Invalidate_All;

    /* semantic routine */
#line 4995 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics TLB Invalidate All\n"));
#line 22049 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_TLB_Invalidate_All, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4995 "../../sim/ppc/powerpc.igen"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else {
        int nr = 0;
        cpu *proc;
        while (1) {
          proc = psim_cpu(cpu_system(processor), nr);
          if (proc == NULL) break;
          cpu_page_tlb_invalidate_all(proc);
          nr++;
        }
      }
    }
#line 22074 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16./,21.370,31./ #undef get-values-from-icache|put-values-in-icache */
#line 22077 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Time_Base(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Time_Base
#undef MY_PREFIX
#define MY_PREFIX Move_From_Time_Base
#line 4838 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move From Time Base\n"));
#line 22096 "icache.c"

  /* validate: 0.31,6.RT,11.tbr,21.371,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002e6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.tbr,21.371,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XFX.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XFX.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XFX.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 4838 "../../sim/ppc/powerpc.igen"
    unsigned const tbr ATTRIBUTE_UNUSED = cache_entry->crack.XFX.tbr = EXTRACTED32(instruction, 11, 20); /* default in icache - extraction */
#line 22113 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Time_Base;

    /* semantic routine */
#line 4838 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move From Time Base\n"));
#line 22123 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Time_Base, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4838 "../../sim/ppc/powerpc.igen"
    {
      int n = (tbr_5_9_ << 5) | tbr_0_4_;
      if (n == 268) {
        if (is_64bit_implementation) *rT = TB;
        else                         *rT = EXTRACTED64(TB, 32, 63);
      }
      else if (n == 269) {
        if (is_64bit_implementation) *rT = EXTRACTED64(TB, 0, 31);
        else                         *rT = EXTRACTED64(TB, 0, 31);
      }
      else
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
    }
#line 22147 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.tbr,21.371,31./ #undef get-values-from-icache|put-values-in-icache */
#line 22150 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic_with_Update_Indexed(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic_with_Update_Indexed
#line 1871 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic with Update Indexed\n"));
#line 22169 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.375,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0002ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.375,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 22197 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic_with_Update_Indexed;

    /* semantic routine */
#line 1871 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic with Update Indexed\n"));
#line 22207 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1871 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *rT = MEM(signed, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK | RB_BITMASK, 0);
    }
#line 22227 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.375,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22231 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_Indexed(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_Indexed
#line 2071 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Half Word Indexed\n"));
#line 22250 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.407,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00032e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.407,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 22278 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_Indexed;

    /* semantic routine */
#line 2071 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Half Word Indexed\n"));
#line 22288 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2071 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 2, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 22307 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.407,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22311 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_with_Complement_Rc0(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_with_Complement
#undef MY_PREFIX
#define MY_PREFIX OR_with_Complement_Rc0
#line 3080 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode OR with Complement\n"));
#line 22330 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000338)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3080 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 22360 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_with_Complement_Rc0;

    /* semantic routine */
#line 3080 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics OR with Complement\n"));
#line 22370 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3080 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS | ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22386 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22390 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_with_Complement_Rc1(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR_with_Complement
#undef MY_PREFIX
#define MY_PREFIX OR_with_Complement_Rc1
#line 3080 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode OR with Complement\n"));
#line 22409 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000338)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3080 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 22439 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_with_Complement_Rc1;

    /* semantic routine */
#line 3080 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics OR with Complement\n"));
#line 22449 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR_with_Complement, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3080 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS | ~*rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22465 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.412,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22469 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_with_Update_Indexed(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_with_Update_Indexed
#line 2098 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Half Word with Update Indexed\n"));
#line 22488 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.439,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00036e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.439,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 22516 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_with_Update_Indexed;

    /* semantic routine */
#line 2098 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Half Word with Update Indexed\n"));
#line 22526 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2098 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 2, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RB_BITMASK | RS_BITMASK, 0);
    }
#line 22546 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.439,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22550 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Rc0(cpu *processor,
              instruction_word instruction,
              unsigned_word cia,
              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR
#undef MY_PREFIX
#define MY_PREFIX OR_Rc0
#line 3020 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode OR\n"));
#line 22569 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000378)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3020 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 22599 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Rc0;

    /* semantic routine */
#line 3020 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics OR\n"));
#line 22609 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3020 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS | *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22625 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22629 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_OR_Rc1(cpu *processor,
              instruction_word instruction,
              unsigned_word cia,
              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_OR
#undef MY_PREFIX
#define MY_PREFIX OR_Rc1
#line 3020 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode OR\n"));
#line 22648 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000378)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3020 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 22678 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_OR_Rc1;

    /* semantic routine */
#line 3020 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics OR\n"));
#line 22688 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_OR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3020 "../../sim/ppc/powerpc.igen"
    {
      *rA = *rS | *rB;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 22704 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.444,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22708 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE0_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE0_Rc0
#line 2767 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 22727 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2767 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2767 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 22759 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE0_Rc0;

    /* semantic routine */
#line 2767 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 22769 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2767 "../../sim/ppc/powerpc.igen"
    {
      uint64_t dividend = (uint32_t)(*rA);
      uint64_t divisor = (uint32_t)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        uint64_t quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 22794 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22798 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE0_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE0_Rc1
#line 2767 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 22817 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2767 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2767 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 22849 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE0_Rc1;

    /* semantic routine */
#line 2767 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 22859 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2767 "../../sim/ppc/powerpc.igen"
    {
      uint64_t dividend = (uint32_t)(*rA);
      uint64_t divisor = (uint32_t)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        uint64_t quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 22884 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 22888 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Special_Purpose_Register_SPR256(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_to_Special_Purpose_Register_SPR256
#line 3344 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move to Special Purpose Register\n"));
#line 22907 "icache.c"

  /* validate: 0.31,6.RS,11.SPR,21.467,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0003a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.XFX.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.XFX.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XFX.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 3344 "../../sim/ppc/powerpc.igen"
    unsigned const SPR ATTRIBUTE_UNUSED = 256; /* default in icache - constant field */
#line 22924 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Special_Purpose_Register_SPR256;

    /* semantic routine */
#line 3344 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move to Special Purpose Register\n"));
#line 22934 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3344 "../../sim/ppc/powerpc.igen"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n)
               || spr_is_readonly(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        spreg new_val = (spr_length(n) == 64
      		   ? *rS
      		   : MASKED(*rS, 32, 63));
        /* HACK - time base registers need to be updated immediately */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_tbu:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 32, 63)
                               | INSERTED64(new_val, 0, 31)));
            break;
          case spr_tbl:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 0, 31)
                               | INSERTED64(new_val, 32, 63)));
            break;
          case spr_dec:
            cpu_set_decrementer(processor, new_val);
            break;
          default:
            SPREG(n) = new_val;
            break;
          }
        }
        else {
          SPREG(n) = new_val;
        }
      }
      PPC_INSN_TO_SPR(RS_BITMASK, n);
    }
#line 22984 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #undef get-values-from-icache|put-values-in-icache */
#line 22987 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Special_Purpose_Register_SPR(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Special_Purpose_Register
#undef MY_PREFIX
#define MY_PREFIX Move_to_Special_Purpose_Register_SPR
#line 3344 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move to Special Purpose Register\n"));
#line 23006 "icache.c"

  /* validate: 0.31,6.RS,11.SPR,21.467,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0003a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.XFX.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.XFX.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XFX.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 3344 "../../sim/ppc/powerpc.igen"
    unsigned const SPR ATTRIBUTE_UNUSED = cache_entry->crack.XFX.SPR = EXTRACTED32(instruction, 11, 20); /* default in icache - extraction */
#line 23023 "icache.c"

#define insn cache_entry->crack.XFX.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Special_Purpose_Register_SPR;

    /* semantic routine */
#line 3344 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move to Special Purpose Register\n"));
#line 23033 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Special_Purpose_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3344 "../../sim/ppc/powerpc.igen"
    {
      int n = (SPR_5_9_ << 5) | SPR_0_4_;
      if (SPR_0_ && IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else if (!spr_is_valid(n)
               || spr_is_readonly(n))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      else {
        spreg new_val = (spr_length(n) == 64
      		   ? *rS
      		   : MASKED(*rS, 32, 63));
        /* HACK - time base registers need to be updated immediately */
        if (WITH_TIME_BASE) {
          switch (n) {
          case spr_tbu:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 32, 63)
                               | INSERTED64(new_val, 0, 31)));
            break;
          case spr_tbl:
            cpu_set_time_base(processor,
                              (MASKED64(cpu_get_time_base(processor), 0, 31)
                               | INSERTED64(new_val, 32, 63)));
            break;
          case spr_dec:
            cpu_set_decrementer(processor, new_val);
            break;
          default:
            SPREG(n) = new_val;
            break;
          }
        }
        else {
          SPREG(n) = new_val;
        }
      }
      PPC_INSN_TO_SPR(RS_BITMASK, n);
    }
#line 23083 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.SPR,21.467,31./ #undef get-values-from-icache|put-values-in-icache */
#line 23086 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_Invalidate(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_Invalidate
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_Invalidate
#line 4916 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Data Cache Block Invalidate\n"));
#line 23105 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.470,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0003ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.470,31./ #define get-values-from-icache|put-values-in-icache */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 23127 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_Invalidate;

    /* semantic routine */
#line 4916 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Data Cache Block Invalidate\n"));
#line 23137 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_Invalidate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4916 "../../sim/ppc/powerpc.igen"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        TRACE(trace_tbd,("Data Cache Block Invalidate\n"));
    }
#line 23154 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.470,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23158 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NAND_Rc0(cpu *processor,
                instruction_word instruction,
                unsigned_word cia,
                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NAND
#undef MY_PREFIX
#define MY_PREFIX NAND_Rc0
#line 3040 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode NAND\n"));
#line 23177 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0003b8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3040 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 23207 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NAND_Rc0;

    /* semantic routine */
#line 3040 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics NAND\n"));
#line 23217 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NAND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3040 "../../sim/ppc/powerpc.igen"
    {
      *rA = ~(*rS & *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 23233 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23237 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_NAND_Rc1(cpu *processor,
                instruction_word instruction,
                unsigned_word cia,
                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_NAND
#undef MY_PREFIX
#define MY_PREFIX NAND_Rc1
#line 3040 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode NAND\n"));
#line 23256 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c0003b8)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3040 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 23286 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_NAND_Rc1;

    /* semantic routine */
#line 3040 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics NAND\n"));
#line 23296 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_NAND, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3040 "../../sim/ppc/powerpc.igen"
    {
      *rA = ~(*rS & *rB);
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK | RB_BITMASK, Rc);
    }
#line 23312 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.476,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23316 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE0_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE0_Rc0
#line 2745 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 23335 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2745 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2745 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 23367 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE0_Rc0;

    /* semantic routine */
#line 2745 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 23377 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2745 "../../sim/ppc/powerpc.igen"
    {
      int64_t dividend = (int32_t)(*rA);
      int64_t divisor = (int32_t)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        int64_t quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23403 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23407 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE0_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE0_Rc1
#line 2745 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 23426 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2745 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 2745 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 23458 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE0_Rc1;

    /* semantic routine */
#line 2745 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 23468 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2745 "../../sim/ppc/powerpc.igen"
    {
      int64_t dividend = (int32_t)(*rA);
      int64_t divisor = (int32_t)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        int64_t quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23494 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23498 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Condition_Register_from_XER(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Condition_Register_from_XER
#undef MY_PREFIX
#define MY_PREFIX Move_to_Condition_Register_from_XER
#line 3444 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move to Condition Register from XER\n"));
#line 23517 "icache.c"

  /* validate: 0.31,6.BF,9./,11./,16./,21.512,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc7fffff) != 0x7c000400)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.BF,9./,11./,16./,21.512,31./ #define get-values-from-icache|put-values-in-icache */
#line 67 "../../sim/ppc/powerpc.igen"
    unsigned const BF ATTRIBUTE_UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "../../sim/ppc/powerpc.igen"
    uint32_t const BF_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 23530 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Condition_Register_from_XER;

    /* semantic routine */
#line 3444 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move to Condition Register from XER\n"));
#line 23540 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Condition_Register_from_XER, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3444 "../../sim/ppc/powerpc.igen"
    error("%s:%d:0x%08lx:%s unimplemented\n",
          itable[MY_INDEX].file, itable[MY_INDEX].line_nr, (long)cia, itable[MY_INDEX].name);
#line 23552 "icache.c"
    return nia;
    /* extraction: 0.31,6.BF,9./,11./,16./,21.512,31./ #undef get-values-from-icache|put-values-in-icache */
#line 23555 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE1_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE1_Rc0
#line 2596 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 23574 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2596 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2596 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 23606 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE1_Rc0;

    /* semantic routine */
#line 2596 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 23616 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2596 "../../sim/ppc/powerpc.igen"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23635 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23639 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Carrying_OE1_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Carrying
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Carrying_OE1_Rc1
#line 2596 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From Carrying\n"));
#line 23658 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000010)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2596 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2596 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 23690 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Carrying_OE1_Rc1;

    /* semantic routine */
#line 2596 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From Carrying\n"));
#line 23700 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2596 "../../sim/ppc/powerpc.igen"
    {
      /* RT <- ~RA + RB + 1 === RT <- RB - RA */
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23719 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.8,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23723 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE1_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE1_Rc0
#line 2586 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 23742 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2586 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2586 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 23774 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE1_Rc0;

    /* semantic routine */
#line 2586 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 23784 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2586 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23800 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23804 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Carrying_OE1_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Carrying
#undef MY_PREFIX
#define MY_PREFIX Add_Carrying_OE1_Rc1
#line 2586 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add Carrying\n"));
#line 23823 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000014)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2586 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2586 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 23855 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Carrying_OE1_Rc1;

    /* semantic routine */
#line 2586 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add Carrying\n"));
#line 23865 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Carrying, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2586 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 23881 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.10,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23885 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_String_Word_Indexed(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_String_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_String_Word_Indexed
#line 2328 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load String Word Indexed\n"));
#line 23904 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.533,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00042a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.533,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 23932 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_String_Word_Indexed;

    /* semantic routine */
#line 2328 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load String Word Indexed\n"));
#line 23942 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_String_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2328 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      unsigned_word b;
      int n;
      int r;
      int i;
      int nr;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      n = EXTRACTED32(XER, 25, 31);
      r = RT - 1;
      i = 32;
      nr = (n + 3) / 4;
      if (((RT + nr >= 32)
           ? ((RA >= RT || RA < (RT + nr) % 32)
              || (RB >= RT || RB < (RT + nr) % 32))
           : ((RA >= RT && RA < RT + nr)
              || (RB >= RT && RB < RT + nr)))
          || (RT == RA || RT == RB))
        program_interrupt(processor, cia,
                        illegal_instruction_program_interrupt);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      while (n > 0) {
        if (i == 32) {
          r = (r + 1) % 32;
          GPR(r) = 0;
        }
        GPR(r) |= INSERTED(MEM(unsigned, EA, 1), i, i+7);
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 23988 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.533,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 23992 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_ByteReverse_Indexed(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_ByteReverse_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Word_ByteReverse_Indexed
#line 2215 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Word Byte-Reverse Indexed\n"));
#line 24011 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.534,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00042c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.534,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24039 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_ByteReverse_Indexed;

    /* semantic routine */
#line 2215 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Word Byte-Reverse Indexed\n"));
#line 24049 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_ByteReverse_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2215 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = SWAP_4(MEM(unsigned, EA, 4));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24068 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.534,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24072 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single_Indexed(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single_Indexed
#line 3477 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single Indexed\n"));
#line 24091 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.535,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00042e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.535,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24126 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single_Indexed;

    /* semantic routine */
#line 3477 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single Indexed\n"));
#line 24136 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3477 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24155 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.535,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 24160 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Word_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Word_Rc0
#line 3271 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Shift Right Word\n"));
#line 24179 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000430)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3271 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 24209 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Word_Rc0;

    /* semantic routine */
#line 3271 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Shift Right Word\n"));
#line 24219 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3271 "../../sim/ppc/powerpc.igen"
    {
      int n = MASKED(*rB, 58, 63);
      uint32_t source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source >> n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, \
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 24244 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24248 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Word_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Word_Rc1
#line 3271 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Shift Right Word\n"));
#line 24267 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000430)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3271 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 24297 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Word_Rc1;

    /* semantic routine */
#line 3271 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Shift Right Word\n"));
#line 24307 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3271 "../../sim/ppc/powerpc.igen"
    {
      int n = MASKED(*rB, 58, 63);
      uint32_t source = *rS;
      signed_word shifted;
      if (n < 32)
        shifted = (source >> n);
      else
        shifted = 0;
      *rA = shifted;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, \
             ("n=%d, source=0x%lx, shifted=0x%lx\n",
              n, (unsigned long)source, (unsigned long)shifted));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 24332 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.536,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24336 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE1_Rc0(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE1_Rc0
#line 2542 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 24355 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2542 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2542 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 24387 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE1_Rc0;

    /* semantic routine */
#line 2542 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 24397 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2542 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 24415 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24419 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_OE1_Rc1(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_OE1_Rc1
#line 2542 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From\n"));
#line 24438 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2542 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2542 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 24470 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_OE1_Rc1;

    /* semantic routine */
#line 2542 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From\n"));
#line 24480 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2542 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD(1);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 24498 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24502 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_TLB_Synchronize(cpu *processor,
                       instruction_word instruction,
                       unsigned_word cia,
                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_TLB_Synchronize
#undef MY_PREFIX
#define MY_PREFIX TLB_Synchronize
#line 5010 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode TLB Synchronize\n"));
#line 24521 "icache.c"

  /* validate: 0.31,6./,11./,16./,21.566,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x7c00046c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16./,21.566,31./ #define get-values-from-icache|put-values-in-icache */
#line 24530 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_TLB_Synchronize;

    /* semantic routine */
#line 5010 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics TLB Synchronize\n"));
#line 24540 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_TLB_Synchronize, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 5010 "../../sim/ppc/powerpc.igen"
    {
      /* nothing happens here - always in sync */
    }
#line 24553 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16./,21.566,31./ #undef get-values-from-icache|put-values-in-icache */
#line 24556 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single_with_Update_Indexed(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single_with_Update_Indexed
#line 3504 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single with Update Indexed\n"));
#line 24575 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.567,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00046e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.567,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24610 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single_with_Update_Indexed;

    /* semantic routine */
#line 3504 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single with Update Indexed\n"));
#line 24620 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3504 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24640 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.567,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 24645 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Segment_Register(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Segment_Register
#undef MY_PREFIX
#define MY_PREFIX Move_From_Segment_Register
#line 4953 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move From Segment Register\n"));
#line 24664 "icache.c"

  /* validate: 0.31,6.RT,11./,12.SR,16./,21.595,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc10ffff) != 0x7c0004a6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,12.SR,16./,21.595,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 4953 "../../sim/ppc/powerpc.igen"
    unsigned const SR ATTRIBUTE_UNUSED = cache_entry->crack.X.SR = EXTRACTED32(instruction, 12, 15); /* default in icache - extraction */
#line 24681 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Segment_Register;

    /* semantic routine */
#line 4953 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move From Segment Register\n"));
#line 24691 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Segment_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4953 "../../sim/ppc/powerpc.igen"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        *rT = SEGREG(SR);
    }
#line 24708 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,12.SR,16./,21.595,31./ #undef get-values-from-icache|put-values-in-icache */
#line 24711 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_String_Word_Immediate(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_String_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Load_String_Word_Immediate
#line 2296 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load String Word Immediate\n"));
#line 24730 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.NB,21.597,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0004aa)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.NB,21.597,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2296 "../../sim/ppc/powerpc.igen"
    unsigned const NB ATTRIBUTE_UNUSED = cache_entry->crack.X.NB = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 24754 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_String_Word_Immediate;

    /* semantic routine */
#line 2296 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load String Word Immediate\n"));
#line 24764 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_String_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2296 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      int n;
      int r;
      int i;
      int nr;
      if (RA_is_0) EA = 0;
      else         EA = *rA;
      if (NB == 0) n = 32;
      else         n = NB;
      r = RT - 1;
      i = 32;
      nr = (n + 3) / 4;
      if ((RT + nr >= 32)
          ? (RA >= RT || RA < (RT + nr) % 32)
          : (RA >= RT && RA < RT + nr))
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      while (n > 0) {
        if (i == 32) {
          r = (r + 1) % 32;
          GPR(r) = 0;
        }
        GPR(r) |= INSERTED(MEM(unsigned, EA, 1), i, i+7);
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 24806 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.NB,21.597,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 24810 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Synchronize(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Synchronize
#undef MY_PREFIX
#define MY_PREFIX Synchronize
#line 2500 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Synchronize\n"));
#line 24829 "icache.c"

  /* validate: 0.31,6./,9.L,11./,16./,21.598,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xff9fffff) != 0x7c0004ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,9.L,11./,16./,21.598,31./ #define get-values-from-icache|put-values-in-icache */
#line 2500 "../../sim/ppc/powerpc.igen"
    unsigned const L ATTRIBUTE_UNUSED = cache_entry->crack.X.L = EXTRACTED32(instruction, 9, 10); /* default in icache - extraction */
#line 24840 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Synchronize;

    /* semantic routine */
#line 2500 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Synchronize\n"));
#line 24850 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Synchronize, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2500 "../../sim/ppc/powerpc.igen"
    {
      /* do nothing */
    }
#line 24863 "icache.c"
    return nia;
    /* extraction: 0.31,6./,9.L,11./,16./,21.598,31./ #undef get-values-from-icache|put-values-in-icache */
#line 24866 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double_Indexed(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double_Indexed
#line 3531 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double Indexed\n"));
#line 24885 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.599,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0004ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.599,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 24920 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double_Indexed;

    /* semantic routine */
#line 3531 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double Indexed\n"));
#line 24930 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3531 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *frT = MEM(unsigned, EA, 8);
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 24949 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.599,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 24954 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE1_Rc0(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE1_Rc0
#line 2676 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 24973 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2676 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2676 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 24999 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE1_Rc0;

    /* semantic routine */
#line 2676 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 25009 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2676 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 25026 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25030 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Negate_OE1_Rc1(cpu *processor,
                      instruction_word instruction,
                      unsigned_word cia,
                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Negate
#undef MY_PREFIX
#define MY_PREFIX Negate_OE1_Rc1
#line 2676 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Negate\n"));
#line 25049 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0000d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2676 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2676 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 25075 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Negate_OE1_Rc1;

    /* semantic routine */
#line 2676 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Negate\n"));
#line 25085 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2676 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(1);
      ALU_END(*rT,0/*CA*/,OE,Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 25102 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.104,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25106 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double_with_Update_Indexed(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double_with_Update_Indexed
#line 3558 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double with Update Indexed\n"));
#line 25125 "icache.c"

  /* validate: 0.31,6.FRT,11.RA,16.RB,21.631,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0004ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.631,31./ #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25160 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double_with_Update_Indexed;

    /* semantic routine */
#line 3558 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double with Update Indexed\n"));
#line 25170 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3558 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      *frT = MEM(unsigned, EA, 8);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 25190 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRT,11.RA,16.RB,21.631,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 25195 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE1_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE1_Rc0
#line 2620 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 25214 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2620 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2620 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 25246 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE1_Rc0;

    /* semantic routine */
#line 2620 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 25256 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2620 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25274 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25278 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Extended_OE1_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Extended_OE1_Rc1
#line 2620 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From Extended\n"));
#line 25297 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2620 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2620 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 25329 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Extended_OE1_Rc1;

    /* semantic routine */
#line 2620 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From Extended\n"));
#line 25339 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2620 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25357 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25361 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE1_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE1_Rc0
#line 2609 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 25380 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2609 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2609 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 25412 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE1_Rc0;

    /* semantic routine */
#line 2609 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 25422 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2609 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25439 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25443 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_Extended_OE1_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_Extended_OE1_Rc1
#line 2609 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add Extended\n"));
#line 25462 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000114)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2609 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2609 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 25494 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_Extended_OE1_Rc1;

    /* semantic routine */
#line 2609 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add Extended\n"));
#line 25504 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2609 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 25521 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.138,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25525 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_Segment_Register_Indirect(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_Segment_Register_Indirect
#undef MY_PREFIX
#define MY_PREFIX Move_From_Segment_Register_Indirect
#line 4964 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move From Segment Register Indirect\n"));
#line 25544 "icache.c"

  /* validate: 0.31,6.RT,11./,16.RB,21.659,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07ff) != 0x7c000526)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11./,16.RB,21.659,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25565 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_Segment_Register_Indirect;

    /* semantic routine */
#line 4964 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move From Segment Register Indirect\n"));
#line 25575 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_Segment_Register_Indirect, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4964 "../../sim/ppc/powerpc.igen"
    {
      if (IS_PROBLEM_STATE(processor))
        program_interrupt(processor, cia,
                          privileged_instruction_program_interrupt);
      else
        *rT = SEGREG(EXTRACTED32(*rB, 0, 3));
    }
#line 25592 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11./,16.RB,21.659,31./ #undef get-values-from-icache|put-values-in-icache */
#line 25595 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_String_Word_Indexed(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_String_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_String_Word_Indexed
#line 2386 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store String Word Indexed\n"));
#line 25614 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.661,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00052a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.661,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25642 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_String_Word_Indexed;

    /* semantic routine */
#line 2386 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store String Word Indexed\n"));
#line 25652 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_String_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2386 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      unsigned_word b;
      int n;
      int r;
      int i;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      n = EXTRACTED32(XER, 25, 31);
      r = RS - 1;
      i = 32;
      while (n > 0) {
        if (i == 32) r = (r + 1) % 32;
        STORE(EA, 1, EXTRACTED(GPR(r), i, i+7));
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 25685 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.661,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25689 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_ByteReversed_Indexed(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_ByteReversed_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Word_ByteReversed_Indexed
#line 2241 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Word Byte-Reversed Indexed\n"));
#line 25708 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.662,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00052c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.662,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25736 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_ByteReversed_Indexed;

    /* semantic routine */
#line 2241 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Word Byte-Reversed Indexed\n"));
#line 25746 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_ByteReversed_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2241 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, SWAP_4(*rS));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 25765 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.662,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 25769 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single_Indexed(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single_Indexed
#line 3590 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single Indexed\n"));
#line 25788 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.663,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00052e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.663,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "../../sim/ppc/powerpc.igen"
    uint64_t * const frS ATTRIBUTE_UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "../../sim/ppc/powerpc.igen"
    uint32_t const FRS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25823 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single_Indexed;

    /* semantic routine */
#line 3590 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single Indexed\n"));
#line 25833 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3590 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, SINGLE(*frS));
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 25852 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.663,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 25857 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single_with_Update_Indexed(cpu *processor,
                                                      instruction_word instruction,
                                                      unsigned_word cia,
                                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single_with_Update_Indexed
#line 3617 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single with Update Indexed\n"));
#line 25876 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.695,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00056e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.695,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "../../sim/ppc/powerpc.igen"
    uint64_t * const frS ATTRIBUTE_UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "../../sim/ppc/powerpc.igen"
    uint32_t const FRS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 25911 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single_with_Update_Indexed;

    /* semantic routine */
#line 3617 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single with Update Indexed\n"));
#line 25921 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3617 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 4, SINGLE(*frS));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 25941 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.695,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 25946 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE1_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE1_Rc0
#line 2665 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 25965 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2665 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2665 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 25991 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE1_Rc0;

    /* semantic routine */
#line 2665 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 26001 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2665 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26018 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26022 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_from_Zero_Extended_OE1_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_from_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_from_Zero_Extended_OE1_Rc1
#line 2665 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract from Zero Extended\n"));
#line 26041 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000190)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2665 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2665 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 26067 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_from_Zero_Extended_OE1_Rc1;

    /* semantic routine */
#line 2665 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract from Zero Extended\n"));
#line 26077 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_from_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2665 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26094 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.200,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26098 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE1_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE1_Rc0
#line 2655 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 26117 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2655 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2655 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 26143 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE1_Rc0;

    /* semantic routine */
#line 2655 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 26153 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2655 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26169 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26173 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Zero_Extended_OE1_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Zero_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Zero_Extended_OE1_Rc1
#line 2655 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add to Zero Extended\n"));
#line 26192 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c000194)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2655 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2655 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 26218 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Zero_Extended_OE1_Rc1;

    /* semantic routine */
#line 2655 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add to Zero Extended\n"));
#line 26228 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Zero_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2655 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26244 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.202,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26248 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_String_Word_Immedate(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_String_Word_Immedate
#undef MY_PREFIX
#define MY_PREFIX Store_String_Word_Immedate
#line 2364 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store String Word Immedate\n"));
#line 26267 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.NB,21.725,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0005aa)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.NB,21.725,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2364 "../../sim/ppc/powerpc.igen"
    unsigned const NB ATTRIBUTE_UNUSED = cache_entry->crack.X.NB = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 26291 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_String_Word_Immedate;

    /* semantic routine */
#line 2364 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store String Word Immedate\n"));
#line 26301 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_String_Word_Immedate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2364 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      int n;
      int r;
      int i;
      if (RA_is_0) EA = 0;
      else         EA = *rA;
      if (NB == 0) n = 32;
      else         n = NB;
      r = RS - 1;
      i = 32;
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT)
        alignment_interrupt(processor, cia, EA);
      while (n > 0) {
        if (i == 32) r = (r + 1) % 32;
        STORE(EA, 1, EXTRACTED(GPR(r), i, i+7));
        i = i + 8;
        if (i == 64) i = 32;
        EA = EA + 1;
        n = n - 1;
      }
    }
#line 26333 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.NB,21.725,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26337 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double_Indexed(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double_Indexed
#line 3644 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double Indexed\n"));
#line 26356 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.727,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0005ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.727,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "../../sim/ppc/powerpc.igen"
    uint64_t * const frS ATTRIBUTE_UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "../../sim/ppc/powerpc.igen"
    uint32_t const FRS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 26391 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double_Indexed;

    /* semantic routine */
#line 3644 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double Indexed\n"));
#line 26401 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3644 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 8, *frS);
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 26420 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.727,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 26425 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE1_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE1_Rc0
#line 2643 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 26444 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2643 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2643 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 26470 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE1_Rc0;

    /* semantic routine */
#line 2643 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 26480 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2643 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26498 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26502 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Subtract_From_Minus_One_Extended_OE1_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Subtract_From_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Subtract_From_Minus_One_Extended_OE1_Rc1
#line 2643 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Subtract From Minus One Extended\n"));
#line 26521 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d0)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2643 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2643 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 26547 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Subtract_From_Minus_One_Extended_OE1_Rc1;

    /* semantic routine */
#line 2643 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Subtract From Minus One Extended\n"));
#line 26557 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Subtract_From_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2643 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_NOT;
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26575 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.232,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26579 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE1_Rc0(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE1_Rc0
#line 2632 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 26598 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2632 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2632 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 26624 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE1_Rc0;

    /* semantic routine */
#line 2632 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 26634 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2632 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26651 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26655 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_to_Minus_One_Extended_OE1_Rc1(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add_to_Minus_One_Extended
#undef MY_PREFIX
#define MY_PREFIX Add_to_Minus_One_Extended_OE1_Rc1
#line 2632 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add to Minus One Extended\n"));
#line 26674 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fbfe) != 0x7c0001d4)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 2632 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2632 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 26700 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_to_Minus_One_Extended_OE1_Rc1;

    /* semantic routine */
#line 2632 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add to Minus One Extended\n"));
#line 26710 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add_to_Minus_One_Extended, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2632 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD_CA;
      ALU_ADD(-1);
      ALU_END(*rT, 1/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK, Rc);
    }
#line 26727 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16./,21.OE,22.234,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26731 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE1_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE1_Rc0
#line 2698 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 26750 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2698 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2698 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 26782 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE1_Rc0;

    /* semantic routine */
#line 2698 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 26792 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2698 "../../sim/ppc/powerpc.igen"
    {
      int64_t a = (int32_t)(*rA);
      int64_t b = (int32_t)(*rB);
      int64_t prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 26813 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26817 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Multiply_Low_Word_OE1_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Multiply_Low_Word
#undef MY_PREFIX
#define MY_PREFIX Multiply_Low_Word_OE1_Rc1
#line 2698 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Multiply Low Word\n"));
#line 26836 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0001d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2698 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2698 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 26868 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Multiply_Low_Word_OE1_Rc1;

    /* semantic routine */
#line 2698 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Multiply Low Word\n"));
#line 26878 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Multiply_Low_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2698 "../../sim/ppc/powerpc.igen"
    {
      int64_t a = (int32_t)(*rA);
      int64_t b = (int32_t)(*rB);
      int64_t prod = a * b;
      signed_word t = prod;
      *rT = *rA * *rB;
      if (t != prod && OE)
        XER |= (xer_overflow | xer_summary_overflow);
      CR0_COMPARE(t, 0, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 26899 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.235,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 26903 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double_with_Update_Indexed(cpu *processor,
                                                      instruction_word instruction,
                                                      unsigned_word cia,
                                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double_with_Update_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double_with_Update_Indexed
#line 3683 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double with Update Indexed\n"));
#line 26922 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.759,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0005ee)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.759,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "../../sim/ppc/powerpc.igen"
    uint64_t * const frS ATTRIBUTE_UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "../../sim/ppc/powerpc.igen"
    uint32_t const FRS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 26957 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double_with_Update_Indexed;

    /* semantic routine */
#line 3683 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double with Update Indexed\n"));
#line 26967 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double_with_Update_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3683 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + *rB;
      STORE(EA, 8, *frS);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 26987 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.759,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 26992 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE1_Rc0(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE1_Rc0
#line 2532 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 27011 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2532 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2532 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 27043 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE1_Rc0;

    /* semantic routine */
#line 2532 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 27053 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2532 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 27069 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27073 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Add_OE1_Rc1(cpu *processor,
                   instruction_word instruction,
                   unsigned_word cia,
                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Add
#undef MY_PREFIX
#define MY_PREFIX Add_OE1_Rc1
#line 2532 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Add\n"));
#line 27092 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000214)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2532 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2532 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 27124 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Add_OE1_Rc1;

    /* semantic routine */
#line 2532 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Add\n"));
#line 27134 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2532 "../../sim/ppc/powerpc.igen"
    {
      ALU_BEGIN(*rA);
      ALU_ADD(*rB);
      ALU_END(*rT, 0/*CA*/, OE, Rc);
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 27150 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.266,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27154 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_ByteReverse_Indexed(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_ByteReverse_Indexed
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_ByteReverse_Indexed
#line 2202 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Halfword Byte-Reverse Indexed\n"));
#line 27173 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.790,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00062c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.790,31./ #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.X.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.X.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 27201 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_ByteReverse_Indexed;

    /* semantic routine */
#line 2202 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Halfword Byte-Reverse Indexed\n"));
#line 27211 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_ByteReverse_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2202 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      *rT = SWAP_2(MEM(unsigned, EA, 2));
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 27230 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.790,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27234 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Rc0
#line 3314 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word\n"));
#line 27253 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000630)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3314 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 27283 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Rc0;

    /* semantic routine */
#line 3314 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word\n"));
#line 27293 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3314 "../../sim/ppc/powerpc.igen"
    {
      uint64_t mask;
      int n = MASKED(*rB, 59, 63);
      int32_t source = (int32_t)*rS; /* signed to keep sign bit */
      int S = (MASKED(*rS,32,32) != 0);
      int64_t r = ((uint64_t) source);
      r = ((uint64_t) source) << 32 | (uint32_t) source;
      r = ROTL64(r,64-n);
      if (MASKED(*rB,58,58) == 0)
      	mask = (uint64_t) MASK64(n+32,63);
      else
      	mask = (uint64_t) 0;
      *rA = (signed_word) ((r & mask) | (((int64_t) -1*S) & ~mask)); /* if 64bit will sign extend */
      if (S && (MASKED(r & ~mask,32,63)!=0))
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27325 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27329 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Rc1
#line 3314 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word\n"));
#line 27348 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000630)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 3314 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 27378 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Rc1;

    /* semantic routine */
#line 3314 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word\n"));
#line 27388 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3314 "../../sim/ppc/powerpc.igen"
    {
      uint64_t mask;
      int n = MASKED(*rB, 59, 63);
      int32_t source = (int32_t)*rS; /* signed to keep sign bit */
      int S = (MASKED(*rS,32,32) != 0);
      int64_t r = ((uint64_t) source);
      r = ((uint64_t) source) << 32 | (uint32_t) source;
      r = ROTL64(r,64-n);
      if (MASKED(*rB,58,58) == 0)
      	mask = (uint64_t) MASK64(n+32,63);
      else
      	mask = (uint64_t) 0;
      *rA = (signed_word) ((r & mask) | (((int64_t) -1*S) & ~mask)); /* if 64bit will sign extend */
      if (S && (MASKED(r & ~mask,32,63)!=0))
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27420 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.792,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27424 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Immediate_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Immediate_Rc0
#line 3292 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word Immediate\n"));
#line 27443 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000670)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3292 "../../sim/ppc/powerpc.igen"
    unsigned const SH ATTRIBUTE_UNUSED = cache_entry->crack.X.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3292 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 27469 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Immediate_Rc0;

    /* semantic routine */
#line 3292 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word Immediate\n"));
#line 27479 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3292 "../../sim/ppc/powerpc.igen"
    {
      int n = SH;
      signed_word r = ROTL32(*rS, /*64*/32-n);
      signed_word m = MASK(n+32, 63);
      int S = MASKED(*rS, 32, 32);
      signed_word shifted = (r & m) | (S ? ~m : 0);
      *rA = shifted;
      if (S && ((r & ~m) & MASK(32, 63)) != 0)
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27505 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27509 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Shift_Right_Algebraic_Word_Immediate_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Shift_Right_Algebraic_Word_Immediate
#undef MY_PREFIX
#define MY_PREFIX Shift_Right_Algebraic_Word_Immediate_Rc1
#line 3292 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Shift Right Algebraic Word Immediate\n"));
#line 27528 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0x7c000670)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3292 "../../sim/ppc/powerpc.igen"
    unsigned const SH ATTRIBUTE_UNUSED = cache_entry->crack.X.SH = EXTRACTED32(instruction, 16, 20); /* default in icache - extraction */
#line 3292 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 27554 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Shift_Right_Algebraic_Word_Immediate_Rc1;

    /* semantic routine */
#line 3292 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Shift Right Algebraic Word Immediate\n"));
#line 27564 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Shift_Right_Algebraic_Word_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3292 "../../sim/ppc/powerpc.igen"
    {
      int n = SH;
      signed_word r = ROTL32(*rS, /*64*/32-n);
      signed_word m = MASK(n+32, 63);
      int S = MASKED(*rS, 32, 32);
      signed_word shifted = (r & m) | (S ? ~m : 0);
      *rA = shifted;
      if (S && ((r & ~m) & MASK(32, 63)) != 0)
        XER |= xer_carry;
      else
        XER &= ~xer_carry;
      CR0_COMPARE(shifted, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx), XER = %ld\n",
      		   (long)*rA, (long)*rA, (long)XER));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27590 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.SH,21.824,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27594 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Enforce_Inorder_Execution_of_IO(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Enforce_Inorder_Execution_of_IO
#undef MY_PREFIX
#define MY_PREFIX Enforce_Inorder_Execution_of_IO
#line 4827 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Enforce In-order Execution of I/O\n"));
#line 27613 "icache.c"

  /* validate: 0.31,6./,11./,16./,21.854,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffffffff) != 0x7c0006ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11./,16./,21.854,31./ #define get-values-from-icache|put-values-in-icache */
#line 27622 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Enforce_Inorder_Execution_of_IO;

    /* semantic routine */
#line 4827 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Enforce In-order Execution of I/O\n"));
#line 27632 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Enforce_Inorder_Execution_of_IO, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4827 "../../sim/ppc/powerpc.igen"
    {
      /* Since this model has no instruction overlap
         this instruction need do nothing */
    }
#line 27646 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11./,16./,21.854,31./ #undef get-values-from-icache|put-values-in-icache */
#line 27649 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_ByteReversed_Indexed(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_ByteReversed_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_ByteReversed_Indexed
#line 2228 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Half Word Byte-Reversed Indexed\n"));
#line 27668 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16.RB,21.918,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c00072c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.918,31./ #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 27696 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_ByteReversed_Indexed;

    /* semantic routine */
#line 2228 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Half Word Byte-Reversed Indexed\n"));
#line 27706 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_ByteReversed_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2228 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 2, SWAP_2(*rS));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK | RS_BITMASK, 0);
    }
#line 27725 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16.RB,21.918,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27729 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Half_Word_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Half_Word
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Half_Word_Rc0
#line 3100 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Extend Sign Half Word\n"));
#line 27748 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.922,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000734)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3100 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 27772 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Half_Word_Rc0;

    /* semantic routine */
#line 3100 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Extend Sign Half Word\n"));
#line 27782 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Half_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3100 "../../sim/ppc/powerpc.igen"
    {
      *rA = (signed_word)(int16_t)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27798 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27802 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Half_Word_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Half_Word
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Half_Word_Rc1
#line 3100 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Extend Sign Half Word\n"));
#line 27821 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.922,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000734)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3100 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 27845 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Half_Word_Rc1;

    /* semantic routine */
#line 3100 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Extend Sign Half Word\n"));
#line 27855 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Half_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3100 "../../sim/ppc/powerpc.igen"
    {
      *rA = (signed_word)(int16_t)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27871 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.922,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27875 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Byte_Rc0(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Byte
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Byte_Rc0
#line 3090 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Extend Sign Byte\n"));
#line 27894 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.954,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000774)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3090 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 27918 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Byte_Rc0;

    /* semantic routine */
#line 3090 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Extend Sign Byte\n"));
#line 27928 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Byte, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3090 "../../sim/ppc/powerpc.igen"
    {
      *rA = (signed_word)(int8_t)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 27944 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 27948 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Extend_Sign_Byte_Rc1(cpu *processor,
                            instruction_word instruction,
                            unsigned_word cia,
                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Extend_Sign_Byte
#undef MY_PREFIX
#define MY_PREFIX Extend_Sign_Byte_Rc1
#line 3090 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Extend Sign Byte\n"));
#line 27967 "icache.c"

  /* validate: 0.31,6.RS,11.RA,16./,21.954,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00fffe) != 0x7c000774)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.X.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.X.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 3090 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 27991 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Extend_Sign_Byte_Rc1;

    /* semantic routine */
#line 3090 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Extend Sign Byte\n"));
#line 28001 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Extend_Sign_Byte, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3090 "../../sim/ppc/powerpc.igen"
    {
      *rA = (signed_word)(int8_t)*rS;
      CR0_COMPARE(*rA, 0, Rc);
      ITRACE(trace_alu, (" Result = %ld (0x%lx)\n", (long)*rA, (long)*rA));
      PPC_INSN_INT(RA_BITMASK, RS_BITMASK, Rc);
    }
#line 28017 "icache.c"
    return nia;
    /* extraction: 0.31,6.RS,11.RA,16./,21.954,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28021 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE1_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE1_Rc0
#line 2767 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 28040 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2767 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2767 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 28072 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE1_Rc0;

    /* semantic routine */
#line 2767 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 28082 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2767 "../../sim/ppc/powerpc.igen"
    {
      uint64_t dividend = (uint32_t)(*rA);
      uint64_t divisor = (uint32_t)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        uint64_t quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28107 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28111 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_Unsigned_OE1_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word_Unsigned
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_Unsigned_OE1_Rc1
#line 2767 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Divide Word Unsigned\n"));
#line 28130 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c000396)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2767 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2767 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 28162 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_Unsigned_OE1_Rc1;

    /* semantic routine */
#line 2767 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Divide Word Unsigned\n"));
#line 28172 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word_Unsigned, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2767 "../../sim/ppc/powerpc.igen"
    {
      uint64_t dividend = (uint32_t)(*rA);
      uint64_t divisor = (uint32_t)(*rB);
      if (divisor == 0) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        uint64_t quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28197 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.459,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28201 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Instruction_Cache_Block_Invalidate(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Instruction_Cache_Block_Invalidate
#undef MY_PREFIX
#define MY_PREFIX Instruction_Cache_Block_Invalidate
#line 4763 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Instruction Cache Block Invalidate\n"));
#line 28220 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.982,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0007ac)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.982,31./ #define get-values-from-icache|put-values-in-icache */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 28242 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Instruction_Cache_Block_Invalidate;

    /* semantic routine */
#line 4763 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Instruction Cache Block Invalidate\n"));
#line 28252 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Instruction_Cache_Block_Invalidate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4763 "../../sim/ppc/powerpc.igen"
    {
      /* blindly flush all instruction cache entries */
#if WITH_IDECODE_CACHE_SIZE
      cpu_flush_icache(processor);
#endif
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0);
    }
#line 28269 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.982,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28273 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Integer_Word_Indexed(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Integer_Word_Indexed
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Integer_Word_Indexed
#line 3656 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Integer Word Indexed\n"));
#line 28292 "icache.c"

  /* validate: 0.31,6.FRS,11.RA,16.RB,21.983,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007ff) != 0x7c0007ae)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.983,31./ #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "../../sim/ppc/powerpc.igen"
    uint64_t * const frS ATTRIBUTE_UNUSED = cache_entry->crack.X.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "../../sim/ppc/powerpc.igen"
    uint32_t const FRS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 28327 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Integer_Word_Indexed;

    /* semantic routine */
#line 3656 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Integer Word Indexed\n"));
#line 28337 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Integer_Word_Indexed, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3656 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + *rB;
      STORE(EA, 4, *frS);
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1) | RB_BITMASK, FRS_BITMASK);
    }
#line 28356 "icache.c"
    return nia;
    /* extraction: 0.31,6.FRS,11.RA,16.RB,21.983,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 28361 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE1_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE1_Rc0
#line 2745 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 28380 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2745 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2745 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 28412 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE1_Rc0;

    /* semantic routine */
#line 2745 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 28422 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2745 "../../sim/ppc/powerpc.igen"
    {
      int64_t dividend = (int32_t)(*rA);
      int64_t divisor = (int32_t)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        int64_t quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28448 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28452 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Divide_Word_OE1_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Divide_Word
#undef MY_PREFIX
#define MY_PREFIX Divide_Word_OE1_Rc1
#line 2745 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Divide Word\n"));
#line 28471 "icache.c"

  /* validate: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0003fe) != 0x7c0003d6)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.XO.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.XO.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.XO.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XO.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 2745 "../../sim/ppc/powerpc.igen"
    unsigned const OE ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 2745 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 28503 "icache.c"

#define insn cache_entry->crack.XO.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Divide_Word_OE1_Rc1;

    /* semantic routine */
#line 2745 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Divide Word\n"));
#line 28513 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Divide_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2745 "../../sim/ppc/powerpc.igen"
    {
      int64_t dividend = (int32_t)(*rA);
      int64_t divisor = (int32_t)(*rB);
      if (divisor == 0 /* nb 0x8000..0 is sign extended */
          || (dividend == 0x80000000 && divisor == -1)) {
        if (OE)
          XER |= (xer_overflow | xer_summary_overflow);
        CR0_COMPARE(0, 0, Rc);
      }
      else {
        int64_t quotent = dividend / divisor;
        *rT = quotent;
        CR0_COMPARE((signed_word)quotent, 0, Rc);
      }
      PPC_INSN_INT(RT_BITMASK, RA_BITMASK | RB_BITMASK, Rc);
    }
#line 28539 "icache.c"
    return nia;
    /* extraction: 0.31,6.RT,11.RA,16.RB,21.OE,22.491,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28543 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Data_Cache_Block_set_to_Zero(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Data_Cache_Block_set_to_Zero
#undef MY_PREFIX
#define MY_PREFIX Data_Cache_Block_set_to_Zero
#line 4803 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Data Cache Block set to Zero\n"));
#line 28562 "icache.c"

  /* validate: 0.31,6./,11.RA,16.RB,21.1014,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xffe007ff) != 0x7c0007ec)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.1014,31./ #define get-values-from-icache|put-values-in-icache */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.X.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.X.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 46 "../../sim/ppc/powerpc.igen"
    unsigned const RB ATTRIBUTE_UNUSED = cache_entry->crack.X.RB = EXTRACTED32(instruction, 16, 20); /* in icache - extraction */
#line 47 "../../sim/ppc/powerpc.igen"
    signed_word * const rB ATTRIBUTE_UNUSED = cache_entry->crack.X.rB = (cpu_registers(processor)->gpr + RB); /* in icache - extraction */
#line 48 "../../sim/ppc/powerpc.igen"
    uint32_t const RB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.RB_BITMASK = (1 << RB); /* in icache - extraction */
#line 28584 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Data_Cache_Block_set_to_Zero;

    /* semantic routine */
#line 4803 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Data Cache Block set to Zero\n"));
#line 28594 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Data_Cache_Block_set_to_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4803 "../../sim/ppc/powerpc.igen"
    {
      TRACE(trace_tbd,("Data Cache Block set to Zero\n"));
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RB_BITMASK, 0/*Rc*/);
    }
#line 28608 "icache.c"
    return nia;
    /* extraction: 0.31,6./,11.RA,16.RB,21.1014,31./ #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28612 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero
#line 1885 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Word and Zero\n"));
#line 28631 "icache.c"

  /* validate: 0.32,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x80000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.32,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 28655 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero;

    /* semantic routine */
#line 1885 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Word and Zero\n"));
#line 28665 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1885 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(unsigned, EA, 4);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 28684 "icache.c"
    return nia;
    /* extraction: 0.32,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28688 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Word_and_Zero_with_Update(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Word_and_Zero_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Word_and_Zero_with_Update
#line 1911 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Word and Zero with Update\n"));
#line 28707 "icache.c"

  /* validate: 0.33,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x84000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.33,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 28731 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Word_and_Zero_with_Update;

    /* semantic routine */
#line 1911 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Word and Zero with Update\n"));
#line 28741 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Word_and_Zero_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1911 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(unsigned, EA, 4);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 28761 "icache.c"
    return nia;
    /* extraction: 0.33,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28765 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero
#line 1722 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero\n"));
#line 28784 "icache.c"

  /* validate: 0.34,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x88000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.34,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 28808 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero;

    /* semantic routine */
#line 1722 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero\n"));
#line 28818 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1722 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(unsigned, EA, 1);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 28837 "icache.c"
    return nia;
    /* extraction: 0.34,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28841 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Byte_and_Zero_with_Update(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Byte_and_Zero_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Byte_and_Zero_with_Update
#line 1749 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Byte and Zero with Update\n"));
#line 28860 "icache.c"

  /* validate: 0.35,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x8c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.35,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 28884 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Byte_and_Zero_with_Update;

    /* semantic routine */
#line 1749 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Byte and Zero with Update\n"));
#line 28894 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Byte_and_Zero_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1749 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(unsigned, EA, 1);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 28914 "icache.c"
    return nia;
    /* extraction: 0.35,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28918 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word(cpu *processor,
                  instruction_word instruction,
                  unsigned_word cia,
                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word
#undef MY_PREFIX
#define MY_PREFIX Store_Word
#line 2112 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Word\n"));
#line 28937 "icache.c"

  /* validate: 0.36,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x90000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.36,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 28961 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word;

    /* semantic routine */
#line 2112 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Word\n"));
#line 28971 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2112 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 4, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RS_BITMASK, 0);
    }
#line 28990 "icache.c"
    return nia;
    /* extraction: 0.36,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 28994 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Word_with_Update(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Word_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_Word_with_Update
#line 2138 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Word with Update\n"));
#line 29013 "icache.c"

  /* validate: 0.37,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x94000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.37,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 29037 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Word_with_Update;

    /* semantic routine */
#line 2138 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Word with Update\n"));
#line 29047 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Word_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2138 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 4, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RS_BITMASK, 0);
    }
#line 29067 "icache.c"
    return nia;
    /* extraction: 0.37,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29071 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte(cpu *processor,
                  instruction_word instruction,
                  unsigned_word cia,
                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte
#undef MY_PREFIX
#define MY_PREFIX Store_Byte
#line 2004 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Byte\n"));
#line 29090 "icache.c"

  /* validate: 0.38,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x98000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.38,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 29114 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte;

    /* semantic routine */
#line 2004 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Byte\n"));
#line 29124 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2004 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 1, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RS_BITMASK, 0);
    }
#line 29143 "icache.c"
    return nia;
    /* extraction: 0.38,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29147 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Byte_with_Update(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Byte_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_Byte_with_Update
#line 2030 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Byte with Update\n"));
#line 29166 "icache.c"

  /* validate: 0.39,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0x9c000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.39,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 29190 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Byte_with_Update;

    /* semantic routine */
#line 2030 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Byte with Update\n"));
#line 29200 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Byte_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2030 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 1, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RS_BITMASK, 0);
    }
#line 29220 "icache.c"
    return nia;
    /* extraction: 0.39,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29224 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero(cpu *processor,
                              instruction_word instruction,
                              unsigned_word cia,
                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero
#line 1777 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero\n"));
#line 29243 "icache.c"

  /* validate: 0.40,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xa0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.40,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 29267 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero;

    /* semantic routine */
#line 1777 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero\n"));
#line 29277 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1777 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(unsigned, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 29296 "icache.c"
    return nia;
    /* extraction: 0.40,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29300 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_and_Zero_with_Update(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_and_Zero_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_and_Zero_with_Update
#line 1803 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Halfword and Zero with Update\n"));
#line 29319 "icache.c"

  /* validate: 0.41,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xa4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.41,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 29343 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_and_Zero_with_Update;

    /* semantic routine */
#line 1803 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Halfword and Zero with Update\n"));
#line 29353 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_and_Zero_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1803 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(unsigned, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 29373 "icache.c"
    return nia;
    /* extraction: 0.41,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29377 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic
#line 1831 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic\n"));
#line 29396 "icache.c"

  /* validate: 0.42,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xa8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.42,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 29420 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic;

    /* semantic routine */
#line 1831 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic\n"));
#line 29430 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1831 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *rT = MEM(signed, EA, 2);
      PPC_INSN_INT(RT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 29449 "icache.c"
    return nia;
    /* extraction: 0.42,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29453 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Halfword_Algebraic_with_Update(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Halfword_Algebraic_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_Halfword_Algebraic_with_Update
#line 1857 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Halfword Algebraic with Update\n"));
#line 29472 "icache.c"

  /* validate: 0.43,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xac000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.43,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 29496 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Halfword_Algebraic_with_Update;

    /* semantic routine */
#line 1857 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Halfword Algebraic with Update\n"));
#line 29506 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Halfword_Algebraic_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 1857 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0 || RA == RT)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *rT = MEM(signed, EA, 2);
      *rA = EA;
      PPC_INSN_INT(RT_BITMASK | RA_BITMASK, RA_BITMASK, 0);
    }
#line 29526 "icache.c"
    return nia;
    /* extraction: 0.43,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29530 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word(cpu *processor,
                       instruction_word instruction,
                       unsigned_word cia,
                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word
#line 2058 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Half Word\n"));
#line 29549 "icache.c"

  /* validate: 0.44,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xb0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.44,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 29573 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word;

    /* semantic routine */
#line 2058 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Half Word\n"));
#line 29583 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2058 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 2, *rS);
      PPC_INSN_INT(0, (RA_BITMASK & ~1) | RS_BITMASK, 0);
    }
#line 29602 "icache.c"
    return nia;
    /* extraction: 0.44,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29606 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Half_Word_with_Update(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Half_Word_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_Half_Word_with_Update
#line 2084 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Half Word with Update\n"));
#line 29625 "icache.c"

  /* validate: 0.45,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xb4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.45,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 29649 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Half_Word_with_Update;

    /* semantic routine */
#line 2084 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Half Word with Update\n"));
#line 29659 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Half_Word_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2084 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 2, *rS);
      *rA = EA;
      PPC_INSN_INT(RA_BITMASK, RA_BITMASK | RS_BITMASK, 0);
    }
#line 29679 "icache.c"
    return nia;
    /* extraction: 0.45,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29683 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_Multiple_Word(cpu *processor,
                          instruction_word instruction,
                          unsigned_word cia,
                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_Multiple_Word
#undef MY_PREFIX
#define MY_PREFIX Load_Multiple_Word
#line 2255 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Multiple Word\n"));
#line 29702 "icache.c"

  /* validate: 0.46,6.RT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xb8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.46,6.RT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 40 "../../sim/ppc/powerpc.igen"
    unsigned const RT ATTRIBUTE_UNUSED = cache_entry->crack.D.RT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 41 "../../sim/ppc/powerpc.igen"
    signed_word * const rT ATTRIBUTE_UNUSED = cache_entry->crack.D.rT = (cpu_registers(processor)->gpr + RT); /* in icache - extraction */
#line 42 "../../sim/ppc/powerpc.igen"
    uint32_t const RT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RT_BITMASK = (1 << RT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 29726 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_Multiple_Word;

    /* semantic routine */
#line 2255 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Multiple Word\n"));
#line 29736 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_Multiple_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2255 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      unsigned_word b;
      int r;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      r = RT;
      if (RA >= r)
        program_interrupt(processor, cia,
                        illegal_instruction_program_interrupt);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT || (EA % 4 != 0))
        alignment_interrupt(processor, cia, EA);
      while (r <= 31) {
        GPR(r) = MEM(unsigned, EA, 4);
        r = r + 1;
        EA = EA + 4;
      }
    }
#line 29765 "icache.c"
    return nia;
    /* extraction: 0.46,6.RT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29769 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_Multiple_Word(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_Multiple_Word
#undef MY_PREFIX
#define MY_PREFIX Store_Multiple_Word
#line 2274 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Multiple Word\n"));
#line 29788 "icache.c"

  /* validate: 0.47,6.RS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xbc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.47,6.RS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#line 43 "../../sim/ppc/powerpc.igen"
    unsigned const RS ATTRIBUTE_UNUSED = cache_entry->crack.D.RS = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 44 "../../sim/ppc/powerpc.igen"
    signed_word * const rS ATTRIBUTE_UNUSED = cache_entry->crack.D.rS = (cpu_registers(processor)->gpr + RS); /* in icache - extraction */
#line 45 "../../sim/ppc/powerpc.igen"
    uint32_t const RS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RS_BITMASK = (1 << RS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 29812 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_Multiple_Word;

    /* semantic routine */
#line 2274 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Multiple Word\n"));
#line 29822 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_Multiple_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 2274 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      unsigned_word b;
      int r;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      if (CURRENT_ALIGNMENT == STRICT_ALIGNMENT
          || (EA % 4 != 0))
        alignment_interrupt(processor, cia, EA);
      r = RS;
      while (r <= 31) {
        STORE(EA, 4, GPR(r));
        r = r + 1;
        EA = EA + 4;
      }
    }
#line 29849 "icache.c"
    return nia;
    /* extraction: 0.47,6.RS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef RA_is_0
#line 29853 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single
#line 3464 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single\n"));
#line 29872 "icache.c"

  /* validate: 0.48,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.48,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 29903 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single;

    /* semantic routine */
#line 3464 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single\n"));
#line 29913 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3464 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 29932 "icache.c"
    return nia;
    /* extraction: 0.48,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 29937 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Single_with_Update(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Single_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Single_with_Update
#line 3490 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Single with Update\n"));
#line 29956 "icache.c"

  /* validate: 0.49,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.49,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 29987 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Single_with_Update;

    /* semantic routine */
#line 3490 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Single with Update\n"));
#line 29997 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Single_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3490 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *frT = DOUBLE(MEM(unsigned, EA, 4));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 30017 "icache.c"
    return nia;
    /* extraction: 0.49,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 30022 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double
#line 3518 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double\n"));
#line 30041 "icache.c"

  /* validate: 0.50,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xc8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.50,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 30072 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double;

    /* semantic routine */
#line 3518 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double\n"));
#line 30082 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3518 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      *frT = MEM(unsigned, EA, 8);
      PPC_INSN_INT_FLOAT(0, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 30101 "icache.c"
    return nia;
    /* extraction: 0.50,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 30106 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Load_FloatingPoint_Double_with_Update(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Load_FloatingPoint_Double_with_Update
#undef MY_PREFIX
#define MY_PREFIX Load_FloatingPoint_Double_with_Update
#line 3544 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Load Floating-Point Double with Update\n"));
#line 30125 "icache.c"

  /* validate: 0.51,6.FRT,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xcc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.51,6.FRT,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.D.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 30156 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Load_FloatingPoint_Double_with_Update;

    /* semantic routine */
#line 3544 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Load Floating-Point Double with Update\n"));
#line 30166 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Load_FloatingPoint_Double_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3544 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      *frT = MEM(unsigned, EA, 8);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, FRT_BITMASK, (RA_BITMASK & ~1), 0);
    }
#line 30186 "icache.c"
    return nia;
    /* extraction: 0.51,6.FRT,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef RA_is_0
#line 30191 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single
#line 3577 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single\n"));
#line 30210 "icache.c"

  /* validate: 0.52,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xd0000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.52,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "../../sim/ppc/powerpc.igen"
    uint64_t * const frS ATTRIBUTE_UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "../../sim/ppc/powerpc.igen"
    uint32_t const FRS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 30241 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single;

    /* semantic routine */
#line 3577 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single\n"));
#line 30251 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3577 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 4, SINGLE(*frS));
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30270 "icache.c"
    return nia;
    /* extraction: 0.52,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30275 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Single_with_Update(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Single_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Single_with_Update
#line 3603 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Single with Update\n"));
#line 30294 "icache.c"

  /* validate: 0.53,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xd4000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.53,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "../../sim/ppc/powerpc.igen"
    uint64_t * const frS ATTRIBUTE_UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "../../sim/ppc/powerpc.igen"
    uint32_t const FRS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 30325 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Single_with_Update;

    /* semantic routine */
#line 3603 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Single with Update\n"));
#line 30335 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Single_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3603 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 4, SINGLE(*frS));
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30355 "icache.c"
    return nia;
    /* extraction: 0.53,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30360 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double
#line 3631 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double\n"));
#line 30379 "icache.c"

  /* validate: 0.54,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xd8000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.54,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "../../sim/ppc/powerpc.igen"
    uint64_t * const frS ATTRIBUTE_UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "../../sim/ppc/powerpc.igen"
    uint32_t const FRS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 30410 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double;

    /* semantic routine */
#line 3631 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double\n"));
#line 30420 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3631 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word b;
      unsigned_word EA;
      if (RA_is_0) b = 0;
      else         b = *rA;
      EA = b + EXTS(D);
      STORE(EA, 8, *frS);
      PPC_INSN_INT_FLOAT(0, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30439 "icache.c"
    return nia;
    /* extraction: 0.54,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30444 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Store_FloatingPoint_Double_with_Update(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Store_FloatingPoint_Double_with_Update
#undef MY_PREFIX
#define MY_PREFIX Store_FloatingPoint_Double_with_Update
#line 3669 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Store Floating-Point Double with Update\n"));
#line 30463 "icache.c"

  /* validate: 0.55,6.FRS,11.RA,16.D */
  if (WITH_RESERVED_BITS && (instruction & 0xfc000000) != 0xdc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.55,6.FRS,11.RA,16.D #define get-values-from-icache|put-values-in-icache */
#define FRS EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 59 "../../sim/ppc/powerpc.igen"
    uint64_t * const frS ATTRIBUTE_UNUSED = cache_entry->crack.D.frS = (cpu_registers(processor)->fpr + FRS); /* in icache - extraction */
#line 60 "../../sim/ppc/powerpc.igen"
    uint32_t const FRS_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.FRS_BITMASK = (1 << FRS); /* in icache - extraction */
#line 36 "../../sim/ppc/powerpc.igen"
    unsigned const RA ATTRIBUTE_UNUSED = cache_entry->crack.D.RA = EXTRACTED32(instruction, 11, 15); /* in icache - extraction */
#line 37 "../../sim/ppc/powerpc.igen"
    signed_word * const rA ATTRIBUTE_UNUSED = cache_entry->crack.D.rA = (cpu_registers(processor)->gpr + RA); /* in icache - extraction */
#line 38 "../../sim/ppc/powerpc.igen"
    uint32_t const RA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.D.RA_BITMASK = (1 << RA); /* in icache - extraction */
#define RA_is_0 (RA == 0) /* semantic compute - extraction */
#line 79 "../../sim/ppc/powerpc.igen"
    unsigned_word const EXTS_D ATTRIBUTE_UNUSED = cache_entry->crack.D.EXTS_D = ((signed_word)(int16_t)(instruction)); /* in icache - extraction */
#line 30494 "icache.c"

#define insn cache_entry->crack.D.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Store_FloatingPoint_Double_with_Update;

    /* semantic routine */
#line 3669 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Store Floating-Point Double with Update\n"));
#line 30504 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Store_FloatingPoint_Double_with_Update, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3669 "../../sim/ppc/powerpc.igen"
    {
      unsigned_word EA;
      if (RA_is_0)
        program_interrupt(processor, cia,
                          illegal_instruction_program_interrupt);
      EA = *rA + EXTS(D);
      STORE(EA, 8, *frS);
      *rA = EA;
      PPC_INSN_INT_FLOAT(RA_BITMASK, 0, (RA_BITMASK & ~1), FRS_BITMASK);
    }
#line 30524 "icache.c"
    return nia;
    /* extraction: 0.55,6.FRS,11.RA,16.D #undef get-values-from-icache|put-values-in-icache */
#undef FRS
#undef RA_is_0
#line 30529 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Single_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Single_Rc0
#line 3932 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Divide Single\n"));
#line 30548 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3932 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 30582 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Single_Rc0;

    /* semantic routine */
#line 3932 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Divide Single\n"));
#line 30592 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3932 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       1 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       1 /*single?*/);
      }
      else {
        /*HACK!*/
        float s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30631 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30637 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Single_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Single_Rc1
#line 3932 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Divide Single\n"));
#line 30656 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3932 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 30690 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Single_Rc1;

    /* semantic routine */
#line 3932 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Divide Single\n"));
#line 30700 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3932 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       1 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       1 /*single?*/);
      }
      else {
        /*HACK!*/
        float s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30739 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30745 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Single_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Single_Rc0
#line 3821 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Subtract Single\n"));
#line 30764 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3821 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 30798 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Single_Rc0;

    /* semantic routine */
#line 3821 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Subtract Single\n"));
#line 30808 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3821 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30840 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30846 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Single_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Single_Rc1
#line 3821 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Subtract Single\n"));
#line 30865 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3821 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 30899 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Single_Rc1;

    /* semantic routine */
#line 3821 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Subtract Single\n"));
#line 30909 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3821 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 30941 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 30947 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Single_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Single_Rc0
#line 3769 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Add Single\n"));
#line 30966 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3769 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 31000 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Single_Rc0;

    /* semantic routine */
#line 3769 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Add Single\n"));
#line 31010 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3769 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 31042 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 31048 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Single_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Single_Rc1
#line 3769 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Add Single\n"));
#line 31067 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xec00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3769 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 31101 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Single_Rc1;

    /* semantic routine */
#line 3769 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Add Single\n"));
#line 31111 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3769 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 31143 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 31149 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Single_Rc0(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Single_Rc0
#line 4726 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Square Root Single\n"));
#line 31168 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4726 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 31197 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Single_Rc0;

    /* semantic routine */
#line 4726 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Square Root Single\n"));
#line 31207 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4726 "../../sim/ppc/powerpc.igen"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31220 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31225 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Single_Rc1(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Single_Rc1
#line 4726 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Square Root Single\n"));
#line 31244 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4726 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 31273 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Single_Rc1;

    /* semantic routine */
#line 4726 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Square Root Single\n"));
#line 31283 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4726 "../../sim/ppc/powerpc.igen"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31296 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31301 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Estimate_Single_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Estimate_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Estimate_Single_Rc0
#line 4729 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Estimate Single\n"));
#line 31320 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4729 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 31349 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Estimate_Single_Rc0;

    /* semantic routine */
#line 4729 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Estimate Single\n"));
#line 31359 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Estimate_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "../../sim/ppc/powerpc.igen"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31372 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31377 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Estimate_Single_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Estimate_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Estimate_Single_Rc1
#line 4729 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Estimate Single\n"));
#line 31396 "icache.c"

  /* validate: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xec000030)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4729 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 31425 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Estimate_Single_Rc1;

    /* semantic routine */
#line 4729 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Estimate Single\n"));
#line 31435 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Estimate_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4729 "../../sim/ppc/powerpc.igen"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 31448 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11./,16.FRB,21./,26.24,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 31453 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_Single_Rc0(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_Single_Rc0
#line 3873 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply Single\n"));
#line 31472 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xec000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3873 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 31506 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_Single_Rc0;

    /* semantic routine */
#line 3873 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply Single\n"));
#line 31516 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3873 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 31548 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 31554 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_Single_Rc1(cpu *processor,
                                    instruction_word instruction,
                                    unsigned_word cia,
                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_Single_Rc1
#line 3873 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply Single\n"));
#line 31573 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xec000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3873 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 31607 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_Single_Rc1;

    /* semantic routine */
#line 3873 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply Single\n"));
#line 31617 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3873 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      1); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 31649 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 31655 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_Single_Rc0(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_Single_Rc0
#line 4106 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract Single\n"));
#line 31674 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4106 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 31713 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_Single_Rc0;

    /* semantic routine */
#line 4106 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract Single\n"));
#line 31723 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4106 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product - *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 31776 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 31783 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_Single_Rc1(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_Single_Rc1
#line 4106 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract Single\n"));
#line 31802 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4106 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 31841 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_Single_Rc1;

    /* semantic routine */
#line 4106 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract Single\n"));
#line 31851 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4106 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product - *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 31904 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 31911 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_Single_Rc0(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_Single_Rc0
#line 4012 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add Single\n"));
#line 31930 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4012 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 31969 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_Single_Rc0;

    /* semantic routine */
#line 4012 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add Single\n"));
#line 31979 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4012 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product + *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32032 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32039 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_Single_Rc1(cpu *processor,
                                       instruction_word instruction,
                                       unsigned_word cia,
                                       idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_Single_Rc1
#line 4012 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add Single\n"));
#line 32058 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4012 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 32097 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_Single_Rc1;

    /* semantic routine */
#line 4012 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add Single\n"));
#line 32107 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4012 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = product + *(double*)frB;
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32160 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32167 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_Single_Rc0(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_Single_Rc0
#line 4294 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract Single\n"));
#line 32186 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4294 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 32225 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_Single_Rc0;

    /* semantic routine */
#line 4294 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract Single\n"));
#line 32235 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4294 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product - *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32288 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32295 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_Single_Rc1(cpu *processor,
                                                     instruction_word instruction,
                                                     unsigned_word cia,
                                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_Single_Rc1
#line 4294 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract Single\n"));
#line 32314 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4294 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 32353 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_Single_Rc1;

    /* semantic routine */
#line 4294 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract Single\n"));
#line 32363 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4294 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product - *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32416 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32423 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_Single_Rc0(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_Single_Rc0
#line 4200 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add Single\n"));
#line 32442 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4200 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 32481 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_Single_Rc0;

    /* semantic routine */
#line 4200 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add Single\n"));
#line 32491 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4200 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product + *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32544 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32551 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_Single_Rc1(cpu *processor,
                                                instruction_word instruction,
                                                unsigned_word cia,
                                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd_Single
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_Single_Rc1
#line 4200 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add Single\n"));
#line 32570 "icache.c"

  /* validate: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xec00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC EXTRACTED32(instruction, 21, 25) /* icache scratch - extraction */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4200 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 32609 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_Single_Rc1;

    /* semantic routine */
#line 4200 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add Single\n"));
#line 32619 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd_Single, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4200 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      float product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               1, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               1, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        float s = -(product + *(double*)frB);
        *(double*)frT = (double)s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 32672 "icache.c"
    return nia;
    /* extraction: 0.59,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 32679 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Compare_Unordered(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Compare_Unordered
#undef MY_PREFIX
#define MY_PREFIX Floating_Compare_Unordered
#line 4621 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Compare Unordered\n"));
#line 32698 "icache.c"

  /* validate: 0.63,6.BF,9./,11.FRA,16.FRB,21.0,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc6007ff) != 0xfc000000)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.0,31./ #define get-values-from-icache|put-values-in-icache */
#line 67 "../../sim/ppc/powerpc.igen"
    unsigned const BF ATTRIBUTE_UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "../../sim/ppc/powerpc.igen"
    uint32_t const BF_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.X.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 32729 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Compare_Unordered;

    /* semantic routine */
#line 4621 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Compare Unordered\n"));
#line 32739 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Compare_Unordered, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4621 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      unsigned c;
      if (is_NaN(*frA, 0) || is_NaN(*frB, 0))
        c = cr_i_summary_overflow; /* 0b0001 - (FRA) ? (FRB) */
      else if (is_less_than(frA, frB))
        c = cr_i_negative; /* 0b1000 - (FRA) < (FRB) */
      else if (is_greater_than(frA, frB))
        c = cr_i_positive; /* 0b0100 - (FRA) > (FRB) */
      else
        c = cr_i_zero; /* 0b0010 - (FRA) = (FRB) */
      FPSCR_SET_FPCC(c);
      CR_SET(BF, c); /* CR[4*BF..4*BF+3] = c */
      if (is_SNaN(*frA, 0) || is_SNaN(*frB, 0))
        FPSCR_OR_VX(fpscr_vxsnan);
      FPSCR_END(0);
      PPC_INSN_FLOAT_CR(0, FRA_BITMASK | FRB_BITMASK, BF_BITMASK);
    }
#line 32767 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.0,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRA
#undef FRB
#line 32772 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Round_to_SinglePrecision_Rc0(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Round_to_SinglePrecision
#undef MY_PREFIX
#define MY_PREFIX Floating_Round_to_SinglePrecision_Rc0
#line 4346 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Round to Single-Precision\n"));
#line 32791 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000018)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4346 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 32820 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Round_to_SinglePrecision_Rc0;

    /* semantic routine */
#line 4346 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Round to Single-Precision\n"));
#line 32830 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Round_to_SinglePrecision, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4346 "../../sim/ppc/powerpc.igen"
    {
      int sign;
      int exp;
      uint64_t frac_grx;
      /***/
        /* split off cases for what to do */
        if (EXTRACTED64(*frB, 1, 11) < 897
            && EXTRACTED64(*frB, 1, 63) > 0) {
            if ((FPSCR & fpscr_ue) == 0) GOTO(Disabled_Exponent_Underflow);
            if ((FPSCR & fpscr_ue) != 0) GOTO(Enabled_Exponent_Underflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 1150
            && EXTRACTED64(*frB, 1, 11) < 2047) {
            if ((FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
            if ((FPSCR & fpscr_oe) != 0) GOTO(Enabled_Exponent_Overflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 896
            && EXTRACTED64(*frB, 1, 11) < 1151) GOTO(Normal_Operand);
        if (EXTRACTED64(*frB, 1, 63) == 0) GOTO(Zero_Operand);
        if (EXTRACTED64(*frB, 1, 11) == 2047) {
          if (EXTRACTED64(*frB, 12, 63) == 0) GOTO(Infinity_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 1) GOTO(QNaN_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 0
              && EXTRACTED64(*frB, 13, 63) > 0) GOTO(SNaN_Operand);
        }
      /**/
      LABEL(Disabled_Exponent_Underflow):
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
          /* G|R|X == zero from above */
          while (exp < -126) {
            exp = exp + 1;
            frac_grx = (INSERTED64(EXTRACTED64(frac_grx, 0, 54), 1, 55)
                        | MASKED64(frac_grx, 55, 55));
          }
        FPSCR_SET_UX(EXTRACTED64(frac_grx, 24, 55) > 0);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (EXTRACTED64(frac_grx, 0, 52) == 0) {
          *frT = INSERTED64(sign, 0, 0);
          if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
          if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        }
        if (EXTRACTED64(frac_grx, 0, 52) > 0) {
          if (EXTRACTED64(frac_grx, 0, 0) == 1) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
          if (EXTRACTED64(frac_grx, 0, 0) == 0) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_denormalized_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_denormalized_number);
          }
          /*Normalize_Operand:*/
            while (EXTRACTED64(frac_grx, 0, 0) == 0) {
              exp = exp - 1;
              frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1,  52), 0, 51);
            }
          *frT = (INSERTED64(sign, 0, 0)
                  | INSERTED64(exp + 1023, 1, 11)
                  | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        }
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Underflow):
        FPSCR_SET_UX(1);
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = (BIT64(0) |
                      INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52));
        }
        /*Normalize_Operand:*/
          while (EXTRACTED64(frac_grx, 0, 0) == 0) {
            exp = exp - 1;
            frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1, 52), 0, 51);
          }
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        exp = exp + 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Disabled_Exponent_Overflow):
        FPSCR_SET_OX(1);
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_to_nearest) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_zero) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_pos_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_neg_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        /* FPSCR[FR] <- undefined */
        FPSCR_SET_FI(1);
        FPSCR_SET_XX(1);
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Overflow):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
      /**/
      LABEL(Enabled_Overflow):
        FPSCR_SET_OX(1);
        exp = exp - 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Zero_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Infinity_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(QNaN_Operand):
        *frT = INSERTED64(EXTRACTED64(*frB, 0, 34), 0, 34);
        FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(SNaN_Operand):
        FPSCR_OR_VX(fpscr_vxsnan);
        if ((FPSCR & fpscr_ve) == 0) {
          *frT = (MASKED64(*frB, 0, 11)
                  | BIT64(12)
                  | MASKED64(*frB, 13, 34));
          FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        }
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Normal_Operand):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (exp > 127 && (FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
        if (exp > 127 && (FPSCR & fpscr_oe) != 0) GOTO(Enabled_Overflow);
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Done):
        PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33054 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33059 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Round_to_SinglePrecision_Rc1(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Round_to_SinglePrecision
#undef MY_PREFIX
#define MY_PREFIX Floating_Round_to_SinglePrecision_Rc1
#line 4346 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Round to Single-Precision\n"));
#line 33078 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000018)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4346 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 33107 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Round_to_SinglePrecision_Rc1;

    /* semantic routine */
#line 4346 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Round to Single-Precision\n"));
#line 33117 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Round_to_SinglePrecision, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4346 "../../sim/ppc/powerpc.igen"
    {
      int sign;
      int exp;
      uint64_t frac_grx;
      /***/
        /* split off cases for what to do */
        if (EXTRACTED64(*frB, 1, 11) < 897
            && EXTRACTED64(*frB, 1, 63) > 0) {
            if ((FPSCR & fpscr_ue) == 0) GOTO(Disabled_Exponent_Underflow);
            if ((FPSCR & fpscr_ue) != 0) GOTO(Enabled_Exponent_Underflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 1150
            && EXTRACTED64(*frB, 1, 11) < 2047) {
            if ((FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
            if ((FPSCR & fpscr_oe) != 0) GOTO(Enabled_Exponent_Overflow);
        }
        if (EXTRACTED64(*frB, 1, 11) > 896
            && EXTRACTED64(*frB, 1, 11) < 1151) GOTO(Normal_Operand);
        if (EXTRACTED64(*frB, 1, 63) == 0) GOTO(Zero_Operand);
        if (EXTRACTED64(*frB, 1, 11) == 2047) {
          if (EXTRACTED64(*frB, 12, 63) == 0) GOTO(Infinity_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 1) GOTO(QNaN_Operand);
          if (EXTRACTED64(*frB, 12, 12) == 0
              && EXTRACTED64(*frB, 13, 63) > 0) GOTO(SNaN_Operand);
        }
      /**/
      LABEL(Disabled_Exponent_Underflow):
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
          /* G|R|X == zero from above */
          while (exp < -126) {
            exp = exp + 1;
            frac_grx = (INSERTED64(EXTRACTED64(frac_grx, 0, 54), 1, 55)
                        | MASKED64(frac_grx, 55, 55));
          }
        FPSCR_SET_UX(EXTRACTED64(frac_grx, 24, 55) > 0);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (EXTRACTED64(frac_grx, 0, 52) == 0) {
          *frT = INSERTED64(sign, 0, 0);
          if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
          if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        }
        if (EXTRACTED64(frac_grx, 0, 52) > 0) {
          if (EXTRACTED64(frac_grx, 0, 0) == 1) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
          if (EXTRACTED64(frac_grx, 0, 0) == 0) {
            if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_denormalized_number);
            if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_denormalized_number);
          }
          /*Normalize_Operand:*/
            while (EXTRACTED64(frac_grx, 0, 0) == 0) {
              exp = exp - 1;
              frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1,  52), 0, 51);
            }
          *frT = (INSERTED64(sign, 0, 0)
                  | INSERTED64(exp + 1023, 1, 11)
                  | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        }
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Underflow):
        FPSCR_SET_UX(1);
        sign = EXTRACTED64(*frB, 0, 0);
        if (EXTRACTED64(*frB, 1, 11) == 0) {
          exp = -1022;
          frac_grx = INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        }
        if (EXTRACTED64(*frB, 1, 11) > 0) {
          exp = EXTRACTED64(*frB, 1, 11) - 1023;
          frac_grx = (BIT64(0) |
                      INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52));
        }
        /*Normalize_Operand:*/
          while (EXTRACTED64(frac_grx, 0, 0) == 0) {
            exp = exp - 1;
            frac_grx = INSERTED64(EXTRACTED64(frac_grx, 1, 52), 0, 51);
          }
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        exp = exp + 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Disabled_Exponent_Overflow):
        FPSCR_SET_OX(1);
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_to_nearest) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_zero) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_pos_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x7FF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xC7EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
          }
        }
        if ((FPSCR & fpscr_rn) == fpscr_rn_round_towards_neg_infinity) {
          if (EXTRACTED64(*frB, 0, 0) == 0) {
            *frT = INSERTED64(0x47EFFFFF, 0, 31) | 0xE0000000;
            FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
          }
          if (EXTRACTED64(*frB, 0, 0) == 1) {
            *frT = INSERTED64(0xFFF00000, 0, 31) | 0x00000000;
            FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
          }
        }
        /* FPSCR[FR] <- undefined */
        FPSCR_SET_FI(1);
        FPSCR_SET_XX(1);
        GOTO(Done);
      /**/
      LABEL(Enabled_Exponent_Overflow):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
      /**/
      LABEL(Enabled_Overflow):
        FPSCR_SET_OX(1);
        exp = exp - 192;
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Zero_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_zero);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_zero);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Infinity_Operand):
        *frT = *frB;
        if (EXTRACTED64(*frB, 0, 0) == 0) FPSCR_SET_FPRF(fpscr_rf_pos_infinity);
        if (EXTRACTED64(*frB, 0, 0) == 1) FPSCR_SET_FPRF(fpscr_rf_neg_infinity);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(QNaN_Operand):
        *frT = INSERTED64(EXTRACTED64(*frB, 0, 34), 0, 34);
        FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(SNaN_Operand):
        FPSCR_OR_VX(fpscr_vxsnan);
        if ((FPSCR & fpscr_ve) == 0) {
          *frT = (MASKED64(*frB, 0, 11)
                  | BIT64(12)
                  | MASKED64(*frB, 13, 34));
          FPSCR_SET_FPRF(fpscr_rf_quiet_nan);
        }
        FPSCR_SET_FR(0);
        FPSCR_SET_FI(0);
        GOTO(Done);
      /**/
      LABEL(Normal_Operand):
        sign = EXTRACTED64(*frB, 0, 0);
        exp = EXTRACTED64(*frB, 1, 11) - 1023;
        frac_grx = BIT64(0) | INSERTED64(EXTRACTED64(*frB, 12, 63), 1, 52);
        Round_Single(processor, sign, &exp, &frac_grx);
        FPSCR_SET_XX(FPSCR & fpscr_fi);
        if (exp > 127 && (FPSCR & fpscr_oe) == 0) GOTO(Disabled_Exponent_Overflow);
        if (exp > 127 && (FPSCR & fpscr_oe) != 0) GOTO(Enabled_Overflow);
        *frT = (INSERTED64(sign, 0, 0)
                | INSERTED64(exp + 1023, 1, 11)
                | INSERTED64(EXTRACTED64(frac_grx, 1, 52), 12, 63));
        if (sign == 0) FPSCR_SET_FPRF(fpscr_rf_pos_normal_number);
        if (sign == 1) FPSCR_SET_FPRF(fpscr_rf_neg_normal_number);
        GOTO(Done);
      /**/
      LABEL(Done):
        PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33341 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.12,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33346 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_Rc0(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_Rc0
#line 4567 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word\n"));
#line 33365 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4567 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 33394 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_Rc0;

    /* semantic routine */
#line 4567 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word\n"));
#line 33404 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4567 "../../sim/ppc/powerpc.igen"
    {
      floating_point_assist_interrupt(processor, cia);
    }
#line 33417 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33422 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_Rc1(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_Rc1
#line 4567 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word\n"));
#line 33441 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4567 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 33470 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_Rc1;

    /* semantic routine */
#line 4567 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word\n"));
#line 33480 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4567 "../../sim/ppc/powerpc.igen"
    {
      floating_point_assist_interrupt(processor, cia);
    }
#line 33493 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.14,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33498 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc0(cpu *processor,
                                                                    instruction_word instruction,
                                                                    unsigned_word cia,
                                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc0
#line 4574 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word with round towards Zero\n"));
#line 33517 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4574 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 33546 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc0;

    /* semantic routine */
#line 4574 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word with round towards Zero\n"));
#line 33556 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4574 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      convert_to_integer(processor, cia,
                         frT, *frB,
                         fpscr_rn_round_towards_zero, 32);
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33574 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33579 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc1(cpu *processor,
                                                                    instruction_word instruction,
                                                                    unsigned_word cia,
                                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero
#undef MY_PREFIX
#define MY_PREFIX Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc1
#line 4574 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Convert To Integer Word with round towards Zero\n"));
#line 33598 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00001e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4574 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 33627 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Convert_To_Integer_Word_with_round_towards_Zero_Rc1;

    /* semantic routine */
#line 4574 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Convert To Integer Word with round towards Zero\n"));
#line 33637 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Convert_To_Integer_Word_with_round_towards_Zero, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4574 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      convert_to_integer(processor, cia,
                         frT, *frB,
                         fpscr_rn_round_towards_zero, 32);
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 33655 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.15,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 33660 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Rc0
#line 3899 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Divide\n"));
#line 33679 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3899 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 33713 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Rc0;

    /* semantic routine */
#line 3899 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Divide\n"));
#line 33723 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3899 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       0 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       0 /*single?*/);
      }
      else {
        /*HACK!*/
        double s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 33762 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 33768 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Divide_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Divide
#undef MY_PREFIX
#define MY_PREFIX Floating_Divide_Rc1
#line 3899 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Divide\n"));
#line 33787 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000024)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3899 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 33821 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Divide_Rc1;

    /* semantic routine */
#line 3899 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Divide\n"));
#line 33831 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Divide, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3899 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxzdz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else if (is_invalid_zero_divide (processor, cia,
                                       *frA, *frB,
                                       0 /*single?*/)) {
        invalid_zero_divide_operation (processor, cia,
                                       frT, *frA, *frB,
                                       0 /*single?*/);
      }
      else {
        /*HACK!*/
        double s = *(double*)frA / *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 33870 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.18,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 33876 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Rc0(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Rc0
#line 3795 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Subtract\n"));
#line 33895 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3795 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 33929 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Rc0;

    /* semantic routine */
#line 3795 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Subtract\n"));
#line 33939 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3795 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 33971 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 33977 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Subtract_Rc1(cpu *processor,
                             instruction_word instruction,
                             unsigned_word cia,
                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Subtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Subtract_Rc1
#line 3795 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Subtract\n"));
#line 33996 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc000028)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3795 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 34030 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Subtract_Rc1;

    /* semantic routine */
#line 3795 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Subtract\n"));
#line 34040 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Subtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3795 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               1) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 34072 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.20,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 34078 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Rc0(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Rc0
#line 3743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Add\n"));
#line 34097 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 34131 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Rc0;

    /* semantic routine */
#line 3743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Add\n"));
#line 34141 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3743 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 34173 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 34179 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Add_Rc1(cpu *processor,
                        instruction_word instruction,
                        unsigned_word cia,
                        idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Add
#undef MY_PREFIX
#define MY_PREFIX Floating_Add_Rc1
#line 3743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Add\n"));
#line 34198 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc0007fe) != 0xfc00002a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 34232 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Add_Rc1;

    /* semantic routine */
#line 3743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Add\n"));
#line 34242 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Add, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3743 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK, Rc);
    }
#line 34274 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21./,26.21,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#line 34280 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Rc0
#line 4723 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Square Root\n"));
#line 34299 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4723 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 34328 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Rc0;

    /* semantic routine */
#line 4723 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Square Root\n"));
#line 34338 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4723 "../../sim/ppc/powerpc.igen"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34351 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34356 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Square_Root_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Square_Root
#undef MY_PREFIX
#define MY_PREFIX Floating_Square_Root_Rc1
#line 4723 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Square Root\n"));
#line 34375 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc00002c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4723 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 34404 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Square_Root_Rc1;

    /* semantic routine */
#line 4723 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Square Root\n"));
#line 34414 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Square_Root, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4723 "../../sim/ppc/powerpc.igen"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34427 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.22,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34432 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC0_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC0_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 34451 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 34490 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC0_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 34500 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 34522 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 34529 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC0_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC0_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 34548 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 34587 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC0_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 34597 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 34619 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 34626 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC0_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC0_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 34645 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 34679 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC0_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 34689 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 34721 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 34727 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC0_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC0_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 34746 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 34780 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC0_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 34790 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 34822 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 34828 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Square_Root_Estimate_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Square_Root_Estimate
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Square_Root_Estimate_Rc0
#line 4732 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Square Root Estimate\n"));
#line 34847 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4732 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 34876 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Square_Root_Estimate_Rc0;

    /* semantic routine */
#line 4732 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Square Root Estimate\n"));
#line 34886 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Square_Root_Estimate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4732 "../../sim/ppc/powerpc.igen"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34899 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34904 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Reciprocal_Square_Root_Estimate_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Reciprocal_Square_Root_Estimate
#undef MY_PREFIX
#define MY_PREFIX Floating_Reciprocal_Square_Root_Estimate_Rc1
#line 4732 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Reciprocal Square Root Estimate\n"));
#line 34923 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000034)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4732 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 34952 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Reciprocal_Square_Root_Estimate_Rc1;

    /* semantic routine */
#line 4732 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Reciprocal Square Root Estimate\n"));
#line 34962 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Reciprocal_Square_Root_Estimate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4732 "../../sim/ppc/powerpc.igen"
    {
      program_interrupt(processor, cia, optional_instruction_program_interrupt);
    }
#line 34975 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21./,26.26,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 34980 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC0_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC0_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 34999 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 35038 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC0_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 35048 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35101 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35108 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC0_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC0_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 35127 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 35166 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC0_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 35176 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35229 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35236 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC0_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC0_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 35255 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 35294 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC0_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 35304 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35357 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35364 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC0_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC0_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 35383 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 35422 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC0_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 35432 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35485 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35492 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC0_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC0_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 35511 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 35550 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC0_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 35560 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35613 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35620 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC0_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC0_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 35639 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 35678 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC0_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 35688 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35741 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35748 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC0_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC0_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 35767 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 35806 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC0_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 35816 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35869 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 35876 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC0_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC0_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 35895 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 0 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 35934 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC0_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 35944 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 35997 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36004 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Compare_Ordered(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Compare_Ordered
#undef MY_PREFIX
#define MY_PREFIX Floating_Compare_Ordered
#line 4643 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Compare Ordered\n"));
#line 36023 "icache.c"

  /* validate: 0.63,6.BF,9./,11.FRA,16.FRB,21.32,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc6007ff) != 0xfc000040)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.32,31./ #define get-values-from-icache|put-values-in-icache */
#line 67 "../../sim/ppc/powerpc.igen"
    unsigned const BF ATTRIBUTE_UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "../../sim/ppc/powerpc.igen"
    uint32_t const BF_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.X.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 36054 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Compare_Ordered;

    /* semantic routine */
#line 4643 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Compare Ordered\n"));
#line 36064 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Compare_Ordered, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4643 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      unsigned c;
      if (is_NaN(*frA, 0) || is_NaN(*frB, 0))
        c = cr_i_summary_overflow; /* 0b0001 - (FRA) ? (FRB) */
      else if (is_less_than(frA, frB))
        c = cr_i_negative; /* 0b1000 - (FRA) < (FRB) */
      else if (is_greater_than(frA, frB))
        c = cr_i_positive; /* 0b0100 - (FRA) > (FRB) */
      else
        c = cr_i_zero; /* 0b0010 - (FRA) = (FRB) */
      FPSCR_SET_FPCC(c);
      CR_SET(BF, c); /* CR[4*BF..4*BF+3] = c */
      if (is_SNaN(*frA, 0) || is_SNaN(*frB, 0)) {
        FPSCR_OR_VX(fpscr_vxsnan);
        if ((FPSCR & fpscr_ve) == 0)
          FPSCR_OR_VX(fpscr_vxvc);
      }
      else if (is_QNaN(*frA, 0) || is_QNaN(*frB, 0)) {
        FPSCR_OR_VX(fpscr_vxvc);
      }
      FPSCR_END(0);
      PPC_INSN_FLOAT_CR(0, FRA_BITMASK | FRB_BITMASK, BF_BITMASK);
    }
#line 36098 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11.FRA,16.FRB,21.32,31./ #undef get-values-from-icache|put-values-in-icache */
#undef FRA
#undef FRB
#line 36103 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_1_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_1
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_1_Rc0
#line 4706 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 1\n"));
#line 36122 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.38,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00004c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 75 "../../sim/ppc/powerpc.igen"
    unsigned const BT ATTRIBUTE_UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "../../sim/ppc/powerpc.igen"
    uint32_t const BT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4706 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 36145 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_1_Rc0;

    /* semantic routine */
#line 4706 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 1\n"));
#line 36155 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_1, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4706 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      uint32_t bit = BIT32(BT);
      if (bit & fpscr_fi)
        bit |= fpscr_xx;
      if ((bit & fpscr_vx_bits))
        bit |= fpscr_fx;
      /* note - omit vx bit */
      if ((bit & (fpscr_ox | fpscr_ux | fpscr_zx | fpscr_xx)))
        bit |= fpscr_fx;
      FPSCR |= bit;
      FPSCR_END(Rc);
    }
#line 36178 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 36181 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_1_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_1
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_1_Rc1
#line 4706 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 1\n"));
#line 36200 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.38,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00004c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 75 "../../sim/ppc/powerpc.igen"
    unsigned const BT ATTRIBUTE_UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "../../sim/ppc/powerpc.igen"
    uint32_t const BT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4706 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 36223 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_1_Rc1;

    /* semantic routine */
#line 4706 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 1\n"));
#line 36233 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_1, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4706 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      uint32_t bit = BIT32(BT);
      if (bit & fpscr_fi)
        bit |= fpscr_xx;
      if ((bit & fpscr_vx_bits))
        bit |= fpscr_fx;
      /* note - omit vx bit */
      if ((bit & (fpscr_ox | fpscr_ux | fpscr_zx | fpscr_xx)))
        bit |= fpscr_fx;
      FPSCR |= bit;
      FPSCR_END(Rc);
    }
#line 36256 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.38,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 36259 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negate_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negate
#undef MY_PREFIX
#define MY_PREFIX Floating_Negate_Rc0
#line 3711 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negate\n"));
#line 36278 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3711 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 36307 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negate_Rc0;

    /* semantic routine */
#line 3711 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negate\n"));
#line 36317 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3711 "../../sim/ppc/powerpc.igen"
    {
      *frT = *frB ^ BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 36332 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 36337 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negate_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negate
#undef MY_PREFIX
#define MY_PREFIX Floating_Negate_Rc1
#line 3711 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negate\n"));
#line 36356 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000050)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3711 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 36385 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negate_Rc1;

    /* semantic routine */
#line 3711 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negate\n"));
#line 36395 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3711 "../../sim/ppc/powerpc.igen"
    {
      *frT = *frB ^ BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 36410 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.40,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 36415 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC1_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC1_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 36434 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 36473 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC1_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 36483 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 36505 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36512 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC1_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC1_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 36531 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 36570 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC1_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 36580 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 36602 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36609 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC1_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC1_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 36628 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 36662 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC1_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 36672 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 36704 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 36710 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC1_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC1_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 36729 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 36763 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC1_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 36773 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 36805 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 36811 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC1_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC1_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 36830 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 36869 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC1_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 36879 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 36932 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 36939 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC1_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC1_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 36958 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 36997 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC1_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 37007 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37060 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37067 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC1_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC1_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 37086 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 37125 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC1_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 37135 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37188 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37195 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC1_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC1_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 37214 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 37253 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC1_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 37263 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37316 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37323 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC1_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC1_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 37342 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 37381 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC1_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 37391 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37444 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37451 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC1_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC1_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 37470 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 37509 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC1_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 37519 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37572 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37579 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC1_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC1_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 37598 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 37637 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC1_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 37647 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37700 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37707 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC1_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC1_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 37726 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 1 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 37765 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC1_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 37775 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 37828 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 37835 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_to_Condition_Register_from_FPSCR(cpu *processor,
                                             instruction_word instruction,
                                             unsigned_word cia,
                                             idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_to_Condition_Register_from_FPSCR
#undef MY_PREFIX
#define MY_PREFIX Move_to_Condition_Register_from_FPSCR
#line 4677 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move to Condition Register from FPSCR\n"));
#line 37854 "icache.c"

  /* validate: 0.63,6.BF,9./,11.BFA,14./,16./,21.64,31./ */
  if (WITH_RESERVED_BITS && (instruction & 0xfc63ffff) != 0xfc000080)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11.BFA,14./,16./,21.64,31./ #define get-values-from-icache|put-values-in-icache */
#line 67 "../../sim/ppc/powerpc.igen"
    unsigned const BF ATTRIBUTE_UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "../../sim/ppc/powerpc.igen"
    uint32_t const BF_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 4677 "../../sim/ppc/powerpc.igen"
    unsigned const BFA ATTRIBUTE_UNUSED = cache_entry->crack.X.BFA = EXTRACTED32(instruction, 11, 13); /* default in icache - extraction */
#line 37877 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_to_Condition_Register_from_FPSCR;

    /* semantic routine */
#line 4677 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move to Condition Register from FPSCR\n"));
#line 37887 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_to_Condition_Register_from_FPSCR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4677 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      unsigned field = FPSCR_FIELD(BFA);
      CR_SET(BF, field);
      FPSCR_SET(BFA, 0); /* FPSCR_END fixes up FEX/VX */
      FPSCR_END(0);
    }
#line 37904 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11.BFA,14./,16./,21.64,31./ #undef get-values-from-icache|put-values-in-icache */
#line 37907 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_0_Rc0(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_0
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_0_Rc0
#line 4700 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 0\n"));
#line 37926 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.70,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00008c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 75 "../../sim/ppc/powerpc.igen"
    unsigned const BT ATTRIBUTE_UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "../../sim/ppc/powerpc.igen"
    uint32_t const BT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4700 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 37949 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_0_Rc0;

    /* semantic routine */
#line 4700 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 0\n"));
#line 37959 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_0, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4700 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      uint32_t bit = BIT32(BT);
      FPSCR &= ~bit;
      FPSCR_END(Rc);
    }
#line 37975 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 37978 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Bit_0_Rc1(cpu *processor,
                               instruction_word instruction,
                               unsigned_word cia,
                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Bit_0
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Bit_0_Rc1
#line 4700 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Bit 0\n"));
#line 37997 "icache.c"

  /* validate: 0.63,6.BT,11./,16./,21.70,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00008c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 75 "../../sim/ppc/powerpc.igen"
    unsigned const BT ATTRIBUTE_UNUSED = cache_entry->crack.X.BT = EXTRACTED32(instruction, 6, 10); /* in icache - extraction */
#line 76 "../../sim/ppc/powerpc.igen"
    uint32_t const BT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.BT_BITMASK = (1 << BT); /* in icache - extraction */
#line 4700 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 38020 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Bit_0_Rc1;

    /* semantic routine */
#line 4700 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Bit 0\n"));
#line 38030 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Bit_0, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4700 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      uint32_t bit = BIT32(BT);
      FPSCR &= ~bit;
      FPSCR_END(Rc);
    }
#line 38046 "icache.c"
    return nia;
    /* extraction: 0.63,6.BT,11./,16./,21.70,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 38049 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Move_Register_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Move_Register
#undef MY_PREFIX
#define MY_PREFIX Floating_Move_Register_Rc0
#line 3702 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Move Register\n"));
#line 38068 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000090)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3702 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 38097 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Move_Register_Rc0;

    /* semantic routine */
#line 3702 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Move Register\n"));
#line 38107 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Move_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3702 "../../sim/ppc/powerpc.igen"
    {
      *frT = *frB;
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 38122 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 38127 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Move_Register_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Move_Register
#undef MY_PREFIX
#define MY_PREFIX Floating_Move_Register_Rc1
#line 3702 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Move Register\n"));
#line 38146 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000090)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3702 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 38175 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Move_Register_Rc1;

    /* semantic routine */
#line 3702 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Move Register\n"));
#line 38185 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Move_Register, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3702 "../../sim/ppc/powerpc.igen"
    {
      *frT = *frB;
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 38200 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.72,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 38205 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC2_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC2_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 38224 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 38263 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC2_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 38273 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 38295 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38302 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC2_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC2_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 38321 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 38360 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC2_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 38370 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 38392 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38399 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC2_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC2_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 38418 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 38452 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC2_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 38462 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 38494 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 38500 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC2_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC2_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 38519 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 38553 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC2_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 38563 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 38595 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 38601 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC2_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC2_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 38620 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 38659 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC2_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 38669 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 38722 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38729 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC2_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC2_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 38748 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 38787 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC2_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 38797 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 38850 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38857 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC2_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC2_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 38876 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 38915 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC2_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 38925 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 38978 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 38985 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC2_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC2_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 39004 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 39043 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC2_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 39053 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39106 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39113 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC2_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC2_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 39132 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 39171 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC2_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 39181 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39234 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39241 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC2_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC2_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 39260 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 39299 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC2_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 39309 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39362 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39369 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC2_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC2_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 39388 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 39427 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC2_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 39437 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39490 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39497 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC2_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC2_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 39516 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 2 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 39555 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC2_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 39565 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 39618 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39625 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC3_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC3_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 39644 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 39683 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC3_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 39693 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 39715 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39722 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC3_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC3_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 39741 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 39780 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC3_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 39790 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 39812 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 39819 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC3_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC3_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 39838 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 39872 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC3_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 39882 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 39914 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 39920 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC3_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC3_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 39939 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 39973 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC3_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 39983 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 40015 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 40021 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC3_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC3_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 40040 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 40079 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC3_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 40089 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40142 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40149 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC3_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC3_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 40168 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 40207 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC3_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 40217 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40270 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40277 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC3_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC3_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 40296 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 40335 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC3_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 40345 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40398 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40405 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC3_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC3_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 40424 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 40463 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC3_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 40473 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40526 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40533 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC3_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC3_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 40552 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 40591 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC3_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 40601 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40654 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40661 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC3_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC3_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 40680 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 40719 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC3_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 40729 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40782 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40789 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC3_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC3_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 40808 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 40847 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC3_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 40857 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 40910 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 40917 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC3_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC3_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 40936 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 3 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 40975 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC3_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 40985 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 41038 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41045 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Field_Immediate_Rc0(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Field_Immediate
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Field_Immediate_Rc0
#line 4684 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Field Immediate\n"));
#line 41064 "icache.c"

  /* validate: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc7f0ffe) != 0xfc00010c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 67 "../../sim/ppc/powerpc.igen"
    unsigned const BF ATTRIBUTE_UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "../../sim/ppc/powerpc.igen"
    uint32_t const BF_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 4684 "../../sim/ppc/powerpc.igen"
    unsigned const U ATTRIBUTE_UNUSED = cache_entry->crack.X.U = EXTRACTED32(instruction, 16, 19); /* default in icache - extraction */
#line 4684 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 41089 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Field_Immediate_Rc0;

    /* semantic routine */
#line 4684 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Field Immediate\n"));
#line 41099 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Field_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4684 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      FPSCR_SET(BF, U);
      FPSCR_END(Rc);
    }
#line 41114 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 41117 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Field_Immediate_Rc1(cpu *processor,
                                         instruction_word instruction,
                                         unsigned_word cia,
                                         idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Field_Immediate
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Field_Immediate_Rc1
#line 4684 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Field Immediate\n"));
#line 41136 "icache.c"

  /* validate: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc7f0ffe) != 0xfc00010c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 67 "../../sim/ppc/powerpc.igen"
    unsigned const BF ATTRIBUTE_UNUSED = cache_entry->crack.X.BF = EXTRACTED32(instruction, 6, 8); /* in icache - extraction */
#line 68 "../../sim/ppc/powerpc.igen"
    uint32_t const BF_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.BF_BITMASK = (1 << BF); /* in icache - extraction */
#line 4684 "../../sim/ppc/powerpc.igen"
    unsigned const U ATTRIBUTE_UNUSED = cache_entry->crack.X.U = EXTRACTED32(instruction, 16, 19); /* default in icache - extraction */
#line 4684 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 41161 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Field_Immediate_Rc1;

    /* semantic routine */
#line 4684 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Field Immediate\n"));
#line 41171 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Field_Immediate, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4684 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      FPSCR_SET(BF, U);
      FPSCR_END(Rc);
    }
#line 41186 "icache.c"
    return nia;
    /* extraction: 0.63,6.BF,9./,11./,16.U,20./,21.134,31.Rc #undef get-values-from-icache|put-values-in-icache */
#line 41189 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_Absolute_Value_Rc0(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_Absolute_Value_Rc0
#line 3729 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Absolute Value\n"));
#line 41208 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3729 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 41237 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_Absolute_Value_Rc0;

    /* semantic routine */
#line 3729 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Absolute Value\n"));
#line 41247 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3729 "../../sim/ppc/powerpc.igen"
    {
      *frT = *frB | BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 41262 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 41267 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_Absolute_Value_Rc1(cpu *processor,
                                            instruction_word instruction,
                                            unsigned_word cia,
                                            idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_Absolute_Value_Rc1
#line 3729 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Absolute Value\n"));
#line 41286 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000110)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3729 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 41315 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_Absolute_Value_Rc1;

    /* semantic routine */
#line 3729 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Absolute Value\n"));
#line 41325 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3729 "../../sim/ppc/powerpc.igen"
    {
      *frT = *frB | BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 41340 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.136,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 41345 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC4_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC4_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 41364 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 41403 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC4_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 41413 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 41435 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41442 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC4_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC4_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 41461 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 41500 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC4_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 41510 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 41532 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41539 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC4_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC4_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 41558 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 41592 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC4_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 41602 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 41634 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 41640 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC4_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC4_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 41659 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 41693 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC4_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 41703 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 41735 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 41741 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC4_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC4_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 41760 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 41799 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC4_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 41809 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 41862 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41869 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC4_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC4_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 41888 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 41927 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC4_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 41937 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 41990 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 41997 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC4_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC4_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 42016 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 42055 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC4_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 42065 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42118 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42125 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC4_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC4_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 42144 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 42183 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC4_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 42193 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42246 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42253 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC4_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC4_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 42272 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 42311 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC4_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 42321 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42374 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42381 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC4_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC4_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 42400 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 42439 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC4_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 42449 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42502 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42509 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC4_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC4_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 42528 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 42567 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC4_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 42577 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42630 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42637 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC4_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC4_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 42656 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 4 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 42695 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC4_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 42705 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 42758 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42765 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC5_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC5_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 42784 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 42823 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC5_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 42833 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 42855 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42862 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC5_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC5_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 42881 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 42920 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC5_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 42930 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 42952 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 42959 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC5_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC5_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 42978 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 43012 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC5_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 43022 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 43054 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 43060 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC5_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC5_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 43079 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 43113 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC5_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 43123 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 43155 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 43161 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC5_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC5_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 43180 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 43219 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC5_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 43229 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43282 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43289 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC5_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC5_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 43308 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 43347 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC5_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 43357 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43410 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43417 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC5_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC5_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 43436 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 43475 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC5_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 43485 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43538 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43545 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC5_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC5_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 43564 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 43603 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC5_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 43613 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43666 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43673 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC5_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC5_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 43692 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 43731 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC5_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 43741 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43794 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43801 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC5_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC5_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 43820 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 43859 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC5_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 43869 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 43922 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 43929 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC5_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC5_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 43948 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 43987 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC5_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 43997 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44050 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44057 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC5_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC5_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 44076 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 5 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 44115 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC5_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 44125 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44178 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44185 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC6_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC6_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 44204 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 44243 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC6_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 44253 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 44275 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44282 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC6_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC6_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 44301 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 44340 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC6_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 44350 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 44372 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44379 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC6_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC6_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 44398 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 44432 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC6_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 44442 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 44474 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 44480 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC6_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC6_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 44499 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 44533 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC6_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 44543 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 44575 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 44581 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC6_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC6_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 44600 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 44639 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC6_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 44649 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44702 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44709 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC6_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC6_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 44728 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 44767 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC6_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 44777 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44830 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44837 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC6_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC6_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 44856 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 44895 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC6_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 44905 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 44958 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 44965 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC6_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC6_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 44984 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 45023 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC6_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 45033 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45086 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45093 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC6_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC6_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 45112 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 45151 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC6_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 45161 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45214 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45221 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC6_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC6_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 45240 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 45279 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC6_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 45289 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45342 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45349 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC6_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC6_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 45368 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 45407 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC6_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 45417 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45470 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45477 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC6_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC6_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 45496 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 6 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 45535 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC6_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 45545 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 45598 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45605 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC7_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC7_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 45624 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 45663 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC7_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 45673 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 45695 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45702 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC7_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC7_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 45721 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 45760 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC7_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 45770 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 45792 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 45799 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC7_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC7_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 45818 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 45852 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC7_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 45862 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 45894 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 45900 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC7_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC7_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 45919 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 45953 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC7_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 45963 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 45995 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 46001 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC7_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC7_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 46020 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 46059 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC7_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 46069 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46122 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46129 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC7_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC7_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 46148 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 46187 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC7_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 46197 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46250 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46257 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC7_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC7_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 46276 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 46315 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC7_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 46325 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46378 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46385 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC7_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC7_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 46404 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 46443 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC7_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 46453 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46506 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46513 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC7_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC7_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 46532 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 46571 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC7_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 46581 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46634 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46641 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC7_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC7_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 46660 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 46699 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC7_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 46709 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46762 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46769 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC7_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC7_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 46788 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 46827 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC7_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 46837 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 46890 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 46897 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC7_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC7_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 46916 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 7 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 46955 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC7_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 46965 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47018 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47025 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Absolute_Value_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Absolute_Value_Rc0
#line 3720 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Absolute Value\n"));
#line 47044 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000210)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3720 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 47073 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Absolute_Value_Rc0;

    /* semantic routine */
#line 3720 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Absolute Value\n"));
#line 47083 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3720 "../../sim/ppc/powerpc.igen"
    {
      *frT = *frB & ~BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 47098 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 47103 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Absolute_Value_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Absolute_Value
#undef MY_PREFIX
#define MY_PREFIX Floating_Absolute_Value_Rc1
#line 3720 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Absolute Value\n"));
#line 47122 "icache.c"

  /* validate: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1f07fe) != 0xfc000210)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.X.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 3720 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 47151 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Absolute_Value_Rc1;

    /* semantic routine */
#line 3720 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Absolute Value\n"));
#line 47161 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Absolute_Value, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3720 "../../sim/ppc/powerpc.igen"
    {
      *frT = *frB & ~BIT64(0);
      CR1_UPDATE(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRB_BITMASK, Rc);
    }
#line 47176 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16.FRB,21.264,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRB
#line 47181 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC8_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC8_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 47200 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 47239 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC8_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 47249 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 47271 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47278 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC8_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC8_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 47297 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 47336 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC8_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 47346 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 47368 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47375 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC8_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC8_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 47394 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 47428 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC8_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 47438 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 47470 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 47476 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC8_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC8_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 47495 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 47529 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC8_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 47539 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 47571 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 47577 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC8_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC8_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 47596 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 47635 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC8_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 47645 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47698 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47705 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC8_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC8_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 47724 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 47763 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC8_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 47773 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47826 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47833 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC8_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC8_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 47852 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 47891 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC8_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 47901 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 47954 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 47961 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC8_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC8_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 47980 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 48019 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC8_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 48029 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48082 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48089 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC8_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC8_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 48108 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 48147 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC8_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 48157 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48210 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48217 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC8_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC8_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 48236 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 48275 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC8_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 48285 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48338 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48345 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC8_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC8_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 48364 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 48403 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC8_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 48413 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48466 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48473 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC8_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC8_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 48492 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 8 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 48531 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC8_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 48541 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 48594 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48601 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC9_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC9_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 48620 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 48659 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC9_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 48669 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 48691 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48698 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC9_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC9_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 48717 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 48756 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC9_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 48766 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 48788 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 48795 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC9_Rc0(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC9_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 48814 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 48848 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC9_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 48858 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 48890 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 48896 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC9_Rc1(cpu *processor,
                                  instruction_word instruction,
                                  unsigned_word cia,
                                  idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC9_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 48915 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 48949 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC9_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 48959 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 48991 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 48997 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC9_Rc0(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC9_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 49016 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 49055 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC9_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 49065 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49118 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49125 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC9_Rc1(cpu *processor,
                                          instruction_word instruction,
                                          unsigned_word cia,
                                          idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC9_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 49144 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 49183 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC9_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 49193 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49246 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49253 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC9_Rc0(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC9_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 49272 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 49311 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC9_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 49321 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49374 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49381 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC9_Rc1(cpu *processor,
                                     instruction_word instruction,
                                     unsigned_word cia,
                                     idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC9_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 49400 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 49439 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC9_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 49449 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49502 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49509 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC9_Rc0(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC9_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 49528 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 49567 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC9_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 49577 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49630 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49637 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC9_Rc1(cpu *processor,
                                                   instruction_word instruction,
                                                   unsigned_word cia,
                                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC9_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 49656 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 49695 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC9_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 49705 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49758 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49765 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC9_Rc0(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC9_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 49784 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 49823 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC9_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 49833 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 49886 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 49893 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC9_Rc1(cpu *processor,
                                              instruction_word instruction,
                                              unsigned_word cia,
                                              idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC9_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 49912 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 9 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 49951 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC9_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 49961 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50014 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50021 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC10_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC10_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 50040 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 50079 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC10_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 50089 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 50111 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50118 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC10_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC10_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 50137 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 50176 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC10_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 50186 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 50208 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50215 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC10_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC10_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 50234 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 50268 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC10_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 50278 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 50310 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 50316 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC10_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC10_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 50335 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 50369 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC10_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 50379 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 50411 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 50417 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC10_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC10_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 50436 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 50475 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC10_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 50485 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50538 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50545 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC10_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC10_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 50564 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 50603 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC10_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 50613 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50666 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50673 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC10_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC10_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 50692 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 50731 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC10_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 50741 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50794 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50801 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC10_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC10_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 50820 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 50859 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC10_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 50869 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 50922 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 50929 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC10_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC10_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 50948 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 50987 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC10_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 50997 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51050 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51057 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC10_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC10_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 51076 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 51115 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC10_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 51125 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51178 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51185 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC10_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC10_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 51204 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 51243 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC10_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 51253 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51306 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51313 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC10_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC10_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 51332 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 10 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 51371 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC10_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 51381 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51434 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51441 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC11_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC11_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 51460 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 51499 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC11_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 51509 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 51531 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51538 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC11_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC11_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 51557 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 51596 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC11_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 51606 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 51628 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51635 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC11_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC11_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 51654 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 51688 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC11_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 51698 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 51730 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 51736 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC11_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC11_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 51755 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 51789 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC11_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 51799 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 51831 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 51837 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC11_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC11_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 51856 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 51895 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC11_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 51905 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 51958 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 51965 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC11_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC11_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 51984 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 52023 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC11_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 52033 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52086 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52093 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC11_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC11_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 52112 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 52151 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC11_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 52161 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52214 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52221 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC11_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC11_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 52240 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 52279 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC11_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 52289 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52342 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52349 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC11_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC11_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 52368 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 52407 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC11_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 52417 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52470 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52477 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC11_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC11_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 52496 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 52535 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC11_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 52545 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52598 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52605 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC11_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC11_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 52624 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 52663 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC11_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 52673 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52726 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52733 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC11_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC11_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 52752 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 11 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 52791 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC11_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 52801 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 52854 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52861 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC12_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC12_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 52880 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 52919 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC12_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 52929 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 52951 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 52958 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC12_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC12_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 52977 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 53016 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC12_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 53026 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 53048 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53055 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC12_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC12_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 53074 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 53108 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC12_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 53118 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 53150 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 53156 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC12_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC12_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 53175 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 53209 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC12_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 53219 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 53251 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 53257 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC12_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC12_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 53276 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 53315 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC12_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 53325 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53378 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53385 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC12_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC12_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 53404 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 53443 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC12_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 53453 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53506 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53513 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC12_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC12_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 53532 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 53571 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC12_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 53581 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53634 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53641 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC12_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC12_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 53660 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 53699 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC12_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 53709 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53762 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53769 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC12_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC12_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 53788 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 53827 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC12_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 53837 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 53890 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 53897 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC12_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC12_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 53916 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 53955 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC12_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 53965 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54018 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54025 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC12_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC12_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 54044 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 54083 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC12_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 54093 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54146 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54153 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC12_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC12_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 54172 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 12 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 54211 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC12_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 54221 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54274 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54281 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC13_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC13_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 54300 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 54339 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC13_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 54349 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 54371 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54378 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC13_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC13_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 54397 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 54436 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC13_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 54446 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 54468 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54475 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC13_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC13_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 54494 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 54528 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC13_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 54538 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 54570 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 54576 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC13_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC13_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 54595 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 54629 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC13_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 54639 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 54671 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 54677 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC13_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC13_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 54696 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 54735 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC13_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 54745 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54798 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54805 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC13_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC13_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 54824 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 54863 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC13_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 54873 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 54926 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 54933 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC13_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC13_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 54952 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 54991 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC13_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 55001 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55054 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55061 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC13_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC13_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 55080 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 55119 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC13_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 55129 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55182 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55189 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC13_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC13_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 55208 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 55247 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC13_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 55257 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55310 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55317 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC13_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC13_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 55336 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 55375 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC13_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 55385 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55438 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55445 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC13_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC13_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 55464 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 55503 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC13_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 55513 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55566 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55573 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC13_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC13_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 55592 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 13 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 55631 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC13_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 55641 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 55694 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55701 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC14_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC14_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 55720 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 55759 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC14_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 55769 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 55791 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55798 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC14_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC14_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 55817 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 55856 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC14_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 55866 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 55888 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 55895 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC14_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC14_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 55914 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 55948 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC14_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 55958 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 55990 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 55996 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC14_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC14_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 56015 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 56049 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC14_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 56059 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 56091 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 56097 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC14_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC14_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 56116 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 56155 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC14_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 56165 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56218 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56225 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC14_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC14_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 56244 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 56283 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC14_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 56293 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56346 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56353 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC14_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC14_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 56372 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 56411 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC14_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 56421 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56474 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56481 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC14_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC14_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 56500 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 56539 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC14_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 56549 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56602 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56609 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC14_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC14_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 56628 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 56667 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC14_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 56677 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56730 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56737 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC14_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC14_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 56756 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 56795 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC14_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 56805 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56858 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56865 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC14_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC14_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 56884 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 56923 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC14_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 56933 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 56986 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 56993 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC14_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC14_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 57012 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 14 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 57051 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC14_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 57061 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57114 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57121 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC15_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC15_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 57140 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 57179 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC15_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 57189 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 57211 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57218 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC15_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC15_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 57237 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 57276 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC15_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 57286 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 57308 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57315 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC15_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC15_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 57334 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 57368 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC15_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 57378 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 57410 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 57416 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC15_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC15_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 57435 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 57469 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC15_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 57479 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 57511 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 57517 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC15_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC15_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 57536 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 57575 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC15_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 57585 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57638 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57645 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC15_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC15_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 57664 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 57703 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC15_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 57713 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57766 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57773 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC15_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC15_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 57792 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 57831 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC15_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 57841 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 57894 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 57901 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC15_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC15_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 57920 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 57959 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC15_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 57969 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58022 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58029 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC15_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC15_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 58048 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 58087 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC15_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 58097 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58150 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58157 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC15_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC15_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 58176 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 58215 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC15_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 58225 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58278 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58285 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC15_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC15_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 58304 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 58343 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC15_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 58353 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58406 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58413 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC15_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC15_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 58432 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 15 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 58471 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC15_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 58481 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 58534 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58541 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC16_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC16_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 58560 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 58599 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC16_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 58609 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 58631 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58638 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC16_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC16_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 58657 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 58696 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC16_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 58706 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 58728 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 58735 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC16_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC16_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 58754 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 58788 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC16_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 58798 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 58830 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 58836 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC16_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC16_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 58855 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 58889 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC16_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 58899 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 58931 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 58937 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC16_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC16_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 58956 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 58995 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC16_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 59005 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59058 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59065 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC16_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC16_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 59084 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 59123 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC16_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 59133 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59186 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59193 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC16_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC16_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 59212 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 59251 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC16_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 59261 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59314 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59321 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC16_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC16_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 59340 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 59379 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC16_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 59389 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59442 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59449 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC16_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC16_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 59468 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 59507 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC16_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 59517 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59570 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59577 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC16_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC16_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 59596 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 59635 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC16_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 59645 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59698 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59705 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC16_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC16_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 59724 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 59763 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC16_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 59773 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59826 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59833 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC16_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC16_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 59852 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 16 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 59891 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC16_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 59901 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 59954 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 59961 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC17_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC17_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 59980 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 60019 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC17_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 60029 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 60051 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60058 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC17_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC17_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 60077 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 60116 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC17_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 60126 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 60148 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60155 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC17_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC17_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 60174 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 60208 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC17_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 60218 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 60250 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 60256 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC17_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC17_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 60275 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 60309 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC17_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 60319 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 60351 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 60357 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC17_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC17_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 60376 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 60415 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC17_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 60425 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60478 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60485 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC17_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC17_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 60504 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 60543 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC17_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 60553 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60606 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60613 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC17_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC17_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 60632 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 60671 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC17_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 60681 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60734 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60741 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC17_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC17_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 60760 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 60799 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC17_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 60809 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60862 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60869 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC17_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC17_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 60888 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 60927 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC17_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 60937 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 60990 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 60997 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC17_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC17_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 61016 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 61055 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC17_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 61065 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 61118 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61125 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC17_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC17_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 61144 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 61183 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC17_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 61193 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 61246 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61253 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC17_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC17_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 61272 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 17 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 61311 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC17_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 61321 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 61374 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61381 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_FPSCR_Rc0(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_FPSCR
#undef MY_PREFIX
#define MY_PREFIX Move_From_FPSCR_Rc0
#line 4672 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move From FPSCR\n"));
#line 61400 "icache.c"

  /* validate: 0.63,6.FRT,11./,16./,21.583,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00048e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 4672 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 61424 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_FPSCR_Rc0;

    /* semantic routine */
#line 4672 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move From FPSCR\n"));
#line 61434 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_FPSCR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4672 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      *frT = FPSCR;
      FPSCR_END(Rc);
    }
#line 61449 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#line 61453 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_From_FPSCR_Rc1(cpu *processor,
                           instruction_word instruction,
                           unsigned_word cia,
                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_From_FPSCR
#undef MY_PREFIX
#define MY_PREFIX Move_From_FPSCR_Rc1
#line 4672 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move From FPSCR\n"));
#line 61472 "icache.c"

  /* validate: 0.63,6.FRT,11./,16./,21.583,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc1ffffe) != 0xfc00048e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.X.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.X.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#line 4672 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 61496 "icache.c"

#define insn cache_entry->crack.X.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_From_FPSCR_Rc1;

    /* semantic routine */
#line 4672 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move From FPSCR\n"));
#line 61506 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_From_FPSCR, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4672 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      *frT = FPSCR;
      FPSCR_END(Rc);
    }
#line 61521 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11./,16./,21.583,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#line 61525 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC18_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC18_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 61544 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 61583 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC18_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 61593 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 61615 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61622 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC18_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC18_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 61641 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 61680 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC18_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 61690 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 61712 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 61719 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC18_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC18_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 61738 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 61772 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC18_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 61782 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 61814 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 61820 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC18_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC18_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 61839 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 61873 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC18_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 61883 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 61915 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 61921 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC18_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC18_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 61940 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 61979 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC18_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 61989 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62042 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62049 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC18_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC18_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 62068 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 62107 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC18_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 62117 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62170 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62177 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC18_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC18_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 62196 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 62235 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC18_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 62245 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62298 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62305 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC18_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC18_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 62324 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 62363 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC18_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 62373 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62426 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62433 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC18_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC18_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 62452 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 62491 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC18_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 62501 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62554 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62561 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC18_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC18_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 62580 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 62619 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC18_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 62629 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62682 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62689 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC18_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC18_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 62708 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 62747 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC18_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 62757 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62810 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62817 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC18_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC18_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 62836 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 18 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 62875 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC18_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 62885 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 62938 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 62945 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC19_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC19_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 62964 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 63003 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC19_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 63013 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 63035 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63042 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC19_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC19_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 63061 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 63100 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC19_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 63110 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 63132 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63139 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC19_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC19_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 63158 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 63192 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC19_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 63202 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 63234 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 63240 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC19_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC19_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 63259 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 63293 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC19_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 63303 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 63335 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 63341 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC19_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC19_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 63360 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 63399 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC19_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 63409 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63462 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63469 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC19_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC19_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 63488 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 63527 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC19_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 63537 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63590 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63597 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC19_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC19_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 63616 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 63655 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC19_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 63665 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63718 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63725 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC19_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC19_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 63744 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 63783 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC19_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 63793 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63846 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63853 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC19_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC19_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 63872 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 63911 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC19_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 63921 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 63974 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 63981 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC19_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC19_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 64000 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 64039 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC19_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 64049 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64102 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64109 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC19_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC19_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 64128 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 64167 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC19_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 64177 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64230 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64237 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC19_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC19_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 64256 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 19 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 64295 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC19_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 64305 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64358 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64365 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC20_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC20_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 64384 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 64423 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC20_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 64433 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 64455 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64462 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC20_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC20_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 64481 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 64520 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC20_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 64530 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 64552 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64559 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC20_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC20_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 64578 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 64612 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC20_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 64622 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 64654 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 64660 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC20_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC20_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 64679 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 64713 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC20_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 64723 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 64755 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 64761 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC20_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC20_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 64780 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 64819 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC20_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 64829 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 64882 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 64889 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC20_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC20_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 64908 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 64947 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC20_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 64957 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65010 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65017 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC20_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC20_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 65036 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 65075 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC20_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 65085 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65138 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65145 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC20_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC20_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 65164 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 65203 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC20_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 65213 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65266 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65273 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC20_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC20_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 65292 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 65331 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC20_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 65341 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65394 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65401 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC20_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC20_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 65420 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 65459 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC20_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 65469 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65522 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65529 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC20_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC20_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 65548 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 65587 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC20_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 65597 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65650 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65657 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC20_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC20_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 65676 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 20 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 65715 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC20_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 65725 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 65778 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65785 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC21_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC21_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 65804 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 65843 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC21_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 65853 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 65875 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65882 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC21_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC21_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 65901 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 65940 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC21_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 65950 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 65972 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 65979 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC21_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC21_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 65998 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 66032 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC21_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 66042 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 66074 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 66080 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC21_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC21_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 66099 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 66133 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC21_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 66143 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 66175 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 66181 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC21_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC21_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 66200 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 66239 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC21_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 66249 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66302 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66309 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC21_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC21_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 66328 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 66367 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC21_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 66377 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66430 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66437 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC21_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC21_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 66456 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 66495 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC21_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 66505 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66558 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66565 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC21_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC21_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 66584 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 66623 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC21_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 66633 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66686 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66693 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC21_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC21_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 66712 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 66751 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC21_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 66761 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66814 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66821 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC21_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC21_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 66840 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 66879 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC21_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 66889 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 66942 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 66949 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC21_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC21_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 66968 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 67007 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC21_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 67017 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 67070 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67077 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC21_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC21_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 67096 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 21 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 67135 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC21_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 67145 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 67198 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67205 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Fields_Rc0(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Fields
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Fields_Rc0
#line 4689 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Fields\n"));
#line 67224 "icache.c"

  /* validate: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfe0107fe) != 0xfc00058e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 4689 "../../sim/ppc/powerpc.igen"
    unsigned const FLM ATTRIBUTE_UNUSED = cache_entry->crack.XFL.FLM = EXTRACTED32(instruction, 7, 14); /* default in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.XFL.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XFL.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4689 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 67250 "icache.c"

#define insn cache_entry->crack.XFL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Fields_Rc0;

    /* semantic routine */
#line 4689 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Fields\n"));
#line 67260 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Fields, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4689 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      int i;
      for (i = 0; i < 8; i++) {
        if ((FLM & BIT8(i))) {
          FPSCR &= ~MASK32(i*4, i*4+3);
          FPSCR |= MASKED32(*frB, i*4, i*4+3);
        }
      }
      FPSCR_END(Rc);
    }
#line 67281 "icache.c"
    return nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRB
#line 67285 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Move_To_FPSCR_Fields_Rc1(cpu *processor,
                                instruction_word instruction,
                                unsigned_word cia,
                                idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Move_To_FPSCR_Fields
#undef MY_PREFIX
#define MY_PREFIX Move_To_FPSCR_Fields_Rc1
#line 4689 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Move To FPSCR Fields\n"));
#line 67304 "icache.c"

  /* validate: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfe0107fe) != 0xfc00058e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #define get-values-from-icache|put-values-in-icache */
#line 4689 "../../sim/ppc/powerpc.igen"
    unsigned const FLM ATTRIBUTE_UNUSED = cache_entry->crack.XFL.FLM = EXTRACTED32(instruction, 7, 14); /* default in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.XFL.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.XFL.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#line 4689 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 67330 "icache.c"

#define insn cache_entry->crack.XFL.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Move_To_FPSCR_Fields_Rc1;

    /* semantic routine */
#line 4689 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Move To FPSCR Fields\n"));
#line 67340 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Move_To_FPSCR_Fields, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4689 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      int i;
      for (i = 0; i < 8; i++) {
        if ((FLM & BIT8(i))) {
          FPSCR &= ~MASK32(i*4, i*4+3);
          FPSCR |= MASKED32(*frB, i*4, i*4+3);
        }
      }
      FPSCR_END(Rc);
    }
#line 67361 "icache.c"
    return nia;
    /* extraction: 0.63,6./,7.FLM,15./,16.FRB,21.711,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRB
#line 67365 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC22_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC22_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 67384 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 67423 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC22_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 67433 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 67455 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67462 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC22_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC22_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 67481 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 67520 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC22_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 67530 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 67552 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67559 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC22_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC22_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 67578 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 67612 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC22_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 67622 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 67654 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 67660 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC22_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC22_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 67679 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 67713 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC22_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 67723 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 67755 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 67761 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC22_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC22_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 67780 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 67819 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC22_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 67829 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 67882 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 67889 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC22_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC22_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 67908 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 67947 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC22_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 67957 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68010 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68017 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC22_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC22_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 68036 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 68075 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC22_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 68085 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68138 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68145 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC22_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC22_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 68164 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 68203 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC22_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 68213 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68266 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68273 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC22_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC22_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 68292 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 68331 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC22_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 68341 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68394 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68401 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC22_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC22_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 68420 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 68459 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC22_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 68469 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68522 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68529 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC22_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC22_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 68548 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 68587 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC22_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 68597 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68650 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68657 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC22_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC22_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 68676 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 22 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 68715 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC22_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 68725 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 68778 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68785 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC23_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC23_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 68804 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 68843 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC23_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 68853 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 68875 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68882 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC23_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC23_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 68901 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 68940 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC23_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 68950 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 68972 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 68979 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC23_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC23_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 68998 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 69032 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC23_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 69042 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 69074 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 69080 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC23_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC23_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 69099 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 69133 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC23_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 69143 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 69175 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 69181 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC23_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC23_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 69200 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 69239 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC23_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 69249 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69302 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69309 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC23_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC23_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 69328 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 69367 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC23_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 69377 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69430 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69437 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC23_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC23_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 69456 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 69495 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC23_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 69505 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69558 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69565 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC23_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC23_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 69584 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 69623 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC23_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 69633 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69686 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69693 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC23_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC23_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 69712 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 69751 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC23_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 69761 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69814 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69821 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC23_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC23_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 69840 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 69879 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC23_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 69889 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 69942 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 69949 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC23_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC23_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 69968 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 70007 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC23_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 70017 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70070 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70077 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC23_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC23_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 70096 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 23 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 70135 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC23_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 70145 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70198 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70205 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC24_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC24_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 70224 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 70263 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC24_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 70273 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 70295 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70302 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC24_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC24_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 70321 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 70360 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC24_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 70370 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 70392 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70399 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC24_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC24_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 70418 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 70452 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC24_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 70462 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 70494 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 70500 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC24_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC24_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 70519 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 70553 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC24_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 70563 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 70595 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 70601 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC24_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC24_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 70620 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 70659 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC24_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 70669 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70722 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70729 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC24_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC24_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 70748 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 70787 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC24_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 70797 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70850 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70857 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC24_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC24_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 70876 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 70915 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC24_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 70925 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 70978 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 70985 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC24_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC24_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 71004 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 71043 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC24_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 71053 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71106 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71113 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC24_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC24_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 71132 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 71171 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC24_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 71181 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71234 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71241 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC24_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC24_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 71260 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 71299 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC24_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 71309 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71362 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71369 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC24_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC24_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 71388 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 71427 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC24_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 71437 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71490 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71497 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC24_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC24_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 71516 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 24 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 71555 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC24_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 71565 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 71618 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71625 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC25_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC25_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 71644 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 71683 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC25_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 71693 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 71715 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71722 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC25_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC25_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 71741 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 71780 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC25_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 71790 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 71812 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 71819 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC25_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC25_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 71838 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 71872 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC25_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 71882 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 71914 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 71920 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC25_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC25_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 71939 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 71973 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC25_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 71983 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 72015 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 72021 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC25_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC25_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 72040 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 72079 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC25_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 72089 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72142 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72149 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC25_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC25_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 72168 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 72207 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC25_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 72217 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72270 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72277 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC25_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC25_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 72296 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 72335 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC25_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 72345 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72398 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72405 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC25_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC25_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 72424 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 72463 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC25_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 72473 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72526 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72533 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC25_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC25_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 72552 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 72591 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC25_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 72601 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72654 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72661 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC25_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC25_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 72680 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 72719 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC25_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 72729 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72782 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72789 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC25_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC25_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 72808 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 72847 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC25_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 72857 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 72910 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 72917 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC25_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC25_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 72936 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 25 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 72975 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC25_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 72985 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73038 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73045 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC26_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC26_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 73064 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 73103 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC26_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 73113 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 73135 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73142 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC26_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC26_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 73161 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 73200 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC26_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 73210 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 73232 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73239 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC26_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC26_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 73258 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 73292 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC26_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 73302 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 73334 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 73340 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC26_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC26_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 73359 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 73393 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC26_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 73403 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 73435 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 73441 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC26_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC26_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 73460 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 73499 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC26_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 73509 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73562 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73569 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC26_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC26_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 73588 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 73627 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC26_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 73637 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73690 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73697 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC26_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC26_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 73716 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 73755 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC26_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 73765 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73818 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73825 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC26_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC26_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 73844 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 73883 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC26_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 73893 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 73946 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 73953 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC26_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC26_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 73972 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 74011 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC26_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 74021 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74074 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74081 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC26_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC26_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 74100 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 74139 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC26_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 74149 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74202 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74209 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC26_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC26_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 74228 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 74267 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC26_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 74277 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74330 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74337 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC26_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC26_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 74356 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 26 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 74395 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC26_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 74405 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74458 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74465 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC27_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC27_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 74484 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 74523 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC27_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 74533 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 74555 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74562 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC27_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC27_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 74581 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 74620 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC27_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 74630 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 74652 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74659 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC27_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC27_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 74678 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 74712 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC27_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 74722 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 74754 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 74760 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC27_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC27_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 74779 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 74813 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC27_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 74823 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 74855 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 74861 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC27_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC27_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 74880 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 74919 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC27_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 74929 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 74982 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 74989 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC27_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC27_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 75008 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 75047 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC27_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 75057 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75110 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75117 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC27_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC27_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 75136 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 75175 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC27_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 75185 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75238 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75245 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC27_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC27_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 75264 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 75303 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC27_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 75313 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75366 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75373 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC27_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC27_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 75392 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 75431 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC27_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 75441 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75494 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75501 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC27_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC27_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 75520 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 75559 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC27_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 75569 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75622 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75629 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC27_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC27_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 75648 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 75687 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC27_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 75697 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75750 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75757 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC27_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC27_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 75776 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 27 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 75815 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC27_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 75825 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 75878 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75885 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC28_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC28_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 75904 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 75943 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC28_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 75953 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 75975 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 75982 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC28_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC28_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 76001 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 76040 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC28_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 76050 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 76072 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76079 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC28_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC28_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 76098 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 76132 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC28_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 76142 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 76174 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 76180 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC28_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC28_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 76199 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 76233 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC28_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 76243 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 76275 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 76281 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC28_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC28_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 76300 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 76339 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC28_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 76349 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76402 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76409 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC28_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC28_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 76428 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 76467 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC28_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 76477 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76530 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76537 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC28_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC28_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 76556 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 76595 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC28_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 76605 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76658 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76665 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC28_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC28_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 76684 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 76723 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC28_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 76733 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76786 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76793 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC28_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC28_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 76812 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 76851 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC28_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 76861 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 76914 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 76921 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC28_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC28_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 76940 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 76979 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC28_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 76989 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77042 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77049 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC28_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC28_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 77068 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 77107 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC28_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 77117 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77170 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77177 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC28_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC28_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 77196 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 28 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 77235 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC28_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 77245 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77298 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77305 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC29_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC29_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 77324 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 77363 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC29_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 77373 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 77395 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77402 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC29_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC29_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 77421 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 77460 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC29_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 77470 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 77492 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77499 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC29_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC29_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 77518 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 77552 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC29_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 77562 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 77594 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 77600 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC29_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC29_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 77619 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 77653 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC29_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 77663 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 77695 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 77701 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC29_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC29_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 77720 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 77759 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC29_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 77769 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77822 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77829 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC29_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC29_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 77848 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 77887 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC29_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 77897 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 77950 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 77957 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC29_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC29_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 77976 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 78015 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC29_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 78025 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78078 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78085 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC29_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC29_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 78104 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 78143 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC29_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 78153 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78206 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78213 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC29_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC29_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 78232 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 78271 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC29_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 78281 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78334 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78341 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC29_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC29_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 78360 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 78399 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC29_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 78409 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78462 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78469 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC29_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC29_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 78488 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 78527 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC29_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 78537 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78590 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78597 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC29_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC29_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 78616 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 29 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 78655 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC29_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 78665 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 78718 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78725 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC30_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC30_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 78744 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 78783 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC30_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 78793 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 78815 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78822 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC30_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC30_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 78841 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 78880 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC30_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 78890 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 78912 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 78919 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC30_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC30_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 78938 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 78972 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC30_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 78982 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 79014 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 79020 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC30_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC30_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 79039 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 79073 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC30_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 79083 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 79115 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 79121 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC30_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC30_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 79140 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 79179 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC30_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 79189 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79242 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79249 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC30_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC30_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 79268 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 79307 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC30_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 79317 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79370 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79377 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC30_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC30_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 79396 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 79435 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC30_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 79445 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79498 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79505 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC30_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC30_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 79524 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 79563 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC30_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 79573 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79626 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79633 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC30_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC30_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 79652 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 79691 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC30_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 79701 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79754 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79761 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC30_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC30_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 79780 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 79819 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC30_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 79829 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 79882 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 79889 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC30_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC30_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 79908 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 79947 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC30_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 79957 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80010 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80017 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC30_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC30_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 80036 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 30 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 80075 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC30_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 80085 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80138 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80145 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC31_Rc0(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC31_Rc0
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 80164 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 80203 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC31_Rc0;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 80213 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 80235 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80242 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Select_FRC31_Rc1(cpu *processor,
                                 instruction_word instruction,
                                 unsigned_word cia,
                                 idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Select
#undef MY_PREFIX
#define MY_PREFIX Floating_Select_FRC31_Rc1
#line 4743 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Select\n"));
#line 80261 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00002e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4743 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 80300 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Select_FRC31_Rc1;

    /* semantic routine */
#line 4743 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Select\n"));
#line 80310 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Select, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4743 "../../sim/ppc/powerpc.igen"
    {
      if (CURRENT_MODEL == MODEL_ppc601) {
        program_interrupt(processor, cia, optional_instruction_program_interrupt);
      } else {
        uint64_t zero = 0;
        FPSCR_BEGIN;
        if (is_NaN(*frA, 0) || is_less_than (frA, &zero)) *frT = *frB;
        else						    *frT = *frC;
        FPSCR_END(Rc);
        PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
      }
    }
#line 80332 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.23,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80339 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC31_Rc0(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC31_Rc0
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 80358 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 80392 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC31_Rc0;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 80402 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 80434 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 80440 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Multiply_FRC31_Rc1(cpu *processor,
                                   instruction_word instruction,
                                   unsigned_word cia,
                                   idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Multiply
#undef MY_PREFIX
#define MY_PREFIX Floating_Multiply_FRC31_Rc1
#line 3847 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply\n"));
#line 80459 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00f83e) != 0xfc000032)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3847 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 80493 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Multiply_FRC31_Rc1;

    /* semantic routine */
#line 3847 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply\n"));
#line 80503 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Multiply, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3847 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = *(double*)frA * *(double*)frC;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRC_BITMASK, Rc);
    }
#line 80535 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16./,21.FRC,26.25,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRC
#line 80541 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC31_Rc0(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC31_Rc0
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 80560 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 80599 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC31_Rc0;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 80609 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80662 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80669 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplySubtract_FRC31_Rc1(cpu *processor,
                                           instruction_word instruction,
                                           unsigned_word cia,
                                           idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplySubtract_FRC31_Rc1
#line 4059 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Subtract\n"));
#line 80688 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc000038)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4059 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 80727 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplySubtract_FRC31_Rc1;

    /* semantic routine */
#line 4059 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Subtract\n"));
#line 80737 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4059 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product - *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80790 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.28,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80797 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC31_Rc0(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC31_Rc0
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 80816 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 80855 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC31_Rc0;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 80865 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 80918 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 80925 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_MultiplyAdd_FRC31_Rc1(cpu *processor,
                                      instruction_word instruction,
                                      unsigned_word cia,
                                      idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_MultiplyAdd_FRC31_Rc1
#line 3965 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Multiply-Add\n"));
#line 80944 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003a)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 3965 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 80983 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_MultiplyAdd_FRC31_Rc1;

    /* semantic routine */
#line 3965 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Multiply-Add\n"));
#line 80993 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 3965 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = product + *(double*)frB;
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81046 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.29,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81053 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC31_Rc0(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC31_Rc0
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 81072 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 81111 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC31_Rc0;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 81121 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81174 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81181 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplySubtract_FRC31_Rc1(cpu *processor,
                                                    instruction_word instruction,
                                                    unsigned_word cia,
                                                    idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplySubtract
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplySubtract_FRC31_Rc1
#line 4247 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Subtract\n"));
#line 81200 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003c)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4247 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 81239 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplySubtract_FRC31_Rc1;

    /* semantic routine */
#line 4247 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Subtract\n"));
#line 81249 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplySubtract, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4247 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the subtract */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product - *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81302 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.30,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81309 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC31_Rc0(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC31_Rc0
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 81328 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 0; /* default in icache - constant field */
#line 81367 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC31_Rc0;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 81377 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81430 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81437 "icache.c"

#undef insn
  }
}

EXTERN_ICACHE\
(unsigned_word)
icache_Floating_Negative_MultiplyAdd_FRC31_Rc1(cpu *processor,
                                               instruction_word instruction,
                                               unsigned_word cia,
                                               idecode_cache *cache_entry)
{
#undef MY_INDEX
#define MY_INDEX itable_Floating_Negative_MultiplyAdd
#undef MY_PREFIX
#define MY_PREFIX Floating_Negative_MultiplyAdd_FRC31_Rc1
#line 4153 "../../sim/ppc/powerpc.igen"
  ITRACE(trace_idecode, ("idecode Floating Negative Multiply-Add\n"));
#line 81456 "icache.c"

  /* validate: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc */
  if (WITH_RESERVED_BITS && (instruction & 0xfc00003e) != 0xfc00003e)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP hardware exists */
  if (CURRENT_FLOATING_POINT != HARD_FLOATING_POINT)
    return icache_illegal(processor, instruction, cia, cache_entry);

  /* Validate: FP available according to MSR[FP] */
  if (!IS_FP_AVAILABLE(processor))
    return icache_floating_point_unavailable(processor, instruction, cia, cache_entry);

  {
    unsigned_word nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #define get-values-from-icache|put-values-in-icache */
#define FRT EXTRACTED32(instruction, 6, 10) /* icache scratch - extraction */
#line 62 "../../sim/ppc/powerpc.igen"
    uint64_t * const frT ATTRIBUTE_UNUSED = cache_entry->crack.A.frT = (cpu_registers(processor)->fpr + FRT); /* in icache - extraction */
#line 63 "../../sim/ppc/powerpc.igen"
    uint32_t const FRT_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRT_BITMASK = (1 << FRT); /* in icache - extraction */
#define FRA EXTRACTED32(instruction, 11, 15) /* icache scratch - extraction */
#line 50 "../../sim/ppc/powerpc.igen"
    uint64_t * const frA ATTRIBUTE_UNUSED = cache_entry->crack.A.frA = (cpu_registers(processor)->fpr + FRA); /* in icache - extraction */
#line 51 "../../sim/ppc/powerpc.igen"
    uint32_t const FRA_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRA_BITMASK = (1 << FRA); /* in icache - extraction */
#define FRB EXTRACTED32(instruction, 16, 20) /* icache scratch - extraction */
#line 53 "../../sim/ppc/powerpc.igen"
    uint64_t * const frB ATTRIBUTE_UNUSED = cache_entry->crack.A.frB = (cpu_registers(processor)->fpr + FRB); /* in icache - extraction */
#line 54 "../../sim/ppc/powerpc.igen"
    uint32_t const FRB_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRB_BITMASK = (1 << FRB); /* in icache - extraction */
#define FRC 31 /* icache scratch - constant field */
#line 56 "../../sim/ppc/powerpc.igen"
    uint64_t * const frC ATTRIBUTE_UNUSED = cache_entry->crack.A.frC = (cpu_registers(processor)->fpr + FRC); /* in icache - extraction */
#line 57 "../../sim/ppc/powerpc.igen"
    uint32_t const FRC_BITMASK ATTRIBUTE_UNUSED = cache_entry->crack.A.FRC_BITMASK = (1 << FRC); /* in icache - extraction */
#line 4153 "../../sim/ppc/powerpc.igen"
    unsigned const Rc ATTRIBUTE_UNUSED = 1; /* default in icache - constant field */
#line 81495 "icache.c"

#define insn cache_entry->crack.A.insn = instruction

    cache_entry->address = cia;
    cache_entry->semantic = semantic_Floating_Negative_MultiplyAdd_FRC31_Rc1;

    /* semantic routine */
#line 4153 "../../sim/ppc/powerpc.igen"
    ITRACE(trace_semantics, ("semantics Floating Negative Multiply-Add\n"));
#line 81505 "icache.c"

    /* monitoring: */
    if (WITH_MON & MONITOR_INSTRUCTION_ISSUE) {
      mon_issue(itable_Floating_Negative_MultiplyAdd, processor, cia);
    }

    /* semantics: */
    nia = cia + 4;
#line 4153 "../../sim/ppc/powerpc.igen"
    {
      FPSCR_BEGIN;
      double product; /*HACK! - incorrectly loosing precision ... */
      /* compute the multiply */
      if (is_invalid_operation(processor, cia,
                               *frA, *frC,
                               fpscr_vxsnan | fpscr_vximz,
                               0, /*single?*/
                               0) /*negate?*/) {
        union { double d; uint64_t u; } tmp;
        invalid_arithemetic_operation(processor, cia,
                                      &tmp.u, *frA, 0, *frC,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
        product = tmp.d;
      }
      else {
        /*HACK!*/
        product = *(double*)frA * *(double*)frC;
      }
      /* compute the add */
      if (is_invalid_operation(processor, cia,
                               product, *frB,
                               fpscr_vxsnan | fpscr_vxisi,
                               0, /*single?*/
                               0) /*negate?*/) {
        invalid_arithemetic_operation(processor, cia,
                                      frT, product, *frB, 0,
                                      0, /*instruction_is_frsp*/
                                      0, /*instruction_is_convert_to_64bit*/
                                      0, /*instruction_is_convert_to_32bit*/
                                      0); /*single-precision*/
      }
      else {
        /*HACK!*/
        double s = -(product + *(double*)frB);
        *(double*)frT = s;
      }
      FPSCR_END(Rc);
      PPC_INSN_FLOAT(FRT_BITMASK, FRA_BITMASK | FRB_BITMASK | FRC_BITMASK, Rc);
    }
#line 81558 "icache.c"
    return nia;
    /* extraction: 0.63,6.FRT,11.FRA,16.FRB,21.FRC,26.31,31.Rc #undef get-values-from-icache|put-values-in-icache */
#undef FRT
#undef FRA
#undef FRB
#undef FRC
#line 81565 "icache.c"

#undef insn
  }
}

#endif /* _ICACHE_C_*/
